/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.
 *
 * Copyright (c) 2010-2011, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/** \file     ContextModel.cpp
    \brief    context model class
*/

#include <algorithm>

#include "ContextModel.h"

using namespace std;

// ====================================================================================================================
// Public member functions
// ====================================================================================================================

#if HHI_SYS_PIPE

#define UNUSED_ST  0
#define ST_ID0(x)  (PIPE_ID_EP  << 4) + ( (x) << 7 ) + 0
#define ST_ID1(x)  (PIPE_ID_EP  << 4) + ( (x) << 7 ) + 1
#define ST_ID2(x)  (PIPE_ID_EP  << 4) + ( (x) << 7 ) + 2

#define ST_ID3(x)  (PIPE_ID_BP3 << 4) + ( (x) << 7 ) + 0
#define ST_ID4(x)  (PIPE_ID_BP3 << 4) + ( (x) << 7 ) + 1
#define ST_ID5(x)  (PIPE_ID_BP3 << 4) + ( (x) << 7 ) + 2
#define ST_ID6(x)  (PIPE_ID_BP3 << 4) + ( (x) << 7 ) + 3
#define ST_ID7(x)  (PIPE_ID_BP3 << 4) + ( (x) << 7 ) + 4
#define ST_ID8(x)  (PIPE_ID_BP3 << 4) + ( (x) << 7 ) + 5
#define ST_ID9(x)  (PIPE_ID_BP3 << 4) + ( (x) << 7 ) + 6

#define ST_ID10(x) (PIPE_ID_BP2 << 4) + ( (x) << 7 ) + 0
#define ST_ID11(x) (PIPE_ID_BP2 << 4) + ( (x) << 7 ) + 1
#define ST_ID12(x) (PIPE_ID_BP2 << 4) + ( (x) << 7 ) + 2
#define ST_ID13(x) (PIPE_ID_BP2 << 4) + ( (x) << 7 ) + 3
#define ST_ID14(x) (PIPE_ID_BP2 << 4) + ( (x) << 7 ) + 4

#define ST_ID15(x) (PIPE_ID_TB  << 4) + ( (x) << 7 ) + 0
#define ST_ID16(x) (PIPE_ID_TB  << 4) + ( (x) << 7 ) + 1
#define ST_ID17(x) (PIPE_ID_TB  << 4) + ( (x) << 7 ) + 2
#define ST_ID18(x) (PIPE_ID_TB  << 4) + ( (x) << 7 ) + 3
#define ST_ID19(x) (PIPE_ID_TB  << 4) + ( (x) << 7 ) + 4
#define ST_ID20(x) (PIPE_ID_TB  << 4) + ( (x) << 7 ) + 5
#define ST_ID21(x) (PIPE_ID_TB  << 4) + ( (x) << 7 ) + 6

#define ST_ID22(x) (PIPE_ID_UR2 << 4) + ( (x) << 7 ) + 0
#define ST_ID23(x) (PIPE_ID_UR2 << 4) + ( (x) << 7 ) + 1
#define ST_ID24(x) (PIPE_ID_UR2 << 4) + ( (x) << 7 ) + 2
#define ST_ID25(x) (PIPE_ID_UR2 << 4) + ( (x) << 7 ) + 3
#define ST_ID26(x) (PIPE_ID_UR2 << 4) + ( (x) << 7 ) + 4
#define ST_ID27(x) (PIPE_ID_UR2 << 4) + ( (x) << 7 ) + 5
#define ST_ID28(x) (PIPE_ID_UR2 << 4) + ( (x) << 7 ) + 6
#define ST_ID29(x) (PIPE_ID_UR2 << 4) + ( (x) << 7 ) + 7
#define ST_ID30(x) (PIPE_ID_UR2 << 4) + ( (x) << 7 ) + 8
#define ST_ID31(x) (PIPE_ID_UR2 << 4) + ( (x) << 7 ) + 9

#define ST_ID32(x) (PIPE_ID_UR3 << 4) + ( (x) << 7 ) + 0
#define ST_ID33(x) (PIPE_ID_UR3 << 4) + ( (x) << 7 ) + 1
#define ST_ID34(x) (PIPE_ID_UR3 << 4) + ( (x) << 7 ) + 2
#define ST_ID35(x) (PIPE_ID_UR3 << 4) + ( (x) << 7 ) + 3
#define ST_ID36(x) (PIPE_ID_UR3 << 4) + ( (x) << 7 ) + 4
#define ST_ID37(x) (PIPE_ID_UR3 << 4) + ( (x) << 7 ) + 5
#define ST_ID38(x) (PIPE_ID_UR3 << 4) + ( (x) << 7 ) + 6
#define ST_ID39(x) (PIPE_ID_UR3 << 4) + ( (x) << 7 ) + 7
#define ST_ID40(x) (PIPE_ID_UR3 << 4) + ( (x) << 7 ) + 8
#define ST_ID41(x) (PIPE_ID_UR3 << 4) + ( (x) << 7 ) + 9
#define ST_ID42(x) (PIPE_ID_UR3 << 4) + ( (x) << 7 ) + 10
#define ST_ID43(x) (PIPE_ID_UR3 << 4) + ( (x) << 7 ) + 11
#define ST_ID44(x) (PIPE_ID_UR3 << 4) + ( (x) << 7 ) + 12
#define ST_ID45(x) (PIPE_ID_UR3 << 4) + ( (x) << 7 ) + 13

#define ST_ID46(x) (PIPE_ID_UR4 << 4) + ( (x) << 7 ) + 0
#define ST_ID47(x) (PIPE_ID_UR4 << 4) + ( (x) << 7 ) + 1
#define ST_ID48(x) (PIPE_ID_UR4 << 4) + ( (x) << 7 ) + 2
#define ST_ID49(x) (PIPE_ID_UR4 << 4) + ( (x) << 7 ) + 3
#define ST_ID50(x) (PIPE_ID_UR4 << 4) + ( (x) << 7 ) + 4
#define ST_ID51(x) (PIPE_ID_UR4 << 4) + ( (x) << 7 ) + 5
#define ST_ID52(x) (PIPE_ID_UR4 << 4) + ( (x) << 7 ) + 6
#define ST_ID53(x) (PIPE_ID_UR4 << 4) + ( (x) << 7 ) + 7
#define ST_ID54(x) (PIPE_ID_UR4 << 4) + ( (x) << 7 ) + 8
#define ST_ID55(x) (PIPE_ID_UR4 << 4) + ( (x) << 7 ) + 9
#define ST_ID56(x) (PIPE_ID_UR4 << 4) + ( (x) << 7 ) + 10
#define ST_ID57(x) (PIPE_ID_UR4 << 4) + ( (x) << 7 ) + 11
#define ST_ID58(x) (PIPE_ID_UR4 << 4) + ( (x) << 7 ) + 12
#define ST_ID59(x) (PIPE_ID_UR4 << 4) + ( (x) << 7 ) + 13
#define ST_ID60(x) (PIPE_ID_UR4 << 4) + ( (x) << 7 ) + 14
#define ST_ID61(x) (PIPE_ID_UR4 << 4) + ( (x) << 7 ) + 15

#define ST_ID62(x) (PIPE_ID_UR5 << 4) + ( (x) << 7 ) + 0

#endif

#if HHI_SYS_PIPE_8BIT_INIT
Void ContextModel::init( Int   iQp, UChar ucCtxInit )
{
  Int iN = ucCtxInit & 15; //id at qp 26
  UChar ucM = ucCtxInit >> 4; //slope index
  Int iIntercept = ( iN << 8 ) - 26 * sm_aiPIPESlopes[ ucM ];
  Int iValue = sm_aiPIPESlopes[ ucM ] * iQp + iIntercept + 128;
  UInt uiMps = 0;
  iValue = max( 0, min( 3839,iValue ) );
  Int iPipeId = iValue >> 8;
  iPipeId -= 7;
  if( iPipeId < 0 )
  {
    iPipeId = -iPipeId;
  }
  else
  {
    uiMps = 128;
  }
  iPipeId = sm_aucIdx2PIPEID[iPipeId];
  Int iClip = (iValue & 255);
  if( iPipeId != 3 )
  {
    if( uiMps == 0 )
    {
        iClip = 255 - iClip;
    }
    iClip = iClip * sm_aucNumOffsetOfPipeId[ iPipeId ] >> 8;
  }
  else
  {
    if( iClip > 127 )
    {
      iClip -= 128;
    }
    else
    {
      iClip = 127 - iClip;
    }
    iClip = iClip * 2 * sm_aucNumOffsetOfPipeId[ iPipeId ] >> 8;
  }
  m_ucState = ( iPipeId<<4 ) + uiMps + iClip;
}


const UChar
ContextModel::sm_aucNumOffsetOfPipeId[ 8 ] =
{
  7, 5, 7, 3, 10, 14, 16, 1,
};


#if HHI_SYS_PIPE_LC_MODE
Void ContextModel::init( Int   iQp,
                         UChar ucCtxInit,
                         UInt  (*decodePipe1Bin[])( const UChar             ),
                         UInt  (*decodePipeNBin[])( const UChar, const UInt ),
                         UInt   *puiCwCounter )
{

  Int iN = ucCtxInit & 15; //id at qp 26
  UChar ucM = ucCtxInit >> 4; //slope index
  Int iIntercept = ( iN << 8 ) - 26 * sm_aiPIPESlopes[ ucM ];
  Int iValue = sm_aiPIPESlopes[ ucM ] * iQp + iIntercept + 128;
  UInt uiMps = 0;
  iValue = max( 0, min( 3839,iValue ) );
  Int iPipeId = iValue >> 8;
  iPipeId -= 7;
  if( iPipeId < 0 )
  {
    iPipeId = -iPipeId;
  }
  else
  {
    uiMps = 128;
  }
  iPipeId = sm_aucIdx2PIPEID[iPipeId];
  Int iClip = (iValue & 255);
  if( iPipeId != 3 )
  {
    if( uiMps == 0 )
    {
        iClip = 255 - iClip;
    }
    iClip = iClip * sm_aucNumOffsetOfPipeId[ iPipeId ] >> 8;
  }
  else
  {
    if( iClip > 127 )
    {
      iClip -= 128;
    }
    else
    {
      iClip = 127 - iClip;
    }
    iClip = iClip * 2 * sm_aucNumOffsetOfPipeId[ iPipeId ] >> 8;
  }
  m_ucState = ( iPipeId<<4 ) + uiMps + iClip;
  m_puiCwCounter      = puiCwCounter + iPipeId;
  decode1Bin          = decodePipe1Bin[ iPipeId ];
  decodeNBin          = decodePipeNBin[ iPipeId ];
}
#endif



const Int
ContextModel::sm_aiPIPESlopes[ 16 ] =
{
  -239,  -143,  -85,  -51,
   -31,   -19,  -11,    0,
    11,    19,   31,   51,
    85,   143,  239,  399
};

const UChar
ContextModel::sm_aucIdx2PIPEID[ 8 ]=
{
  PIPE_ID_EP, PIPE_ID_BP3, PIPE_ID_BP2, PIPE_ID_TB, PIPE_ID_UR2, PIPE_ID_UR3, PIPE_ID_UR4, PIPE_ID_UR5
};


#else

/**
 - initialize context model with respect to QP and initial probability
 .
 \param  iQp         input QP value
 \param  asCtxInit   initial probability table
 */
Void
ContextModel::init( Int iQp, Short asCtxInit[] )
{
  Int iInitState  = ( ( asCtxInit[ 0 ] * iQp ) >> 4 ) + asCtxInit[ 1 ];
  iInitState      = min( max( 1, iInitState ), 126 );
#if HHI_SYS_PIPE
  m_ucState = m_aucStateToStateId[iInitState];

#else
  if( iInitState >= 64 )
  {
    m_ucState     = min( 62, iInitState - 64 );
    m_ucState    += m_ucState + 1;
  }
  else
  {
    m_ucState     = min( 62, 63 - iInitState );
    m_ucState    += m_ucState;
  }
#endif
}

#if HHI_SYS_PIPE
const UChar
ContextModel::m_aucStateToStateId[127] =
{
  0,          ST_ID62(0), ST_ID61(0), ST_ID60(0), ST_ID59(0), ST_ID58(0), ST_ID57(0), ST_ID56(0),
  ST_ID55(0), ST_ID54(0), ST_ID53(0), ST_ID52(0), ST_ID51(0), ST_ID50(0), ST_ID49(0), ST_ID48(0),
  ST_ID47(0), ST_ID46(0), ST_ID45(0), ST_ID44(0), ST_ID43(0), ST_ID42(0), ST_ID41(0), ST_ID40(0),
  ST_ID39(0), ST_ID38(0), ST_ID37(0), ST_ID36(0), ST_ID35(0), ST_ID34(0), ST_ID33(0), ST_ID32(0),
  ST_ID31(0), ST_ID30(0), ST_ID29(0), ST_ID28(0), ST_ID27(0), ST_ID26(0), ST_ID25(0), ST_ID24(0),
  ST_ID23(0), ST_ID22(0), ST_ID21(0), ST_ID20(0), ST_ID19(0), ST_ID18(0), ST_ID17(0), ST_ID16(0),
  ST_ID15(0), ST_ID14(0), ST_ID13(0), ST_ID12(0), ST_ID11(0), ST_ID10(0), ST_ID9(0),  ST_ID8(0),
  ST_ID7(0),  ST_ID6(0),  ST_ID5(0),  ST_ID4(0),  ST_ID3(0),  ST_ID2(0),  ST_ID1(0),  ST_ID0(0),

  ST_ID0(1),  ST_ID1(1),  ST_ID2(1),  ST_ID3(1),  ST_ID4(1),  ST_ID5(1),  ST_ID6(1),  ST_ID7(1),
  ST_ID8(1),  ST_ID9(1),  ST_ID10(1), ST_ID11(1), ST_ID12(1), ST_ID13(1), ST_ID14(1), ST_ID15(1),
  ST_ID16(1), ST_ID17(1), ST_ID18(1), ST_ID19(1), ST_ID20(1), ST_ID21(1), ST_ID22(1), ST_ID23(1),
  ST_ID24(1), ST_ID25(1), ST_ID26(1), ST_ID27(1), ST_ID28(1), ST_ID29(1), ST_ID30(1), ST_ID31(1),
  ST_ID32(1), ST_ID33(1), ST_ID34(1), ST_ID35(1), ST_ID36(1), ST_ID37(1), ST_ID38(1), ST_ID39(1),
  ST_ID40(1), ST_ID41(1), ST_ID42(1), ST_ID43(1), ST_ID44(1), ST_ID45(1), ST_ID46(1), ST_ID47(1),
  ST_ID48(1), ST_ID49(1), ST_ID50(1), ST_ID51(1), ST_ID52(1), ST_ID53(1), ST_ID54(1), ST_ID55(1),
  ST_ID56(1), ST_ID57(1), ST_ID58(1), ST_ID59(1), ST_ID60(1), ST_ID61(1), ST_ID62(1)
};
#endif

#endif

#if !HHI_SYS_PIPE_LC_MODE
#if HHI_SYS_PIPE

#define STATE_IDS_EP(x) \
{ ST_ID1(x),  ST_ID0(1-x)  }, { ST_ID2(x),  ST_ID0(x)  }, { ST_ID3(x),  ST_ID1(x)  }, { UNUSED_ST,  UNUSED_ST  }, \
{ UNUSED_ST,  UNUSED_ST    }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, \
{ UNUSED_ST,  UNUSED_ST    }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, \
{ UNUSED_ST,  UNUSED_ST    }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  },


#define STATE_IDS_BP3(x) \
{ ST_ID4(x),  ST_ID2(x)  }, { ST_ID5(x),  ST_ID2(x)  }, { ST_ID6(x),  ST_ID4(x)  }, { ST_ID7(x),  ST_ID4(x)  }, \
{ ST_ID8(x),  ST_ID5(x)  }, { ST_ID9(x),  ST_ID6(x)  }, { ST_ID10(x), ST_ID7(x)  }, { UNUSED_ST,  UNUSED_ST  }, \
{ UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, \
{ UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  },


#define STATE_IDS_BP2(x) \
{ ST_ID11(x), ST_ID8(x)  }, { ST_ID12(x), ST_ID9(x)  }, { ST_ID13(x), ST_ID9(x)  }, { ST_ID14(x), ST_ID11(x) }, \
{ ST_ID15(x), ST_ID11(x) }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, \
{ UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, \
{ UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  },


#define STATE_IDS_TB(x) \
{ ST_ID16(x), ST_ID12(x) }, { ST_ID17(x), ST_ID13(x) }, { ST_ID18(x), ST_ID13(x) }, { ST_ID19(x), ST_ID15(x) }, \
{ ST_ID20(x), ST_ID15(x) }, { ST_ID21(x), ST_ID16(x) }, { ST_ID22(x), ST_ID16(x) }, { UNUSED_ST,  UNUSED_ST  }, \
{ UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, \
{ UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  },


#define STATE_IDS_UR2(x) \
{ ST_ID23(x), ST_ID18(x) }, { ST_ID24(x), ST_ID18(x) }, { ST_ID25(x), ST_ID19(x) }, { ST_ID26(x), ST_ID19(x) }, \
{ ST_ID27(x), ST_ID21(x) }, { ST_ID28(x), ST_ID21(x) }, { ST_ID29(x), ST_ID22(x) }, { ST_ID30(x), ST_ID22(x) }, \
{ ST_ID31(x), ST_ID23(x) }, { ST_ID32(x), ST_ID24(x) }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, \
{ UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  },


#define STATE_IDS_UR3(x) \
{ ST_ID33(x), ST_ID24(x) }, { ST_ID34(x), ST_ID25(x) }, { ST_ID35(x), ST_ID26(x) }, { ST_ID36(x), ST_ID26(x) }, \
{ ST_ID37(x), ST_ID27(x) }, { ST_ID38(x), ST_ID27(x) }, { ST_ID39(x), ST_ID28(x) }, { ST_ID40(x), ST_ID29(x) }, \
{ ST_ID41(x), ST_ID29(x) }, { ST_ID42(x), ST_ID30(x) }, { ST_ID43(x), ST_ID30(x) }, { ST_ID44(x), ST_ID30(x) }, \
{ ST_ID45(x), ST_ID31(x) }, { ST_ID46(x), ST_ID32(x) }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  },


#define STATE_IDS_UR4(x) \
{ ST_ID47(x), ST_ID32(x) }, { ST_ID48(x), ST_ID33(x) }, { ST_ID49(x), ST_ID33(x) }, { ST_ID50(x), ST_ID33(x) }, \
{ ST_ID51(x), ST_ID34(x) }, { ST_ID52(x), ST_ID34(x) }, { ST_ID53(x), ST_ID35(x) }, { ST_ID54(x), ST_ID35(x) }, \
{ ST_ID55(x), ST_ID35(x) }, { ST_ID56(x), ST_ID36(x) }, { ST_ID57(x), ST_ID36(x) }, { ST_ID58(x), ST_ID36(x) }, \
{ ST_ID59(x), ST_ID37(x) }, { ST_ID60(x), ST_ID37(x) }, { ST_ID61(x), ST_ID37(x) }, { ST_ID62(x), ST_ID38(x) },


#define STATE_IDS_UR5(x) \
{ ST_ID62(x), ST_ID38(x) }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, \
{ UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, \
{ UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, \
{ UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  }, { UNUSED_ST,  UNUSED_ST  },


const UChar
ContextModel::m_aucNextStateMpsLps[256][2] =
{
  STATE_IDS_BP3(0)
  STATE_IDS_BP2(0)
  STATE_IDS_TB (0)
  STATE_IDS_EP (0)
  STATE_IDS_UR2(0)
  STATE_IDS_UR3(0)
  STATE_IDS_UR4(0)
  STATE_IDS_UR5(0)

  STATE_IDS_BP3(1)
  STATE_IDS_BP2(1)
  STATE_IDS_TB (1)
  STATE_IDS_EP (1)
  STATE_IDS_UR2(1)
  STATE_IDS_UR3(1)
  STATE_IDS_UR4(1)
  STATE_IDS_UR5(1)
};

#else
const UChar
ContextModel::m_aucNextStateMPS[ 64 ] =
{
  1, 2, 3, 4, 5, 6, 7, 8,
  9, 10,11,12,13,14,15,16,
  17,18,19,20,21,22,23,24,
  25,26,27,28,29,30,31,32,
  33,34,35,36,37,38,39,40,
  41,42,43,44,45,46,47,48,
  49,50,51,52,53,54,55,56,
  57,58,59,60,61,62,62,63
};

const UChar
ContextModel::m_aucNextStateLPS[ 64 ] =
{
  0, 0, 1, 2, 2, 4, 4, 5,
  6, 7, 8, 9, 9, 11,11,12,
  13,13,15,15,16,16,18,18,
  19,19,21,21,22,22,23,24,
  24,25,26,26,27,27,28,29,
  29,30,30,30,31,32,32,33,
  33,33,34,34,35,35,35,36,
  36,36,37,37,37,38,38,63
};
#endif
#endif

#if HHI_SYS_PIPE
const Int ContextModel::m_entropyBits[ 16 ] =
{
  21762,  15987,  11450,  32768,
  7980,   4013,   2016,   1022,
  47328,  59701,  73517,  32768,
  88878, 119473, 151039, 182650,
};
#else
const Int ContextModel::m_entropyBits[128] =
{
  0x08000, 0x08000, 0x076da, 0x089a0, 0x06e92, 0x09340, 0x0670a, 0x09cdf, 0x06029, 0x0a67f, 0x059dd, 0x0b01f, 0x05413, 0x0b9bf, 0x04ebf, 0x0c35f,
  0x049d3, 0x0ccff, 0x04546, 0x0d69e, 0x0410d, 0x0e03e, 0x03d22, 0x0e9de, 0x0397d, 0x0f37e, 0x03619, 0x0fd1e, 0x032ee, 0x106be, 0x02ffa, 0x1105d,
  0x02d37, 0x119fd, 0x02aa2, 0x1239d, 0x02836, 0x12d3d, 0x025f2, 0x136dd, 0x023d1, 0x1407c, 0x021d2, 0x14a1c, 0x01ff2, 0x153bc, 0x01e2f, 0x15d5c,
  0x01c87, 0x166fc, 0x01af7, 0x1709b, 0x0197f, 0x17a3b, 0x0181d, 0x183db, 0x016d0, 0x18d7b, 0x01595, 0x1971b, 0x0146c, 0x1a0bb, 0x01354, 0x1aa5a,
  0x0124c, 0x1b3fa, 0x01153, 0x1bd9a, 0x01067, 0x1c73a, 0x00f89, 0x1d0da, 0x00eb7, 0x1da79, 0x00df0, 0x1e419, 0x00d34, 0x1edb9, 0x00c82, 0x1f759,
  0x00bda, 0x200f9, 0x00b3c, 0x20a99, 0x00aa5, 0x21438, 0x00a17, 0x21dd8, 0x00990, 0x22778, 0x00911, 0x23118, 0x00898, 0x23ab8, 0x00826, 0x24458,
  0x007ba, 0x24df7, 0x00753, 0x25797, 0x006f2, 0x26137, 0x00696, 0x26ad7, 0x0063f, 0x27477, 0x005ed, 0x27e17, 0x0059f, 0x287b6, 0x00554, 0x29156,
  0x0050e, 0x29af6, 0x004cc, 0x2a497, 0x0048d, 0x2ae35, 0x00451, 0x2b7d6, 0x00418, 0x2c176, 0x003e2, 0x2cb15, 0x003af, 0x2d4b5, 0x0037f, 0x2de55
};
#endif

