/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.  
 *
 * Copyright (c) 2010-2011, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */


/** \file     ContextModel.h
    \brief    context model class (header)
*/

#ifndef __CONTEXT_MODEL__
#define __CONTEXT_MODEL__

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "CommonDef.h"


// ====================================================================================================================
// Class definition
// ====================================================================================================================

/// context model class
class ContextModel
{
public:
  ContextModel  ()                        { m_ucState = 0;             }
  ~ContextModel ()                        {}
  
#if HHI_SYS_PIPE
  const UChar getState  ()   const        { return m_ucState; }
  const UChar getMps    ()                { return m_ucState >> 7; }
  const UChar getMps    ()   const        { return m_ucState >> 7; }

  const UChar getPipeId ()   const        { return ( m_ucState >> 4 ) & 7; }
  const UChar getPipeIdTwoSided() const   { return m_ucState >> 4; }

  UInt (*decode1Bin) ( const UChar             );
  UInt (*decodeNBin) ( const UChar, const UInt );
  UInt *m_puiCwCounter;

#else
  const UChar getState  ()                { return ( m_ucState >> 1 ); }                    ///< get current state
  const UChar getMps    ()                { return ( m_ucState  & 1 ); }                    ///< get curret MPS
#endif

#if HHI_SYS_PIPE_8BIT_INIT
  Void        init      ( Int   iQp,
                          UChar ucCtxInit );
#if HHI_SYS_PIPE
  Void        init      ( Int   iQp,
                          UChar ucCtxInit,
                          UInt  (*decodePipe1Bin[])( const UChar             ),
                          UInt  (*decodePipeNBin[])( const UChar, const UInt ),
                          UInt   *puiCwCounter );
#endif
#else
  Void        init      ( Int   iQp, 
                         Short asCtxInit[] );                                              ///< initialize state with initial prob.
#endif

#if !HHI_SYS_PIPE_LC_MODE
#if HHI_SYS_PIPE
  Void updateState( UInt uiIsLps ) { m_ucState = ContextModel::m_aucNextStateMpsLps[ m_ucState ][ uiIsLps ]; }
#else
  Void        updateLPS ()
  {
    UChar ucMPS = ( m_ucState > 1    ? m_ucState  & 1 :    1   - ( m_ucState & 1 ) );
    m_ucState   = ( m_aucNextStateLPS[ m_ucState >> 1 ] << 1 ) + ucMPS;
  }
  
  Void        updateMPS ()
  {
    m_ucState   = ( m_aucNextStateMPS[ m_ucState >> 1 ] << 1 ) + ( m_ucState & 1 );
  }
#endif
#endif

  
#if HHI_SYS_PIPE
  Int getEntropyBits(Short val) { return m_entropyBits[ getPipeIdTwoSided() ^ ((val != 0) << 3) ]; }
#else
  Int getEntropyBits(Short val) { return m_entropyBits[m_ucState ^ val]; }
#endif
  
private:
  UChar         m_ucState;                                                                  ///< internal state variable
#if HHI_SYS_PIPE
#if HHI_SYS_PIPE_8BIT_INIT
  static const Int sm_aiPIPESlopes[16];
  static const UChar sm_aucIdx2PIPEID[8];
  static const UChar sm_aucNumOffsetOfPipeId[8];
#else
  static const UChar m_aucStateToStateId[127];
#endif

#endif

#if !HHI_SYS_PIPE_LC_MODE
#if HHI_SYS_PIPE
  static const UChar m_aucNextStateMpsLps[256][2];
#else
  static const  UChar m_aucNextStateMPS[ 64 ];
  static const  UChar m_aucNextStateLPS[ 64 ];
#endif
#endif
#if HHI_SYS_PIPE
  static const Int m_entropyBits[ 16 ];
#else
  static const Int m_entropyBits[ 128 ];
#endif


};

#endif

