/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.
 *
 * Copyright (c) 2010-2011, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/** \file     ContextTables.h
    \brief    Defines constants and tables for SBAC
    \todo     number of context models is not matched to actual use, should be fixed
*/

#ifndef __CONTEXTTABLES__
#define __CONTEXTTABLES__

// ====================================================================================================================
// Constants
// ====================================================================================================================

#define NUM_SPLIT_FLAG_CTX            3       ///< number of context models for split flag
#define NUM_SKIP_FLAG_CTX             3       ///< number of context models for skip flag

#define NUM_MERGE_FLAG_EXT_CTX        3       ///< number of context models for merge flag of merge extended
#define NUM_MERGE_IDX_EXT_CTX         4       ///< number of context models for merge index of merge extended

#define NUM_ALF_CTRL_FLAG_CTX         3       ///< number of context models for ALF control flag
#define NUM_PART_SIZE_CTX             5       ///< number of context models for partition size
#define NUM_PRED_MODE_CTX             2       ///< number of context models for prediction mode
#if MTK_DCM_MPM
#define NUM_ADI_CTX                   3       ///< number of context models for intra prediction
#else
#define NUM_ADI_CTX                   2
#endif
#if ADD_PLANAR_MODE
#define NUM_PLANARFLAG_CTX            2       ///< number of context models for planar mode flag (intra prediction)
#endif

#define NUM_CHROMA_PRED_CTX           4       ///< number of context models for intra prediction (chroma)
#define NUM_INTER_DIR_CTX             4       ///< number of context models for inter prediction direction
#define NUM_MV_RES_CTX                7       ///< number of context models for motion vector difference

#define NUM_REF_NO_CTX                6       ///< number of context models for reference index
#define NUM_TRANS_SUBDIV_FLAG_CTX     10      ///< number of context models for transform subdivision flags
#define NUM_QT_CBF_CTX                5       ///< number of context models for QT CBF
#define NUM_QT_ROOT_CBF_CTX           4       ///< number of context models for QT ROOT CBF
#define NUM_DELTA_QP_CTX              4       ///< number of context models for dQP

#if HHI_SYS_PIPE_LC_MODE
#define NUM_SIG_FLAG_CTX              28      ///< number of context models for sig flag
#else
#if SIMPLE_CONTEXT_SIG
#define NUM_SIG_FLAG_CTX              50      ///< number of context models for sig flag
#else
#define NUM_SIG_FLAG_CTX              16      ///< number of context models for sig flag
#endif
#endif
#if PCP_SIGMAP_SIMPLE_LAST
#define NUM_CTX_LAST_FLAG_XY          26      ///< number of context models for PCP last flag
#else
#define NUM_LAST_FLAG_CTX             16      ///< number of context models for last flag
#endif
#if HHI_SYS_PIPE_LC_MODE
#define NUM_ONE_FLAG_CTX              60      ///< number of context models for greater than one
#define NUM_ABS_FLAG_CTX              60      ///< number of context models for magnitude
#else
#define NUM_ONE_FLAG_CTX              30      ///< number of context models for greater than one
#define NUM_ABS_FLAG_CTX              30      ///< number of context models for magnitude
#endif

#define NUM_MVP_IDX_CTX               2       ///< number of context models for MVP index

#define NUM_ALF_FLAG_CTX              1       ///< number of context models for ALF flag
#define NUM_ALF_UVLC_CTX              2       ///< number of context models for ALF UVLC (filter length)
#define NUM_ALF_SVLC_CTX              3       ///< number of context models for ALF SVLC (filter coeff.)

#if MTK_SAO
#define NUM_AO_FLAG_CTX              1       ///< number of context models for AO flag
#define NUM_AO_UVLC_CTX              2       ///< number of context models for AO UVLC (filter length)
#define NUM_AO_SVLC_CTX              3       ///< number of context models for AO SVLC (filter coeff.)
#endif

#if HHI_SYS_PIPE_8BIT_INIT
// these defines can be moved to TComRom

#if HHI_SYS_PIPE_LC_MODE
#define INIT_CONTEXT_MODEL_DEC(v,i)      (v).initBuffer( eSliceType, iQp, (UChar*)(i), \
                                                      decodePipe1Bin, \
                                                      decodePipeNBin, \
                                                      m_auiCounter );
#else
#define INIT_CONTEXT_MODEL_DEC(v,i)     (v).initBuffer( eSliceType, iQp, (UChar*)(i));
#endif
#define INIT_CONTEXT_MODEL_ENC(v,i)     (v).initBuffer( eSliceType, iQp, (UChar*)(i));

#endif
// ====================================================================================================================
// Tables
// ====================================================================================================================
#if HHI_SYS_PIPE_8BIT_INIT

#define CNU 119


#if HHI_UNIFIED_PIPEv2_CTX_INIT

#if HHI_SYS_PIPE_LC_MODE
static const UChar
INIT_ALF_FLAG[3][1] =
{
  {
     CNU,

  },
  {
     CNU,

  },
  {
     CNU,

  },
};


static const UChar
INIT_ALF_SVLC[3][3] =
{
  {
     CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU,

  },
};


static const UChar
INIT_ALF_UVLC[3][2] =
{
  {
     CNU, CNU,

  },
  {
     CNU, CNU,

  },
  {
     CNU, CNU,

  },
};


static const UChar
INIT_AO_FLAG[3][1] =
{
  {
      56,

  },
  {
      56,

  },
  {
      56,

  },
};


static const UChar
INIT_AO_SVLC[3][3] =
{
  {
     183, 103, 196,

  },
  {
     151, 119, 195,

  },
  {
     151, 119, 181,

  },
};


static const UChar
INIT_AO_UVLC[3][2] =
{
  {
      92, 152,

  },
  {
      88, 135,

  },
  {
     120, 151,

  },
};


static const UChar
INIT_ABS_FLAG[3][120] =
{
  {
      71,  38,  38,  39,  71,  54,  54,  38,  38,  71,   5, CNU,  71,  54,  86,  37, 182,   3,   2, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,  56,  55,  55,  54,  54,  70,  37,  36,   1,   2,   1,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
      53,  37,   0,  16,  70,  38,   1,   6, CNU, CNU, CNU, CNU,  70, 165,  69,   1,   1, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
      69,  38,  38,   4,  68,  70,  70,  21,   4, CNU, CNU, CNU,  70, 149,  70,  70, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, 102, 102,   4,  32, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
      84,  69,   0, CNU,  85,   3, CNU, CNU, CNU, CNU, CNU, CNU, 180,  20, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
      54,  54,  54,   3,  85, 102,  54,  38, 200,  16, CNU, CNU, 102, 102,  53,  37,  70,   3,   3,   2, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, 118,  22,  38,  53,  53,  20,  21,   3,   3,   3,   3,   2,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
      53,  36,  38, CNU,  85,  52, CNU, CNU, CNU, CNU, CNU, CNU, 165,   1,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_ALF_CTRL_FLAG[3][3] =
{
  {
     CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU,

  },
};


static const UChar
INIT_CHROMA_PRED_MODE[3][4] =
{
  {
      38, CNU, CNU,  71,

  },
  {
      54, CNU, CNU, 118,

  },
  {
      55, CNU, CNU, 118,

  },
};


static const UChar
INIT_DQP[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_INTER_DIR[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU, CNU,

  },
  {
      84, CNU, CNU, CNU,

  },
};


static const UChar
INIT_INTRA_PRED_MODE[3][3] =
{
  {
     151, 103, 103,

  },
  {
     119, 119, 119,

  },
  {
     119, 119, 119,

  },
};


static const UChar
INIT_MERGE_FLAG_EXT[3][3] =
{
  {
     CNU, CNU, CNU,

  },
  {
     119, CNU, CNU,

  },
  {
     119, CNU, CNU,

  },
};


static const UChar
INIT_MERGE_IDX_EXT[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
     119, 118,  87, 119,

  },
  {
     119,  70, 119, 119,

  },
};


static const UChar
INIT_MVD[3][14] =
{
  {
     CNU, CNU, CNU, CNU, CNU, CNU, CNU,
     CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     151, CNU, CNU, 135, 184, 151, 154,
     119, CNU, CNU, 135, 184, 120, 123,

  },
  {
     151, CNU, CNU, 167, 169, 136, 123,
     119, CNU, CNU, 167, 169, 136, 123,

  },
};


static const UChar
INIT_ONE_FLAG[3][120] =
{
  {
      54,  37,  37,  38,  53,  52,  52,  52,  52,  37,  54,  22,  54,  53,  52,  51,  50,  66,  50,  98,  34,  67,  16,   0, CNU, CNU, CNU, CNU,  56,  54,  39,  38,  38,  37,  20,  51,  50,  50,  50,  50,  50,  34,  34,  66,  66,  34,  33,  16,   0,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
      99,  50,  34,  33, 147, 179,  67,  34,  16,  32, CNU, CNU, 148, 162, 162,  82,  66,  33,  16,  16,  32,  16, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     179,  83,  36,  37, 179, 178,  67,  67,  36,  69,   3, CNU, 179, 113, 113, 129,  66,  66,  67,  67, 212,   4, CNU, CNU, CNU, CNU, CNU, CNU, 101,  98, 113, 113, 176, 176,  65,  65,  65,  65,  33,  34,   0,  16,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
     178,  66,  33,  16, 179, 177,  66,  34, CNU, CNU, CNU, CNU, 179, 177, 210,  64, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     179,  83,  52,  37, 179, 114,  83,  83,  36,  37, 184,   5,  85,  83, 114, 114,  98,  67,  67,  67,  51,  36,   3,   1,   2, CNU, CNU, CNU,  86,  37,  37,  37,  20,  19,  35,  50,  66,  67,  67,  67,  51,  35,  19,   2,   3,   3,   2,   1,   0,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
     179,  66,  66,  34, 180, 178,  98,  66,  16, CNU, CNU, CNU, 179,  65,  64,  32,  32,  32,  16,  32,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_PART_SIZE[3][5] =
{
  {
     152, CNU, CNU, CNU, CNU,

  },
  {
     151, 118, 202, CNU, CNU,

  },
  {
     119,  85, 181,  84,  55,

  },
};


static const UChar
INIT_PRED_MODE[3][2] =
{
  {
     CNU, CNU,

  },
  {
     CNU, 131,

  },
  {
     CNU, CNU,

  },
};


static const UChar
INIT_QT_CBF[3][15] =
{
  {
      57, CNU, CNU, CNU, CNU,
      52, CNU, CNU, CNU, CNU,
      52, CNU, CNU, CNU, CNU,

  },
  {
     119, CNU, CNU, CNU, CNU,
      51, CNU, CNU, CNU, CNU,
      51, CNU, CNU, CNU, CNU,

  },
  {
     119, CNU, CNU, CNU, CNU,
      51, CNU, CNU, CNU, CNU,
      51, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_QT_ROOT_CBF[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
      55, CNU, CNU, CNU,

  },
  {
      55, CNU, CNU, CNU,

  },
};


static const UChar
INIT_REF_PIC[3][6] =
{
  {
     CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     118, CNU, CNU, CNU, 119, 119,

  },
  {
     118, CNU, CNU, CNU, 119, 119,

  },
};


static const UChar
INIT_SIG_FLAG[3][56] =
{
  {
      90,  73,  87,  87,
      73,  88, 103,  87,  71,  71,
      91,  73,  72, 103,  87,  71,  69,  84,
      61,  60,  58,  57,  72,  72,  56,  55,  53,   52,
     186, 152,  54,  54,
     186, 151, 150, 150, 100,  68,
     155, 104, 103,  71,  53, 149,  51,  51,
     CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     183, 119, 102, 118,
     120, 119,  87,  70,  69,  86,
     106,  88, 103, 102, 101,  84,  68,  85,
     107, 120,  88, 119, 103,  86, 100,  84,  67,  84,
     185, 183,  70, 134,
     186, 183, 118, 117, 131, 147,
     186, 183, 119, 134,  68,  51,  50,  67,
     CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     183, 119, 102,  87,
     120, 119, 118,  86,  85,  85,
     106, 104, 119,  87,  86,  85,  84,  84,
     107, 120, 104, 119,  71,  71,  87,  71,  68,  84,
     185, 135, 102, 134,
     186, 135, 118, 133, 131, 131,
     186, 183, 119, 134,  84,  51,  50,  67,
     CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
  },
};

static const UChar
INIT_SKIP_FLAG[3][3] =
{
  {
     CNU, CNU, CNU,

  },
  {
     182, CNU, CNU,

  },
  {
     182, CNU, CNU,

  },
};


static const UChar
INIT_SPLIT_FLAG[3][3] =
{
  {
      72, CNU, CNU,

  },
  {
      55, CNU, CNU,

  },
  {
      55, CNU, CNU,

  },
};


static const UChar
INIT_TRANS_SUBDIV_FLAG[3][10] =
{
  {
     CNU,  84,  69,  99, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     CNU,  53,  54,  71, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     CNU,  53,  54,  71, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_LAST_X[3][52] =
{
  {
     150, 166, 167, 165, 165, 165, 102, 103, 150, 165, 165, 165, 181, 166, 103, 180, 147, 179, 179, 164,  99, 179, 181, 181,  86,  69,
     184, 184, 201, 186, 167, 184, 183, 168, 199, 182, 183, 183, 166, 181, 183, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     166, 182, 167, 150, 181, 182, 181, 166, 134, 181, 182, 181, 180, 181, 118, 103, 180, 180, 181, 181, 179, 178, 193, 180, 149,  35,
     184, 184, 185, 169, 184, 185, 183, 151, 184, 182, 199, 183, 152,  38, 184, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     166, 182, 135, 166, 181, 182, 181, 118, 134, 181, 181, 181, 181, 181, 117, 118, 180, 180, 180, 180, 147, 147, 182, 180, 147,   2,
     184, 184, 185, 169, 184, 184, 183, 184, 152, 182, 199, 183, 135, 147,  70, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_LAST_Y[3][52] =
{
  {
     151, 167, 167, 183, 103, 182, 182, 182, 165, 165, 165, 165, 181, 181,  70, 179, 179, 147, 179, 179, 163, 163, 181,  84,  99, 148,
     185, 201, 202, 170, 184, 184, 183, 184, 199, 198, 182, 182, 166, 165, 166, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     167, 167, 167, 182, 182, 182, 181, 182, 166, 181, 182, 181, 181, 180, 132, 150, 180, 181, 181, 181, 116, 115, 179, 146,  66, 132,
     185, 185, 201, 170, 184, 183, 182, 149, 185, 183, 198, 183,  71,  67,  68, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     182, 183, 135, 182, 182, 182, 196, 181, 150, 181, 181, 181, 181, 180, 163, 134, 180, 180, 180, 180, 147, 131, 181,  67,  50, 132,
     185, 185, 201, 185, 184, 184, 182, 134, 199, 183, 182, 182,  86,  67,  69, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_MVP_IDX[3][2] =
{
  {
     CNU, CNU,

  },
  {
     119, 119,

  },
  {
     119, 118,

  },
};


static const UChar
INIT_PLANARFLAG[3][2] =
{
  {
     104, 170,

  },
  {
     122, 124,

  },
  {
     185, 187,

  },
};






#else


static const UChar
INIT_ALF_FLAG[3][1] =
{
  {
     119,

  },
  {
     102,

  },
  {
     102,

  },
};


static const UChar
INIT_ALF_SVLC[3][3] =
{
  {
     140, 119, 107,

  },
  {
      90, 119, 108,

  },
  {
     137, 119, 108,

  },
};


static const UChar
INIT_ALF_UVLC[3][2] =
{
  {
     104, 119,

  },
  {
     119, 119,

  },
  {
     119, 119,

  },
};


static const UChar
INIT_AO_FLAG[3][1] =
{
  {
      88,

  },
  {
     119,

  },
  {
      56,

  },
};


static const UChar
INIT_AO_SVLC[3][3] =
{
  {
     184, 119, 197,

  },
  {
     120, 119, 150,

  },
  {
     167, 119, 150,

  },
};


static const UChar
INIT_AO_UVLC[3][2] =
{
  {
     108, 104,

  },
  {
     151, 120,

  },
  {
     120, 120,

  },
};


static const UChar
INIT_ABS_FLAG[3][60] =
{
  {
     101, 119,  88, 120, 105, 117, 118, 119, 119,  89, 134, 119, 119,  88, 121,  83,  85,  86, 118,  71, 132, 133, 102, 118,  71, 133, 134,  71, 119, 119,
      85, 119, 120, 152, 105, 133, 134, 119, 167, 137, 118, 119, 119, 167, 137, 114, 115, 148, 117, 166, 163, 164, 149, 165, 150, 164, 133, 134, 150,  87,

  },
  {
     115, 133, 119, 119, 136, 116, 102, 118, 119, 136, 118,  87, 119, 119,  89,  83, 116, 117, 102, 118, 100,  85, 117, 118, 134, 117, 118, 118,  87, 119,
     178, 118, 135, 120, 105, 100,  86, 134, 135, 137, 117,  70,  87, 103, 121, 177,  83, 101, 101,  69,  35,  69,  19,  69,   2, 147, 102,  68, 101,  70,

  },
  {
     131, 102, 119, 119, 136, 101, 102, 119, 119, 136, 118, 119, 119, 119,  89,  99, 116, 117, 118, 118, 116, 117,  86, 118,  87, 133, 118, 118,  87, 119,
     163, 118, 151, 120, 121, 132, 118, 119, 167,  89, 102, 102, 135, 104, 153, 114,  99,  68, 196, 215, 163,  84,  52, 215,  20, 101, 181,  69, 199, 216,

  },
};


static const UChar
INIT_ALF_CTRL_FLAG[3][3] =
{
  {
     153, CNU, CNU,

  },
  {
     119, CNU, CNU,

  },
  {
     135, CNU, CNU,

  },
};


static const UChar
INIT_CHROMA_PRED_MODE[3][4] =
{
  {
     119,  53,  53, 119,

  },
  {
      54,  70,  70, 118,

  },
  {
      70,  86,  70, 118,

  },
};


static const UChar
INIT_DQP[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_INTER_DIR[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU, CNU,

  },
  {
      84, 100,  69, CNU,

  },
};


static const UChar
INIT_INTRA_PRED_MODE[3][3] =
{
  {
     120, 119, 119,

  },
  {
     119, 119, 119,

  },
  {
     119, 119, 119,

  },
};


static const UChar
INIT_MERGE_FLAG_EXT[3][3] =
{
  {
     CNU, CNU, CNU,

  },
  {
      87,  87,  87,

  },
  {
     118,  87,  87,

  },
};


static const UChar
INIT_MERGE_IDX_EXT[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
     119, 118,  87, 119,

  },
  {
     119, 102, 119, 119,

  },
};


static const UChar
INIT_MVD[3][14] =
{
  {
     CNU, CNU, CNU, CNU, CNU, CNU, CNU,
     CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     135, 120,  89, 134, 152, 135, 122,
     119,  88,  88, 166, 168, 135, 122,

  },
  {
     119,  89, 121, 166, 168, 120, 122,
     119, 120, 105, 166, 168, 120, 122,

  },
};


static const UChar
INIT_ONE_FLAG[3][60] =
{
  {
      88, 115, 116, 101, 102, 119, 148,  86, 102, 118, 120, 118, 119, 119, 119,  85, 114,  83,  83, 115, 102, 100, 132, 132, 101, 119, 117, 133, 102, 102,
      88, 163, 164, 165, 118, 103, 116, 165, 149, 134, 152, 134, 150, 134, 134, 116, 177, 146, 114, 114, 149, 115, 164, 148, 148, 118, 132, 148, 133, 101,

  },
  {
     166, 181, 180, 164, 164, 119, 164, 133, 117, 149, 135, 118, 118, 118, 118, 132, 147, 115,  99,  99, 149, 147, 116, 116, 148, 118, 133, 133, 133, 133,
     135, 194, 179, 179, 180, 103, 147, 179, 116, 132,  88, 101, 102, 118, 134, 115, 178, 146, 177, 130,  85, 131,  83, 147, 131,  70, 147, 101, 179,  85,

  },
  {
     166, 181, 180, 164, 149, 119, 164, 149, 133, 165, 104, 118, 119, 118, 118, 148, 163, 115,  99, 115, 149, 116, 132, 148, 148,  87, 133, 133, 149, 102,
     167, 196, 179, 180, 180, 103, 179, 180, 180, 133, 136, 133,  70,  86, 134, 131, 178, 146, 130, 114,  85, 162, 115,  99, 131, 134,  68,  99,  52, 133,

  },
};


static const UChar
INIT_PART_SIZE[3][5] =
{
  {
     167, CNU, CNU, CNU, CNU,

  },
  {
     151, 119, 186, CNU, CNU,

  },
  {
     119, 118, 168,  70, 102,

  },
};


static const UChar
INIT_PRED_MODE[3][2] =
{
  {
     CNU, CNU,

  },
  {
     CNU, 113,

  },
  {
     CNU, CNU,

  },
};


static const UChar
INIT_QT_CBF[3][15] =
{
  {
      57,  88,  59,  58,  60,
      53,  84,  86,  85,  71,
      52,  99,  86,  85,  71,

  },
  {
     118,  73,  74,  59,  60,
      50,  52,  70,  70,  87,
      50,  52,  86,  86,  87,

  },
  {
     118,  73,  59,  59,  60,
      50,  52,  70,  86,  87,
      50,  52,  86, 102, 119,

  },
};


static const UChar
INIT_QT_ROOT_CBF[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
      68,  55,  56,  57,

  },
  {
      52,  55,  56,  57,

  },
};


static const UChar
INIT_REF_PIC[3][6] =
{
  {
     CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     102, 118, 118, 118, 119, 119,

  },
  {
     102, 119, 119, 119, 119, 119,

  },
};


static const UChar
INIT_SIG_FLAG[3][100] =
{
  {
     121, 104,  72,  54,  73,  72, 119,  86,  71, 119, 119,  71,  54,  70,  71, 104, 119,  71,  53, 119, 119,  71, 100, 102,  71, 119, 117,  68,  84, 117,  86,  59,  57,  73, 119, 115, 118,  88, 100,  71,  72, 114, 100,  86,  71,  72,  76,  58,  74,  56,
     154, 167, 119,  52, 121, 151, 151,  85,  86, 119, 167, 102,  52,  52,  70, 167, 150, 148, 146, 118, 166, 165,  98,  69,  85, 150,  84,  67,  83,  52, 119, 122, 119, 151, 134, 115, 102, 119, 100, 118, 119,  99, 132, 165, 119, 167, CNU, CNU, CNU, CNU,

  },
  {
     152, 119, 119,  70, 119,  87,  87,  87,  70,  87, 119,  87,  70,  70,  87, 119,  87, 118,  69, 102, 102, 118,  70,  69, 117, 118, 102,  84, 117, 102, 118, 121, 119, 119, 119, 148, 119, 119, 116, 102, 119, 115, 117, 118, 119, 135, 121, 119, 119, 119,
     184, 119, 119,  84, 135, 134, 182,  70,  85, 118, 119, 135,  53, 100, 103, 151, 118, 148, 146,  86, 181, 133, 212,  84, 100,  85, 180,  67,  68,  86, 199, 184, 182, 135, 134, 148, 118, 119, 116, 149, 118, 115, 133, 118, 119, 135, CNU, CNU, CNU, CNU,

  },
  {
     152, 119, 119,  70, 119, 119, 119,  87,  86, 119, 119,  87,  70,  70, 119, 151, 119, 118,  69, 102, 118, 118,  70,  85,  86, 118,  70,  68,  85, 102, 102, 121, 135, 119, 119, 148, 119, 119, 116, 118, 119, 115, 117, 118, 119, 104,  90, 119, 135, 119,
     184, 119, 119,  68, 151, 182, 182,  86, 117, 119, 119, 135,  53,  84, 103, 120, 118, 133, 147, 133, 181, 133, 196,  68, 100, 149, 197,  67,  84, 182, 150, 184, 182, 135, 119, 148, 118, 119, 100, 102,  87, 115, 117, 118, 119, 167, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_SKIP_FLAG[3][3] =
{
  {
     CNU, CNU, CNU,

  },
  {
     181, 184, 185,

  },
  {
     181, 183, 185,

  },
};


static const UChar
INIT_SPLIT_FLAG[3][3] =
{
  {
     134,  73,  91,

  },
  {
      53,  72,  89,

  },
  {
      53,  72,  89,

  },
};


static const UChar
INIT_TRANS_SUBDIV_FLAG[3][10] =
{
  {
     CNU, 147, 101,  69, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     CNU, 117,  55,  71, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     CNU, 117,  55,  71, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_LAST_X[3][52] =
{
  {
     150, 166, 183, 165, 149, 134, 134, 119, 181, 165, 165, 180, 164, 164, 166, 180, 179, 179, 178, 178, 178, 178, 147, 179, 117,  70,
     184, 184, 185, 155, 167, 168, 168, 168, 170, 103, 167, 183, 167, 102,  55, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     166, 151, 183, 119, 166, 182, 182, 167, 119, 166, 166, 181, 181, 165, 118, 104, 118, 165, 165, 180, 163, 147, 178, 163, 116, 100,
     169, 200, 185, 169, 168, 184, 184, 185, 138, 152, 183, 168, 168, 182, 134, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     166, 182, 183, 119, 166, 182, 182, 167, 119, 166, 166, 181, 181, 165, 118,  88, 118, 181, 165, 180, 179, 147, 163, 147, 115,  68,
     184, 200, 185, 169, 184, 184, 184, 185, 138, 167, 183, 183, 183, 102, 102, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_LAST_Y[3][52] =
{
  {
     167, 183, 184, 198, 103, 119, 119, 151, 165, 180, 180, 180, 180, 165, 118, 179, 178, 162, 162, 178, 178, 178, 179, 163, 100, 116,
     186, 170, 186, 170, 152, 184, 184, 184, 185, 151, 167, 151, 182, 150, 118, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     167, 183, 183, 151, 182, 167, 167, 119, 119, 166, 166, 166, 181, 149, 117, 104, 119, 181, 165, 180, 179, 163, 147,  99, 114, 116,
     185, 185, 201, 170, 184, 200, 184, 119, 154, 168, 168, 183, 166, 165, 116, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     182, 183, 183, 151, 182, 167, 182, 119, 119, 166, 166, 181, 181, 149, 117, 135, 119, 181, 165, 180, 179, 163, 147,  99, 114, 116,
     185, 185, 201, 185, 184, 200, 184, 167, 154, 168, 152, 183, 182, 117, 132, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_MVP_IDX[3][2] =
{
  {
     CNU, CNU,

  },
  {
     119, 119,

  },
  {
     119, 119,

  },
};


static const UChar
INIT_PLANARFLAG[3][2] =
{
  {
     152, 137,

  },
  {
     121, 122,

  },
  {
     121, 122,

  },
};

#endif

#else

#if HHI_SYS_PIPE_LC_MODE
#if HHI_SYS_PIPE_ABSX_ALTCTX
static const UChar
INIT_ALF_FLAG[3][1] =
{
  {
     CNU,

  },
  {
     CNU,

  },
  {
     CNU,

  },
};


static const UChar
INIT_ALF_SVLC[3][3] =
{
  {
     CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU,

  },
};


static const UChar
INIT_ALF_UVLC[3][2] =
{
  {
     CNU, CNU,

  },
  {
     CNU, CNU,

  },
  {
     CNU, CNU,

  },
};


static const UChar
INIT_AO_FLAG[3][1] =
{
  {
      56,

  },
  {
      56,

  },
  {
      56,

  },
};


static const UChar
INIT_AO_SVLC[3][3] =
{
  {
     183, 103, 196,

  },
  {
     151, 119, 195,

  },
  {
     135, 119, 181,

  },
};


static const UChar
INIT_AO_UVLC[3][2] =
{
  {
      92, 152,

  },
  {
      88, 135,

  },
  {
     120, 151,

  },
};


static const UChar
INIT_ABS_FLAG[3][120] =
{
  {
      55,  71,  54,  38,  86,  70,  70,  70,  70,  38, CNU, CNU,  56,  71,  71,  86,  54,  54, 183, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,  58,  57,  56,  56,  55,  55,  55,  71,  38, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
      53,  53,  37,  19,  70,  70,  86,  70,   3, CNU, CNU, CNU,  70, 102, 102, 150, 165,  69,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
      69,  38,  38,  54,  68,  68,  69,  69,  70,   1, CNU, CNU,  69,  70,  70,  69, 101, 182, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, 101,  86,  86,  86,  86, 102,  53, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
      84,  37,  36,  37,  85,  69,  37,  21, CNU, CNU, CNU, CNU, 180, 165, 165, 101,   2, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
      54,  54,  54,  54,  69,  69,  85,  70,  70,  39, CNU, CNU,  86,  86,  86,  86,  86,  70, 183, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, 102, 118, 102,  70,  54,  22,  21,  52,   3, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
      53,  53,  20,  36,  85,  85,  53,  37,   0, CNU, CNU, CNU, 165, 165, 118,  70, 182,   3,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_ALF_CTRL_FLAG[3][3] =
{
  {
     CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU,

  },
};


static const UChar
INIT_CHROMA_PRED_MODE[3][4] =
{
  {
      53, CNU, CNU,  87,

  },
  {
      54, CNU, CNU, 118,

  },
  {
      55, CNU, CNU, 118,

  },
};


static const UChar
INIT_DQP[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_INTER_DIR[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU, CNU,

  },
  {
      84, CNU, CNU, CNU,

  },
};


static const UChar
INIT_INTRA_PRED_MODE[3][3] =
{
  {
     151, 103, 103,

  },
  {
     119, 119, 119,

  },
  {
     119, 119, 119,

  },
};


static const UChar
INIT_MERGE_FLAG_EXT[3][3] =
{
  {
     CNU, CNU, CNU,

  },
  {
     119, CNU, CNU,

  },
  {
     119, CNU, CNU,

  },
};


static const UChar
INIT_MERGE_IDX_EXT[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
     119, 118, 119, 119,

  },
  {
     119,  70, 119, 119,

  },
};


static const UChar
INIT_MVD[3][14] =
{
  {
     CNU, CNU, CNU, CNU, CNU, CNU, CNU,
     CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     135, CNU, CNU, 135, 184, 120, 154,
     119, CNU, CNU, 135, 184, 120, 123,

  },
  {
     135, CNU, CNU, 167, 169, 136, 123,
     119, CNU, CNU, 167, 169, 136, 123,

  },
};


static const UChar
INIT_ONE_FLAG[3][120] =
{
  {
      71,  54,  53,  37,  54,  53,  52,  83,  83,  51, CNU, CNU,  56,  55,  54,  53,  68,  67,  66,  67, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,  58,  57,  56,  55,  39,  54,  38,  52,  50,  33, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
      68,  99,  50,  34, 147, 147, 162, 162, 162,  17, CNU, CNU,  54,  53,  99,  67, 146, 162,  66,  32, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     180, 179,  83,  36, 180, 178, 178, 178,  98,  99, CNU, CNU, 181, 179, 178, 178, 177, 113, 129,  64, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, 103, 101,  85,  84,  83,  82,  97, 177, 177, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
     178, 178,  66,  66, 180, 178, 177, 177, 177,   0, CNU, CNU, 180, 178, 178, 177, 177, 177,  16, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     180, 179,  99,  84, 180, 179, 178, 178, 178,  99, CNU, CNU, 103, 101, 100,  99,  98,  98,  66,  34, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,  88, 102,  86,  85,  84,  37,  22,  20,  67,  19, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
     179, 178,  82,  66, 180, 179, 178, 178,  66,   1, CNU, CNU, 180, 179, 178, 130, 145,  97, 194,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_PART_SIZE[3][5] =
{
  {
     152, CNU, CNU, CNU, CNU,

  },
  {
     151, 103, 202, CNU, CNU,

  },
  {
     119, 101, 181,  83,  55,

  },
};


static const UChar
INIT_PRED_MODE[3][2] =
{
  {
     CNU, CNU,

  },
  {
     CNU, 115,

  },
  {
     CNU, CNU,

  },
};


static const UChar
INIT_QT_CBF[3][15] =
{
  {
      57, CNU, CNU, CNU, CNU,
      52, CNU, CNU, CNU, CNU,
      52, CNU, CNU, CNU, CNU,

  },
  {
     119, CNU, CNU, CNU, CNU,
      51, CNU, CNU, CNU, CNU,
      51, CNU, CNU, CNU, CNU,

  },
  {
     119, CNU, CNU, CNU, CNU,
      51, CNU, CNU, CNU, CNU,
      51, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_QT_ROOT_CBF[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
      55, CNU, CNU, CNU,

  },
  {
      55, CNU, CNU, CNU,

  },
};


static const UChar
INIT_REF_PIC[3][6] =
{
  {
     CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     118, CNU, CNU, CNU, 119, 119,

  },
  {
     134, CNU, CNU, CNU, 119, 119,

  },
};


static const UChar
INIT_SIG_FLAG[3][56] =
{
  {
      75,  73,  87,  87,  74,  88,  87,  87,  86,  86,  91,  89,  72, 103,  71,  86,  84,  52,  92,  59,  58,  57,  56,  56,  56,  55,  53,  68,
     186, 152, 102, 150, 186, 151, 150, 102, 100,  51, 155, 104,  71, 150,  53,  52,  51,  51, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     183, 119, 102, 118, 104, 119, 102,  86,  85,  85,  90,  88, 103, 102,  85,  84,  99,  84,  91, 104, 119, 119,  87,  86,  84,  83,  50,  83,
     185, 183, 102, 134, 186, 183, 118, 180, 147, 147, 185, 135,  87, 118, 116,  67,  50,  50, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     183, 119, 102, 118, 120, 119, 118, 101,  84,  85, 106, 104, 119,  87, 101,  84,  83,  68, 107, 120, 119, 119,  71,  71,  87,  71,  52,  68,
     185, 183, 102, 134, 186, 183, 118, 164, 131, 131, 186, 135,  87, 134,  68,  51,  50,  67, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_SKIP_FLAG[3][3] =
{
  {
     CNU, CNU, CNU,

  },
  {
     182, CNU, CNU,

  },
  {
     182, CNU, CNU,

  },
};


static const UChar
INIT_SPLIT_FLAG[3][3] =
{
  {
      72, CNU, CNU,

  },
  {
      55, CNU, CNU,

  },
  {
      55, CNU, CNU,

  },
};


static const UChar
INIT_TRANS_SUBDIV_FLAG[3][10] =
{
  {
     CNU,  68,  69,  99, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     CNU,  53,  54,  71, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     CNU,  53,  54,  71, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_LAST_X[3][52] =
{
  {
     166, 166, 167, 165, 165, 102, 102, 103, 150, 165, 165, 165, 165, 181, 166, 180, 163, 179, 179, 179, 147, 179, 180, 180, 182,  85,
     184, 184, 201, 186, 167, 184, 167, 152, 200, 182, 198, 182, 182, 182,  71, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     166, 182, 167, 181, 181, 182, 181, 119, 134, 181, 182, 180, 180, 180, 165, 119, 133, 180, 180, 181, 179, 177, 193, 163, 132, 131,
     184, 184, 186, 169, 184, 185, 183, 152, 184, 182, 183, 183, 119, 118, 133, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     166, 182, 167, 166, 181, 182, 182, 119, 134, 181, 181, 181, 180, 180, 133, 150, 149, 180, 180, 180, 147, 147, 182, 132, 148, 146,
     184, 184, 186, 153, 183, 184, 183, 184, 184, 182, 183, 183, 166, 132,  70, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_LAST_Y[3][52] =
{
  {
     182, 183, 167, 183, 103, 182, 182, 166, 149, 180, 180, 180, 181, 165,  86, 179, 163, 178, 147, 179, 147, 179, 181,  84,  68,  68,
     185, 201, 202, 186, 184, 184, 183, 184, 183, 182, 182, 182, 182, 101, 166, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     167, 167, 167, 182, 182, 182, 181, 134, 119, 181, 182, 181, 180, 180,  84, 150, 181, 181, 181, 180, 179, 147, 179, 115,  66,  84,
     185, 185, 201, 170, 184, 184, 182, 183, 185, 183, 198, 183, 183,  67, 147, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     182, 183, 135, 182, 182, 182, 181, 165, 150, 181, 181, 181, 181, 180, 131, 150, 180, 180, 180, 180, 147, 147, 182,  83,  66, 115,
     185, 185, 201, 185, 184, 184, 198, 150, 184, 183, 183, 183, 133,  99, 133, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_MVP_IDX[3][2] =
{
  {
     CNU, CNU,

  },
  {
     119, 119,

  },
  {
     119, 118,

  },
};


static const UChar
INIT_PLANARFLAG[3][2] =
{
  {
     104, 171,

  },
  {
     186, 188,

  },
  {
     186, 187,

  },
};

#else
static const UChar
INIT_ALF_FLAG[3][1] =
{
  {
     CNU,

  },
  {
     CNU,

  },
  {
     CNU,

  },
};


static const UChar
INIT_ALF_SVLC[3][3] =
{
  {
     CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU,

  },
};


static const UChar
INIT_ALF_UVLC[3][2] =
{
  {
     CNU, CNU,

  },
  {
     CNU, CNU,

  },
  {
     CNU, CNU,

  },
};


static const UChar
INIT_AO_FLAG[3][1] =
{
  {
      56,

  },
  {
      56,

  },
  {
      56,

  },
};


static const UChar
INIT_AO_SVLC[3][3] =
{
  {
     183, 103, 196,

  },
  {
     151, 119, 195,

  },
  {
     135, 119, 181,

  },
};


static const UChar
INIT_AO_UVLC[3][2] =
{
  {
     171, 152,

  },
  {
      88, 135,

  },
  {
     120, 151,

  },
};


static const UChar
INIT_ABS_FLAG[3][120] =
{
  {
      71,  38,  38,  38,  71,  54,  70,  70,  38,   4, CNU, CNU,  71, 102,  86,  70, 182,   3,   1,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,  56,  55,  54,  54,  54,  70,  70, 182, 181,   3,   3,   2,   0,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
      53,  36,   1,   0,  70,  70,   1,   0, CNU, CNU, CNU, CNU,  70, 182,  36,   2,   2,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
      69,  38,  54,  38,  68,  70,  70,  38,   6,   0, CNU, CNU,  70, 102,  70,   3,   1,   0,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,  86,  70,  54,   2, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
      84,  36,   0, CNU,  85,   4, CNU, CNU, CNU, CNU, CNU, CNU, 117,   2, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
      54,  54,  38,  71,  85,  86,  70,  54,  39,   6, CNU, CNU, 102, 102, 150,  37,  21,   5,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, 118,  22,  38,  37,  20,  20,  20,   3,   3,   3,   3,   3,   3,   3,   2,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
      53,  20,  35, CNU,  85,  21,   0, CNU, CNU, CNU, CNU, CNU, 165,  52,   3,   1,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_ALF_CTRL_FLAG[3][3] =
{
  {
     CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU,

  },
};


static const UChar
INIT_CHROMA_PRED_MODE[3][4] =
{
  {
      53, CNU, CNU,  71,

  },
  {
      54, CNU, CNU, 118,

  },
  {
      55, CNU, CNU, 118,

  },
};


static const UChar
INIT_DQP[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_INTER_DIR[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU, CNU,

  },
  {
      84, CNU, CNU, CNU,

  },
};


static const UChar
INIT_INTRA_PRED_MODE[3][3] =
{
  {
     151, 103, 103,

  },
  {
     119, 119, 119,

  },
  {
     119, 119, 119,

  },
};


static const UChar
INIT_MERGE_FLAG_EXT[3][3] =
{
  {
     CNU, CNU, CNU,

  },
  {
     119, CNU, CNU,

  },
  {
     119, CNU, CNU,

  },
};


static const UChar
INIT_MERGE_IDX_EXT[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
     119, 118, 119, 119,

  },
  {
     119,  70, 119, 119,

  },
};


static const UChar
INIT_MVD[3][14] =
{
  {
     CNU, CNU, CNU, CNU, CNU, CNU, CNU,
     CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     135, CNU, CNU, 135, 184, 120, 154,
     119, CNU, CNU, 135, 184, 120, 123,

  },
  {
     135, CNU, CNU, 167, 169, 136, 123,
     119, CNU, CNU, 167, 169, 136, 123,

  },
};


static const UChar
INIT_ONE_FLAG[3][120] =
{
  {
      54,  37,  37,  38,  53,  68,  83,  83,  51,  52,  38,  38,  70,  68,  67,  50,  66,  66,  66,  50,  17,  17,   0,   0, CNU, CNU, CNU, CNU,  56,  39,  38,  38,  37,  52,  20,  35,  50,  50,  34,  34,  34,  66,  66,  33,  33,  16,  16,  32,  16,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
     147,  50,  34,  34, 147, 162, 146, 162,  16,   0, CNU, CNU, 148, 146, 162,  66,  66,  33,  16,  16,  32,  16,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     179,  83,  36,  37, 179, 178,  98,  67,  68,  37,  38, CNU, 179, 177, 113,  97,  66,  66,  34,   0,   4,   0,   0,   0, CNU, CNU, CNU, CNU,  85,  82,  65,  64,  64,  64,  64, 177,  64,  64,  16,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
     178,  66,  34,  17, 179, 177,  65,   0,   0, CNU, CNU, CNU, 179, 177,  64,  64,  16, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     179,  83,  36,  37, 179, 178,  98,  99,  68,  70,  21, CNU, 100,  98,  82,  82,  66,  98, 147,  67,  67,  18,   0, CNU, CNU, CNU, CNU, CNU,  86,  37,  22,  22,  21,  19,  35,  50,  67,  67,  51,  51,  51,  19,  19,  19,  19,  19,   2,   1,   1,   1,   1,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,
     179,  82,  34,  50, 179, 178,  66,  35,   2,   0,   0, CNU, 179, 145, 178, 194, 178,  66,  50,   1,   0,   0,   0, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_PART_SIZE[3][5] =
{
  {
     152, CNU, CNU, CNU, CNU,

  },
  {
     151, 103, 202, CNU, CNU,

  },
  {
     119, 101, 181,  83,  55,

  },
};


static const UChar
INIT_PRED_MODE[3][2] =
{
  {
     CNU, CNU,

  },
  {
     CNU, 115,

  },
  {
     CNU, CNU,

  },
};


static const UChar
INIT_QT_CBF[3][15] =
{
  {
      57, CNU, CNU, CNU, CNU,
      52, CNU, CNU, CNU, CNU,
      52, CNU, CNU, CNU, CNU,

  },
  {
     119, CNU, CNU, CNU, CNU,
      51, CNU, CNU, CNU, CNU,
      51, CNU, CNU, CNU, CNU,

  },
  {
     119, CNU, CNU, CNU, CNU,
      51, CNU, CNU, CNU, CNU,
      51, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_QT_ROOT_CBF[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
      55, CNU, CNU, CNU,

  },
  {
      55, CNU, CNU, CNU,

  },
};


static const UChar
INIT_REF_PIC[3][6] =
{
  {
     CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     118, CNU, CNU, CNU, 119, 119,

  },
  {
     134, CNU, CNU, CNU, 119, 119,

  },
};


static const UChar
INIT_SIG_FLAG[3][56] =
{
  {
      75,  73,  87,  87,
      89,  88, 103,  87,  71, 102,
      91,  73,  72, 103,  71,  86,  84,  99,
      92,  59,  58,  57,  57,  56,  56,  55,  53,  52,
      186, 152,  54,  54,
      186, 151, 150, 150,  68,  99,
      155, 104, 103,  71,  53,  52,  51,  51,
      CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU ,

  },
  {
     119, 119, 102, 118,
     104, 119, 102,  86,  69,  85,
     90,  88, 103, 102,  85,  84,  99,  84,
     91, 104, 119, 119,  87,  85,  84,  83,  67,  83,
     185, 183, 102, 134,
     186, 119, 118, 148, 131, 147,
     185, 135,  87, 118,  68,  67,  50,  50,
     CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     183, 119, 102, 118,
     120, 119, 102, 101,  84,  85,
     106, 104, 119, 118,  85,  84,  83,  68,
     107, 120, 119, 119,  71,  71,  87,  40,  37,  52,
     185, 183, 102, 134,
     186, 183, 118, 164, 131, 115,
     186, 135,  87, 134,  84,  51,  50,  67,
     CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};

static const UChar
INIT_SKIP_FLAG[3][3] =
{
  {
     CNU, CNU, CNU,

  },
  {
     182, CNU, CNU,

  },
  {
     182, CNU, CNU,

  },
};


static const UChar
INIT_SPLIT_FLAG[3][3] =
{
  {
      72, CNU, CNU,

  },
  {
      55, CNU, CNU,

  },
  {
      55, CNU, CNU,

  },
};


static const UChar
INIT_TRANS_SUBDIV_FLAG[3][10] =
{
  {
     CNU,  68, 100,  99, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     CNU,  53,  54,  71, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     CNU,  53,  54,  71, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_LAST_X[3][52] =
{
  {
     166, 166, 167, 150, 165, 102, 102, 103, 150, 165, 165, 165, 165, 150, 166, 180, 179, 179, 179, 179,  99, 179, 180, 180, 182,  85,
     184, 184, 201, 155, 167, 184, 167, 152, 199, 182, 198, 182, 197, 182, 183, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     166, 182, 167, 150, 181, 182, 182, 119, 134, 181, 182, 181, 180, 180, 165, 119, 118, 180, 180, 181, 179, 177, 193, 163, 132,  68,
     184, 184, 186, 153, 184, 184, 183, 136, 184, 182, 183, 183, 119,  70, 149, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     166, 182, 167, 166, 181, 182, 182, 119, 134, 181, 181, 181, 180, 180, 133, 150, 149, 180, 180, 180, 147, 147, 182, 180, 116, 131,
     184, 184, 186, 153, 183, 184, 183, 151, 184, 182, 183, 183, 182, 132,  70, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_LAST_Y[3][52] =
{
  {
     151, 183, 167, 183, 166, 182, 182, 182, 165, 165, 180, 165, 181, 166,  70, 179, 179, 147, 179, 179,  99, 179, 181,  84,  67,  84,
     185, 201, 202, 170, 184, 184, 183, 184, 199, 182, 182, 182, 165, 149, 166, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     167, 167, 167, 182, 182, 182, 181, 118, 119, 181, 182, 181, 180, 180,  84, 150, 181, 181, 181, 180, 179, 147, 179, 115,  82,  84,
     185, 185, 201, 170, 184, 199, 198, 183, 185, 183, 198, 183, 134, 131, 147, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     135, 183, 135, 182, 182, 182, 181, 165, 150, 181, 181, 181, 181, 180,  84, 150, 165, 180, 180, 180, 147, 147, 181,  83,  66, 147,
     185, 185, 201, 170, 184, 184, 182, 183, 184, 183, 198, 183, 165,  99, 133, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_MVP_IDX[3][2] =
{
  {
     CNU, CNU,

  },
  {
     119, 119,

  },
  {
     119, 118,

  },
};


static const UChar
INIT_PLANARFLAG[3][2] =
{
  {
     104, 171,

  },
  {
     186, 188,

  },
  {
     186, 187,

  },
};





#endif
#else


static const UChar
INIT_ALF_FLAG[3][1] =
{
  {
     118,

  },
  {
     102,

  },
  {
     102,

  },
};


static const UChar
INIT_ALF_SVLC[3][3] =
{
  {
     140, 119, 123,

  },
  {
      90, 119, 124,

  },
  {
     137, 119, 124,

  },
};


static const UChar
INIT_ALF_UVLC[3][2] =
{
  {
     104, 119,

  },
  {
     119, 119,

  },
  {
     119, 119,

  },
};


static const UChar
INIT_AO_FLAG[3][1] =
{
  {
      71,

  },
  {
     119,

  },
  {
      56,

  },
};


static const UChar
INIT_AO_SVLC[3][3] =
{
  {
     169, 119, 211,

  },
  {
     120, 119, 134,

  },
  {
     120, 119, 150,

  },
};


static const UChar
INIT_AO_UVLC[3][2] =
{
  {
     108, 104,

  },
  {
     151, 120,

  },
  {
     183, 120,

  },
};


static const UChar
INIT_ABS_FLAG[3][60] =
{
  {
     101, 119,  88, 120,  74, 117, 118, 119, 103, 105, 134, 119, 119,  88,  90,  83,  69,  70, 118,  71, 116, 117,  86, 118,  71, 133, 134,  71, 119, 119,
      85, 119, 120, 121, 138, 164, 134, 119, 167, 169, 150, 119, 103, 120, 138, 145, 147, 148, 165, 134, 163, 180, 164, 166, 118, 181, 149, 118, 166,  87,

  },
  {
      67,  85,  87, 119, 121,  84, 117, 118, 119, 121, 118,  87, 119, 119, 121,  82, 115,  85, 102, 118, 115, 116, 117, 102, 134,  85, 118, 118,  87, 119,
     146, 118, 167, 136, 122,  83, 165, 118, 135, 185,  52,  86, 118, 103, 169,  65,  83, 116, 117,   4,  51,  36, 100,  68,   4, 196,  86,  51,   5,  36,

  },
  {
      83,  70, 119, 119, 121, 116, 102, 119, 119, 121, 118, 119, 119, 119, 121,  98,  84,  85,  86, 118, 115,  85, 117, 118,  87, 133, 118, 118,  87, 119,
     146, 118, 151, 136, 122,  68, 134, 119, 120, 137,  86,  70, 119, 104, 153,  81,  99, 149, 165, 118,  99,  36,  69,  86, 215, 133,  53, 134,  53, 118,

  },
};


static const UChar
INIT_ALF_CTRL_FLAG[3][3] =
{
  {
     153, CNU, CNU,

  },
  {
     119, CNU, CNU,

  },
  {
     135, CNU, CNU,

  },
};


static const UChar
INIT_CHROMA_PRED_MODE[3][4] =
{
  {
     119,  37,  52, 119,

  },
  {
      38,  53,  53, 118,

  },
  {
      54,  54,  54, 118,

  },
};


static const UChar
INIT_DQP[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_INTER_DIR[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
     CNU, CNU, CNU, CNU,

  },
  {
      68,  84,  53, CNU,

  },
};


static const UChar
INIT_INTRA_PRED_MODE[3][3] =
{
  {
     152, 119, 119,

  },
  {
     119, 119, 119,

  },
  {
     119, 119, 119,

  },
};


static const UChar
INIT_MERGE_FLAG_EXT[3][3] =
{
  {
     CNU, CNU, CNU,

  },
  {
      87,  87,  87,

  },
  {
     118,  87,  87,

  },
};


static const UChar
INIT_MERGE_IDX_EXT[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
     119, 102,  87, 119,

  },
  {
     119, 117, 119, 119,

  },
};


static const UChar
INIT_MVD[3][14] =
{
  {
     CNU, CNU, CNU, CNU, CNU, CNU, CNU,
     CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     135, 120,  89, 134, 168, 135, 122,
     119, 104,  89, 166, 168, 135, 122,

  },
  {
     119,  89, 121, 166, 168, 120, 154,
     119, 120,  89, 166, 184, 120, 154,

  },
};


static const UChar
INIT_ONE_FLAG[3][60] =
{
  {
      88,  99,  84, 116, 102, 119, 132,  86, 102, 118, 120, 118, 119, 119, 119,  85,  66, 114, 114,  99, 102,  84, 116, 132, 148, 119,  85, 133,  86, 102,
      88, 163, 163, 149, 118, 151, 116, 165, 133, 150, 153, 134, 150, 166, 150, 116, 176, 145, 161, 114, 149, 115, 163, 147, 132, 118, 148, 148, 133, 117,

  },
  {
     166, 180, 194, 179, 164, 119, 179, 164, 148, 164, 135, 118, 118, 118, 118, 100, 114, 114,  98, 114, 133, 115, 147, 147, 147, 118, 164, 133, 117, 133,
     167, 194, 178, 179, 180, 103, 131, 179, 179, 132,  88, 181, 150, 101, 181,  99, 177, 145, 176, 129,  52,  98,  99, 180,  82, 102,  18, 148, 132,  51,

  },
  {
     166, 180, 179, 164, 164, 119, 179, 164, 164, 165, 120, 118, 119, 118, 118, 132, 146, 114, 114, 130, 149, 131, 163, 163, 116,  87, 117, 133, 133, 102,
     167, 210, 179, 179, 180, 103, 179, 180, 116, 133, 136,  85,  87, 118, 118, 131, 177, 145, 113, 177, 101, 130,  83,  99, 131,  86,  84, 165, 149, 134,

  },
};


static const UChar
INIT_PART_SIZE[3][5] =
{
  {
     136, CNU, CNU, CNU, CNU,

  },
  {
     151, 119, 187, CNU, CNU,

  },
  {
     119, 102, 168,  39,  70,

  },
};


static const UChar
INIT_PRED_MODE[3][2] =
{
  {
     CNU, CNU,

  },
  {
     CNU, 113,

  },
  {
     CNU, CNU,

  },
};


static const UChar
INIT_QT_CBF[3][15] =
{
  {
      57,  88,  60,  58,  60,
      53,  84,  86,  85,  71,
      52,  99,  86,  85,  71,

  },
  {
     118,  73,  59,  59,  61,
      50,  51,  85,  54,  87,
      65,  51,  86,  86,  87,

  },
  {
     118,  73,  59,  59,  61,
      50,  51,  70,  70,  87,
      65,  51,  86, 102, 119,

  },
};


static const UChar
INIT_QT_ROOT_CBF[3][4] =
{
  {
     CNU, CNU, CNU, CNU,

  },
  {
      68,  55,  56,  57,

  },
  {
      52,  55,  56,  57,

  },
};


static const UChar
INIT_REF_PIC[3][6] =
{
  {
     CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     102, 118, 118, 118, 119, 119,

  },
  {
     102, 119, 119, 119, 119, 119,

  },
};


static const UChar
INIT_SIG_FLAG[3][100] =
{
  {
     106,  73,  72,  54,  73,  72, 119,  54,  71, 119, 119,  71,  38,  54,  71,  73, 119, 102,  52, 119, 119, 102,  83,  86,  86, 134,  84,  52,  83, 100,  85,  75,  57,  73, 119,  83, 118,  88,  99,  71,  72, 113, 115,  85,  71,  72,  61,  59,  59,  56,
     186, 136, 119,  51, 169, 151, 151,  52,  86, 119, 167, 102,  36,  51,  70, 168, 181,  99,  81, 102, 197, 164,  49,  52, 100, 150,  67,  50,  50,  35, 119, 170, 119, 151, 134,  98, 117, 119,  83,  86, 119, 113, 115, 149, 119, 167, CNU, CNU, CNU, CNU,

  },
  {
     184, 119, 119,  69, 119,  87,  87,  87,  85,  87, 119,  87,  54,  69,  87, 119,  87, 102,  68,  70, 102, 102,  69,  68, 100, 102,  70,  67,  84,  70, 102, 137, 119, 119, 118, 115,  87, 119,  83, 102, 119, 113, 147, 149, 119, 135, 106, 119, 119, 118,
     185, 119, 119,  67, 151, 134, 182,  69,  53, 118, 151, 183,  36,  67, 103, 183, 149, 147, 146,  85, 180, 180, 195,  67, 115, 181, 196,  82,  67, 117, 103, 185, 182, 182, 181, 146, 165, 135,  99, 117, 118,  98, 163, 149, 119, 167, CNU, CNU, CNU, CNU,

  },
  {
     184, 119, 119,  69, 119, 119, 119,  87,  70, 119, 119,  87,  54,  69, 119, 151, 119, 102,  68,  86, 118, 102,  69,  68,  85, 102,  85,  67,  68, 117, 102, 153, 135, 119, 119, 115, 118, 119, 146, 102, 119, 113, 100, 118, 119, 104, 106, 119, 135, 119,
     186, 135, 119,  83, 183, 182, 182,  69,  53, 119, 151, 135,  52,  67, 103, 152, 118, 148, 146,  85, 180, 148, 179,  67,  99, 181, 197,  66,  67, 133, 150, 185, 182, 182, 166, 146, 118, 119,  99, 117,  87, 113, 147, 165, 119, 167, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_SKIP_FLAG[3][3] =
{
  {
     CNU, CNU, CNU,

  },
  {
     181, 184, 185,

  },
  {
     181, 183, 185,

  },
};


static const UChar
INIT_SPLIT_FLAG[3][3] =
{
  {
     134,  73,  91,

  },
  {
      68,  72,  74,

  },
  {
      53,  72,  74,

  },
};


static const UChar
INIT_TRANS_SUBDIV_FLAG[3][10] =
{
  {
     CNU, 146, 100,  84, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     CNU, 163,  70,  71, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     CNU, 148,  70,  71, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_LAST_X[3][52] =
{
  {
     150, 166, 183, 165, 149, 165, 134, 119, 181, 180, 180, 179, 179, 148, 166, 180, 179, 179, 178, 178, 178, 162, 178, 179, 117,  85,
     184, 184, 201, 155, 167, 168, 136, 168, 170, 134, 182, 182, 166, 102,  71, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     166, 151, 183, 119, 181, 182, 182, 167, 119, 165, 181, 180, 180, 164, 149, 104, 149, 180, 179, 179, 178, 162, 178, 163, 131,  99,
     169, 200, 201, 185, 168, 184, 183, 184, 138, 151, 183, 183, 166, 180, 118, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     166, 182, 183, 119, 166, 182, 182, 167, 119, 165, 181, 180, 180, 164, 117,  88, 149, 165, 164, 179, 178, 162, 162, 162, 162,  99,
     184, 200, 201, 169, 184, 184, 183, 184, 138, 151, 183, 182, 134, 116, 118, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_LAST_Y[3][52] =
{
  {
     167, 183, 184, 198, 103, 119, 119, 151, 180, 179, 179, 179, 179, 180, 118, 179, 178, 162, 162, 177, 162, 178, 194, 163, 115, 116,
     186, 186, 187, 186, 152, 184, 199, 184, 185, 182, 182, 150, 181, 165, 118, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     167, 183, 183, 151, 182, 167, 182, 119, 119, 181, 181, 181, 180, 164, 116, 104, 119, 181, 180, 179, 178, 162, 162,  83,  98, 115,
     185, 201, 202, 170, 184, 200, 183, 119, 170, 183, 182, 182, 180, 163, 100, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
  {
     182, 183, 183, 151, 182, 167, 182, 119, 119, 181, 181, 181, 180, 179, 116, 135, 119, 181, 180, 179, 178, 162, 162, 114, 113, 115,
     185, 201, 202, 185, 184, 200, 183, 135, 154, 183, 182, 182, 180,  99, 116, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU, CNU,

  },
};


static const UChar
INIT_MVP_IDX[3][2] =
{
  {
     CNU, CNU,

  },
  {
     119, 119,

  },
  {
     119, 119,

  },
};


static const UChar
INIT_PLANARFLAG[3][2] =
{
  {
     153, 139,

  },
  {
     154, 124,

  },
  {
     154, 124,

  },
};

#endif

#endif

#else

// initial probability for split flag
static const Short
INIT_SPLIT_FLAG[3][NUM_SPLIT_FLAG_CTX][2] =
{
  {
    {   -7,   68 }, {  -10,   87 }, {  -10,  105 }
  },
  {
    {  -14,   71 }, {   -6,   73 }, {   -6,   91 }
  },
  {
    {  -14,   71 }, {   -7,   74 }, {  -10,   92 }
  }
};

// initial probability for skip flag
static const Short
INIT_SKIP_FLAG[3][NUM_SKIP_FLAG_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  }
};

// initial probability for skip flag
static const Short
INIT_ALF_CTRL_FLAG[3][NUM_ALF_CTRL_FLAG_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  }
};

// initial probability for merge flag
static const Short
INIT_MERGE_FLAG_EXT[3][NUM_MERGE_FLAG_EXT_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }
  }
};

static const Short
INIT_MERGE_IDX_EXT[3][NUM_MERGE_IDX_EXT_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    1,   65 }, {    6,   42 }, {   -7,   75 }, {   -4,   72 }
  }
};

// initial probability for PU size
static const Short
INIT_PART_SIZE[3][NUM_PART_SIZE_CTX][2] =
{
  {
    {    0,   73 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }
  },
  {
    {   -1,   64 }, {   -3,   63 }, {    6,   78 }, {    0,   64 },
    {    0,   64 }
  },
  {
    {    6,   50 }, {   -1,   56 }, {   13,   53 }, {  -11,   76 },
    {  -11,   70 }
  }
};

// initial probability for prediction mode
static const Short
INIT_PRED_MODE[3][NUM_PRED_MODE_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {  -25,   89 }
  },
  {
    {    0,   64 }, {    0,   64 }
  }
};

// initial probability for intra direction of luma
#if MTK_DCM_MPM
static const Short
INIT_INTRA_PRED_MODE[3][NUM_ADI_CTX][2] =
{
  {
    {    2,   54 }, {  -3,   65  }, {   -3,   65 }
  },
  {
    {    0,   50 }, {  -2,   61  }, {   -2,   61 }
  },
  {
    {    0,   51 }, {  1,   55   }, {    1,   55 }
  }
};
#else
static const Short
INIT_INTRA_PRED_MODE[3][NUM_ADI_CTX][2] =
{
  {
    {    2,   54 }, {   -3,   65 }
  },
  {
    {    0,   50 }, {   -2,   61 }
  },
  {
    {    0,   51 }, {    1,   55 }
  }
};
#endif

#if ADD_PLANAR_MODE
// initial probability for planar mode flag
static const Short
INIT_PLANARFLAG[3][NUM_PLANARFLAG_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }
  }
};
#endif

// initial probability for intra direction of chroma
static const Short
INIT_CHROMA_PRED_MODE[3][4][2] =
{
#if CHROMA_CODEWORD
  {
    {  0,   64 }, {   0,   64 }, {  0,  64 }, {  0,  64 }
  },
  {
    {  0,   64 }, {   0,   64 }, {  0,  64 }, {  0,  64 }
  },
  {
    {  0,   64 }, {   0,   64 }, {  0,  64 }, {  0,  64 }
  }
#else
  {
    {  -14,   78 }, {   -5,   78 }, {  -13,  109 }, {  -13,  100 }
  },
  {
    {  -26,  113 }, {  -13,   92 }, {  -23,  110 }, {  -11,  105 }
  },
  {
    {  -20,  101 }, {  -25,  111 }, {  -29,  118 }, {  -24,  121 }
  }
#endif
};

// initial probability for temporal direction
static const Short
INIT_INTER_DIR[3][4][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {   -2,   58 }, {   -5,   70 }, {   -9,   85 }, {    1,   61 }
  }
};

// initial probability for motion vector difference
static const Short INIT_MVD[3][14][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }
  },
  {
    {   -6,   80 }, {   -6,   84 }, {   -9,   90 }, {    4,   62 },
    {   13,   55 }, {    2,   70 }, {    8,   74 }, {   -6,   77 },
    {   -7,   84 }, {   -9,   89 }, {    5,   59 }, {   10,   62 },
    {    4,   68 }, {    7,   75 }
  },
  {
    {   -4,   75 }, {   -5,   82 }, {  -12,   94 }, {    7,   55 },
    {   11,   59 }, {    6,   63 }, {    8,   71 }, {   -2,   71 },
    {   -5,   81 }, {  -21,  111 }, {    6,   58 }, {   10,   60 },
    {    5,   64 }, {   10,   67 }
  }
};

// initial probability for reference frame index
static const Short
INIT_REF_PIC[3][NUM_REF_NO_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }
  },
  {
    {   -6,   59 }, {  -10,   75 }, {   -8,   75 }, {  -17,   96 },
    {    1,   59 }, {    0,   64 }
  },
  {
    {   -9,   55 }, {   -9,   71 }, {   -9,   76 }, {  -12,   86 },
    {  -18,   55 }, {    0,   64 }
  }
};

// initial probability for dQP
static const Short
INIT_DQP[3][NUM_DELTA_QP_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  }
};

static const Short
INIT_QT_CBF[3][3*NUM_QT_CBF_CTX][2] =
{
  {
    {  -22,  116 }, {   -5,   75 }, {  -16,  112 }, {  -16,  111 }, {  -32,  165 },
    {  -35,  116 }, {  -12,   61 }, {   -9,   73 }, {  -10,   75 }, {  -14,   96 },
    {  -29,  104 }, {  -12,   59 }, {   -5,   65 }, {   -6,   67 }, {  -11,   90 }
  },
  {
    {  -18,   98 }, {  -41,  120 }, {  -29,  117 }, {  -23,  108 }, {  -35,  143 },
    {  -46,  114 }, {  -42,  119 }, {  -11,   74 }, {  -19,   90 }, {  -42,  139 },
    {  -43,  107 }, {  -41,  118 }, {  -17,   86 }, {  -25,  101 }, {  -14,   91 }
  },
  {
    {  -11,   80 }, {  -32,   83 }, {  -19,   89 }, {  -16,   85 }, {  -19,  102 },
    {  -22,   52 }, {  -48,  123 }, {   -7,   68 }, {  -37,  121 }, {  -58,  164 },
    {  -19,   45 }, {  -48,  123 }, {  -21,   94 }, {   -9,   73 }, {  -42,  138 }
  }
};

static const Short
INIT_QT_ROOT_CBF[3][NUM_QT_ROOT_CBF_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
  },
  {
    {  -22,   85 }, {  -15,   86 }, {  -13,   84 }, {  -23,  116 },
  },
  {
    {  -36,  103 }, {  -21,   95 }, {  -21,   97 }, {  -24,  114 },
  }
};

#if PCP_SIGMAP_SIMPLE_LAST
static const Short
INIT_LAST_X[3][2*NUM_CTX_LAST_FLAG_XY][2] =
{
  {
    {   8,  31 }, {   8,  39 }, {   8,  48 }, {  12,  31 },
    {   8,  38 }, {   6,  45 }, {   3,  46 }, {  -1,  56 },
    {  18,  16 }, {  14,  22 }, {  15,  22 }, {  16,  17 },
    {  16,  16 }, {  12,  24 }, {  -4,  59 }, {  33, -26 },
    {  18,   1 }, {  20,   2 }, {  22,  -1 }, {  17,  14 },
    {  11,  21 }, {  31, -24 }, {  38, -38 }, {  12,  11 },
    {  -4,  47 }, { -13,  69 }, {  32,  11 }, {  25,  27 },
    {  50,  -1 }, {  32,  20 }, {  12,  38 }, {  18,  32 },
    {  12,  41 }, {   5,  70 }, {  40,  -6 }, {  14,  29 },
    {  17,  26 }, {   7,  43 }, {  15,  26 }, {  15,  27 },
    {   9,  51 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 }
  },
  {
    {   9,  40 }, {   9,  44 }, {  10,  52 }, {  17,  24 },
    {  21,  15 }, {  20,  20 }, {  14,  29 }, {   8,  46 },
    {   7,  46 }, {  18,  18 }, {  26,   3 }, {  26,   0 },
    {  25,   2 }, {  15,  18 }, {   9,  27 }, {  23,  20 },
    {  19,  16 }, {  27,  -1 }, {  33, -19 }, {  38, -30 },
    {  41, -39 }, {  40, -36 }, {  26, -12 }, {   3,  20 },
    {  -7,  33 }, {  -4,  20 }, {  16,  42 }, {  16,  45 },
    {  49,   7 }, {   6,  61 }, {  17,  36 }, {  24,  24 },
    {  24,  21 }, {  61, -22 }, { -11, 101 }, {   6,  53 },
    {  14,  40 }, {  17,  30 }, {  20,  22 }, { -10,  67 },
    {  -6,  68 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 }
  },
  {
    {   9,  40 }, {   9,  44 }, {  10,  52 }, {  17,  24 },
    {  21,  15 }, {  20,  20 }, {  14,  29 }, {   8,  46 },
    {   7,  46 }, {  18,  18 }, {  26,   3 }, {  26,   0 },
    {  25,   2 }, {  15,  18 }, {   9,  27 }, {  23,  20 },
    {  19,  16 }, {  27,  -1 }, {  33, -19 }, {  38, -30 },
    {  41, -39 }, {  40, -36 }, {  26, -12 }, {   3,  20 },
    {  -7,  33 }, {  -4,  20 }, {  16,  42 }, {  16,  45 },
    {  49,   7 }, {   6,  61 }, {  17,  36 }, {  24,  24 },
    {  24,  21 }, {  61, -22 }, { -11, 101 }, {   6,  53 },
    {  14,  40 }, {  17,  30 }, {  20,  22 }, { -10,  67 },
    {  -6,  68 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 }
  }
};

static const Short
INIT_LAST_Y[3][2*NUM_CTX_LAST_FLAG_XY][2] =
{
  {
    {  19,  19 }, {  12,  36 }, {  16,  34 }, {  22,  18 },
    {  12,  35 }, {  12,  35 }, {  12,  32 }, {   5,  46 },
    {  16,  21 }, {  15,  20 }, {  17,  13 }, {  17,  14 },
    {  19,  10 }, {  19,  12 }, {   4,  37 }, {  13,  22 },
    {  22,  -4 }, {  27, -19 }, {  26, -12 }, {  18,   6 },
    {   6,  27 }, {  12,  10 }, {  34, -33 }, {  38, -42 },
    {  24, -15 }, {  14,   7 }, {  41,   7 }, {  45,   1 },
    {  56,  -9 }, {  30,  22 }, {  14,  40 }, {  23,  24 },
    {  19,  32 }, {  25,  26 }, {  29,  10 }, {  29,   4 },
    {  19,  20 }, {  10,  37 }, {  12,  38 }, {  -2,  60 },
    { -37, 114 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 }
  },
  {
    {  15,  29 }, {  14,  36 }, {  15,  41 }, {  25,   8 },
    {  26,   4 }, {  25,  10 }, {  21,  16 }, {  12,  34 },
    {  18,  21 }, {  25,   1 }, {  32, -12 }, {  32, -15 },
    {  29, -11 }, {  24,  -5 }, {  13,  12 }, {  27,   6 },
    {  40, -32 }, {  42, -39 }, {  40, -39 }, {  45, -51 },
    {  43, -51 }, {  45, -56 }, {  38, -46 }, {  15,  -6 },
    {  11,  -4 }, {  11,  -4 }, {  26,  29 }, {  28,  29 },
    {  21,  46 }, {  15,  45 }, {  27,  17 }, {  29,  12 },
    {  23,  15 }, {  -1,  57 }, {   6,  65 }, {  22,  17 },
    {  31,  -4 }, {  37, -25 }, {  43, -44 }, {  43, -46 },
    {  48, -44 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 }
  },
  {
    {  15,  29 }, {  14,  36 }, {  15,  41 }, {  25,   8 },
    {  26,   4 }, {  25,  10 }, {  21,  16 }, {  12,  34 },
    {  18,  21 }, {  25,   1 }, {  32, -12 }, {  32, -15 },
    {  29, -11 }, {  24,  -5 }, {  13,  12 }, {  27,   6 },
    {  40, -32 }, {  42, -39 }, {  40, -39 }, {  45, -51 },
    {  43, -51 }, {  45, -56 }, {  38, -46 }, {  15,  -6 },
    {  11,  -4 }, {  11,  -4 }, {  26,  29 }, {  28,  29 },
    {  21,  46 }, {  15,  45 }, {  27,  17 }, {  29,  12 },
    {  23,  15 }, {  -1,  57 }, {   6,  65 }, {  22,  17 },
    {  31,  -4 }, {  37, -25 }, {  43, -44 }, {  43, -46 },
    {  48, -44 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 },
    {   0,  64 }, {   0,  64 }, {   0,  64 }, {   0,  64 }
  }
};
#endif

#if SIMPLE_CONTEXT_SIG
static const Short
INIT_SIG_FLAG[3][2 * NUM_SIG_FLAG_CTX][2] =
{
  {
    // Luma 4x4
    {   -4,   94 }, {   -2,   84 }, {   -4,   83 }, {   16,   39 },
    {    0,   83 }, {   -1,   81 }, {   -4,   82 }, {  -16,   91 },
    {   -9,   86 }, {   -8,   89 }, {   -2,   82 }, {  -14,   92 },
    {  -16,   91 }, {   13,   44 }, {  -12,   91 },
    // Luma 8x8
    {  -10,   91 }, {   -9,   76 }, {   -3,   61 }, {    1,   46 },
    {  -13,   84 }, {  -13,   81 }, {   -2,   60 }, {   -5,   61 },
    {  -11,   77 }, {  -10,   76 }, {   -3,   65 }, {   -3,   63 },
    {   -4,   59 }, {  -14,   79 }, {    1,   61 }, {  -33,  126 },
    // Luma 16x16
    {   -5,   99 }, {   -9,   92 }, {   -5,   90 }, {   -7,   83 },
    {   -3,   37 }, {   -1,   62 }, {   -5,   81 }, {   -2,   41 },
    {    1,   64 }, {   -6,   82 }, {    0,   12 }, {    1,   43 },
    {    0,   57 }, {   -1,   66 }, {   -4,   79 },
    // Luma 32x32
    {   -3,  102 }, {  -17,  114 }, {   -7,   97 }, {  -12,   96 },
    // Chroma 4x4
    {   29,   42 }, {    1,   74 }, {   -6,   77 }, {   23,   41 },
    {   13,   63 }, {   -6,   88 }, {   -8,   80 }, {   35,   17 },
    {    0,   70 }, {  -17,  100 }, {  -25,  111 }, {  -39,  122 },
    {   26,   29 }, {  -36,  114 }, {  -41,  130 },
    // Chroma 8x8
    {   19,   45 }, {    8,   48 }, {    2,   40 }, {   -3,   40 },
    {    8,   46 }, {   16,   31 }, {  -10,   73 }, {  -51,  114 },
    {    2,   52 }, {   -9,   71 }, {  -37,  118 }, {   -8,   38 },
    {  -10,   64 }, {  -47,  113 }, { -105,  234 }, {  -32,  123 },
    // Chroma 16x16
    {   23,   51 }, {    0,   69 }, {   10,   54 }, {    0,   65 },
    {   -1,   42 }, {    4,   51 }, {   -3,   70 }, {  -12,   61 },
    {   -4,   50 }, {   -2,   68 }, {   -18,  57 }, {    5,   38 },
    {    1,   54 }, {   -3,   66 }, {   -9,   80 },
    // Chroma 32x32 (unused)
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    // Luma 4x4
    {    6,   67 }, {   -1,   72 }, {   -9,   84 }, {  -29,  100 },
    {    4,   65 }, {    0,   70 }, {   -7,   80 }, {  -34,  117 },
    {  -11,   79 }, {  -14,   91 }, {  -15,   93 }, {  -34,  119 },
    {   26,   26 }, {  -34,  111 }, {  -35,  126 },
    // Luma 8x8
    {    2,   69 }, {  -10,   82 }, {   -1,   57 }, {   -7,   59 },
    {   -5,   71 }, {  -13,   85 }, {  -28,  107 }, {  -29,  100 },
    {  -21,   93 }, {  -28,  108 }, {   -6,   71 }, {  -48,  136 },
    {  -35,  111 }, {  -10,   74 }, {    4,   56 }, {    5,   56 },
    // Luma 16x16
    {    0,   75 }, {    1,   59 }, {   -1,   67 }, {   -3,   67 },
    {    6,   29 }, {    2,   57 }, {   -5,   77 }, {   -5,   44 },
    {   -4,   64 }, {   -7,   78 }, {   -9,   34 }, {    2,   43 },
    {    2,   53 }, {    2,   60 }, {   -8,   83 },
    // Luma 32x32
    {   -7,   88 }, {    5,   52 }, {   -4,   74 }, {    5,   56 },
    // Chroma 4x4
    {  -15,  103 }, {   25,   19 }, {   23,   29 }, {  -65,  142 },
    {  -21,  108 }, {   27,   19 }, {  -34,  111 }, {  -71,  174 },
    {  -30,  104 }, {   34,   12 }, { -144,  285 }, { -168,  304 },
    {   43,    6 }, { -125,  248 }, { -129,  252 },
    // Chroma 8x8
    {   -3,   84 }, {  -35,  122 }, {  -42,  111 }, {  101, -147 },
    {  -14,   87 }, {  -70,  179 }, {    6,   30 }, {    0,   64 },
    {    8,   32 }, {  -64,  156 }, {    0,   64 }, {    0,   64 },
    {  -65,  144 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    // Chroma 16x16
    {   19,   22 }, {   11,   48 }, {    9,   51 }, {  -24,   95 },
    {  -27,   82 }, {  -22,   92 }, {  -20,   89 }, {  -25,   82 },
    {  -15,   77 }, {   24,   32 }, {  -31,   76 }, {  -19,   77 },
    {  -21,   84 }, {  -23,   95 }, {  -35,  112 },
    // Chroma 32x32 (unused)
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    // Luma 4x4
    {    6,   67 }, {   -1,   72 }, {   -9,   84 }, {  -29,  100 },
    {    4,   65 }, {    0,   70 }, {   -7,   80 }, {  -34,  117 },
    {  -11,   79 }, {  -14,   91 }, {  -15,   93 }, {  -34,  119 },
    {   26,   26 }, {  -34,  111 }, {  -35,  126 },
    // Luma 8x8
    {    2,   69 }, {  -10,   82 }, {   -1,   57 }, {   -7,   59 },
    {   -5,   71 }, {  -13,   85 }, {  -28,  107 }, {  -29,  100 },
    {  -21,   93 }, {  -28,  108 }, {   -6,   71 }, {  -48,  136 },
    {  -35,  111 }, {  -10,   74 }, {    4,   56 }, {    5,   56 },
    // Luma 16x16
    {    0,   75 }, {    1,   59 }, {   -1,   67 }, {   -3,   67 },
    {    6,   29 }, {    2,   57 }, {   -5,   77 }, {   -5,   44 },
    {   -4,   64 }, {   -7,   78 }, {   -9,   34 }, {    2,   43 },
    {    2,   53 }, {    2,   60 }, {   -8,   83 },
    // Luma 32x32
    {   -7,   88 }, {    5,   52 }, {   -4,   74 }, {    5,   56 },
    // Chroma 4x4
    {  -15,  103 }, {   25,   19 }, {   23,   29 }, {  -65,  142 },
    {  -21,  108 }, {   27,   19 }, {  -34,  111 }, {  -71,  174 },
    {  -30,  104 }, {   34,   12 }, { -144,  285 }, { -168,  304 },
    {   43,    6 }, { -125,  248 }, { -129,  252 },
    // Chroma 8x8
    {   -3,   84 }, {  -35,  122 }, {  -42,  111 }, {  101, -147 },
    {  -14,   87 }, {  -70,  179 }, {    6,   30 }, {    0,   64 },
    {    8,   32 }, {  -64,  156 }, {    0,   64 }, {    0,   64 },
    {  -65,  144 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    // Chroma 16x16
    {   19,   22 }, {   11,   48 }, {    9,   51 }, {  -24,   95 },
    {  -27,   82 }, {  -22,   92 }, {  -20,   89 }, {  -25,   82 },
    {  -15,   77 }, {   24,   32 }, {  -31,   76 }, {  -19,   77 },
    {  -21,   84 }, {  -23,   95 }, {  -35,  112 },
    // Chroma 32x32 (unused)
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  }
};
#else
static const Short
INIT_SIG_FLAG[3][224][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {  -24,  145 }, {  -40,  159 }, {  -38,  155 }, {  -44,  159 },
    {   -1,   27 }, {   -4,   68 }, {  -12,  100 }, {  -15,   61 },
    {  -14,   82 }, {  -23,  117 }, {    0,    6 }, {  -11,   54 },
    {    8,   40 }, {   -1,   65 }, {  -18,  104 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {  -31,  151 }, {  -19,  114 }, {  -22,  124 }, {  -14,  100 },
    {   -7,   39 }, {   -1,   61 }, {  -12,   96 }, {  -13,   55 },
    {   -1,   61 }, {  -16,  101 }, {    2,    5 }, {    8,   23 },
    {    6,   43 }, {    1,   62 }, {  -16,   99 }, {    0,   64 },
    {  -19,  119 }, {  -12,   91 }, {  -19,  106 }, {  -25,  108 },
    {   19,   -5 }, {  -24,   96 }, {   -7,   78 }, {   19,    1 },
    {    1,   54 }, {  -10,   82 }, {    1,   15 }, {   -3,   51 },
    {   -4,   62 }, {    1,   61 }, {   -5,   74 }, {    0,   64 },
    {  -10,  107 }, {   -9,   90 }, {   -7,   89 }, {  -12,   87 },
    {   -3,   32 }, {    0,   60 }, {   -3,   75 }, {   -6,   49 },
    {    3,   54 }, {   -5,   78 }, {  -16,   39 }, {   -3,   43 },
    {   -6,   64 }, {   -3,   67 }, {   -6,   80 }, {    0,   64 },
    {  -14,  112 }, {   -6,   78 }, {   -6,   83 }, {    2,   61 },
    {    9,   22 }, {   -6,   65 }, {  -26,  107 }, {    4,   33 },
    {    4,   52 }, {  -27,  110 }, {    9,    4 }, {    5,   38 },
    {    6,   46 }, {    6,   54 }, {   -3,   72 }, {    0,   64 },
    {  -10,   91 }, {   -9,   76 }, {   -3,   61 }, {    1,   46 },
    {  -13,   84 }, {  -13,   81 }, {   -2,   60 }, {   -5,   61 },
    {  -11,   77 }, {  -10,   76 }, {   -3,   65 }, {   -3,   63 },
    {   -4,   59 }, {  -14,   79 }, {    1,   61 }, {  -33,  126 },
    {   19,   45 }, {    8,   48 }, {    2,   40 }, {   -3,   40 },
    {    8,   46 }, {   16,   31 }, {  -10,   73 }, {  -51,  114 },
    {    2,   52 }, {   -9,   71 }, {  -37,  118 }, {   -8,   38 },
    {  -10,   64 }, {  -47,  113 }, { -105,  234 }, {  -32,  123 },
    {  -11,  105 }, {   -8,   84 }, {   -5,   72 }, {   -7,   71 },
    {  -13,   96 }, {   -9,   85 }, {   -4,   75 }, {   -7,   75 },
    {  -10,   84 }, {   -7,   81 }, {    0,   76 }, {   -3,   78 },
    {  -11,   84 }, {   -4,   75 }, {   -1,   83 }, {    0,   64 },
    {   32,   35 }, {    3,   59 }, {   -1,   61 }, {  -31,  104 },
    {   29,   28 }, {   11,   51 }, {   -6,   77 }, {  -41,  125 },
    {    9,   53 }, {   -3,   76 }, {  -34,  130 }, {  -42,  133 },
    {  -29,  109 }, {  -49,  143 }, {  -47,  144 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {  -22,  128 }, {  -14,   96 }, {  -17,   99 }, {  -26,  113 },
    {    5,   31 }, {   -2,   65 }, {  -32,  125 }, {  -19,   72 },
    {   -7,   69 }, {  -24,  111 }, {    0,   25 }, {   -1,   49 },
    {    2,   54 }, {    0,   64 }, {  -15,   97 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {   -4,   88 }, {    0,   68 }, {   -4,   75 }, {   -5,   74 },
    {   17,    7 }, {    2,   56 }, {   -8,   80 }, {    1,   36 },
    {    2,   52 }, {  -11,   85 }, {    6,   12 }, {    5,   41 },
    {    2,   54 }, {    0,   65 }, {   -8,   82 }, {    0,   64 },
    {   -8,   95 }, {   -7,   77 }, {   -7,   73 }, {   -9,   74 },
    {  -18,   77 }, {   -7,   72 }, {  -11,   81 }, {  -29,   92 },
    {    4,   47 }, {  -14,   83 }, {  -26,   79 }, {    2,   46 },
    {    5,   50 }, {    6,   52 }, {  -22,  104 }, {    0,   64 },
    {   -7,   93 }, {   -5,   79 }, {   -6,   79 }, {   -8,   79 },
    {    5,   36 }, {    3,   56 }, {  -18,   99 }, {  -10,   59 },
    {    0,   56 }, {  -11,   84 }, {   -3,   31 }, {    8,   35 },
    {    4,   51 }, {   -1,   66 }, {   -7,   81 }, {    0,   64 },
    {    1,   80 }, {  -28,  116 }, {  -26,  112 }, {    2,   56 },
    {  -17,   75 }, {    3,   52 }, {  -10,   79 }, {  -26,   91 },
    {   -2,   61 }, {   -3,   66 }, {  -45,  114 }, {  -21,   91 },
    {    1,   59 }, {    2,   62 }, {  -10,   82 }, {    0,   64 },
    {    3,   68 }, {    0,   62 }, {   -8,   69 }, {    1,   43 },
    {   -6,   70 }, {   -4,   66 }, {   -2,   60 }, {   -6,   61 },
    {  -13,   78 }, {   -8,   71 }, {   -6,   71 }, {   -5,   67 },
    {   -7,   63 }, {    0,   56 }, {  -13,   83 }, {   -4,   77 },
    {    6,   70 }, {  -19,   94 }, {  -21,   80 }, {   -7,   30 },
    {  -15,   88 }, {  -19,   95 }, {  -39,  117 }, {  -10,   42 },
    {  -15,   78 }, {  -30,  104 }, {    4,   36 }, {   91, -137 },
    {  -45,  125 }, {    0,   42 }, {   30,    2 }, {    0,   64 },
    {   -3,   84 }, {   -8,   83 }, {  -13,   86 }, {  -12,   75 },
    {   -6,   81 }, {   -9,   83 }, {   -5,   77 }, {   -8,   75 },
    {   -8,   79 }, {   -7,   82 }, {    0,   73 }, {   -7,   83 },
    {  -13,   84 }, {   -4,   73 }, {   -3,   82 }, {    0,   64 },
    {   15,   63 }, {    0,   69 }, {  -28,  110 }, {  -29,  104 },
    {   -1,   74 }, {  -22,  105 }, {  -50,  150 }, {  -57,  151 },
    {  -21,   96 }, {  -29,  113 }, {  -49,  151 }, {  -85,  197 },
    {  -46,  133 }, {  -60,  159 }, {  -84,  198 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {  -34,  146 }, {  -20,  108 }, {  -23,  111 }, {  -38,  141 },
    {    3,   34 }, {   -4,   69 }, {  -17,  103 }, {   -9,   50 },
    {   -4,   66 }, {  -16,   97 }, {   -1,   24 }, {    0,   50 },
    {   -1,   61 }, {  -12,   90 }, {  -14,   97 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {  -12,   99 }, {   -4,   75 }, {   -8,   81 }, {   -7,   80 },
    {   15,   13 }, {    2,   57 }, {  -10,   85 }, {    4,   27 },
    {  -10,   75 }, {   -9,   81 }, {   -9,   46 }, {    6,   41 },
    {    3,   56 }, {   -1,   69 }, {  -14,   96 }, {    0,   64 },
    {   -7,   85 }, {  -22,  102 }, {  -12,   79 }, {  -35,  121 },
    {   -5,   55 }, {  -16,   88 }, {   -9,   78 }, {  -26,   73 },
    {  -28,   99 }, {  -38,  123 }, {   -3,   35 }, {  -25,   97 },
    {   -9,   75 }, {    6,   54 }, {  -15,   92 }, {    0,   64 },
    {   -9,   94 }, {  -14,   94 }, {   -4,   74 }, {  -14,   92 },
    {    7,   37 }, {    3,   57 }, {   -7,   78 }, {   -8,   54 },
    {   -7,   70 }, {   -8,   78 }, {   -2,   31 }, {    4,   49 },
    {    1,   60 }, {   -5,   75 }, {  -10,   87 }, {    0,   64 },
    {   -3,   82 }, {  -19,  102 }, {  -14,   91 }, {  -33,  123 },
    {  -46,  124 }, {  -12,   82 }, {  -43,  138 }, {  -46,  121 },
    {  -38,  125 }, {  -44,  138 }, {  -32,   89 }, {   -3,   60 },
    {  -22,  102 }, {  -27,  114 }, {  -37,  128 }, {    0,   64 },
    {    2,   69 }, {  -10,   82 }, {   -1,   57 }, {   -7,   59 },
    {   -5,   71 }, {  -13,   85 }, {  -28,  107 }, {  -29,  100 },
    {  -21,   93 }, {  -28,  108 }, {   -6,   71 }, {  -48,  136 },
    {  -35,  111 }, {  -10,   74 }, {    4,   56 }, {    5,   56 },
    {   -3,   84 }, {  -35,  122 }, {  -42,  111 }, {  101, -147 },
    {  -14,   87 }, {  -70,  179 }, {    6,   30 }, {    0,   64 },
    {    8,   32 }, {  -64,  156 }, {    0,   64 }, {    0,   64 },
    {  -65,  144 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    4,   67 }, {   -5,   77 }, {  -12,   85 }, {  -28,  105 },
    {   -5,   77 }, {   -7,   79 }, {   -7,   79 }, {  -28,  109 },
    {  -17,   94 }, {  -11,   88 }, {   -6,   81 }, {  -32,  125 },
    {  -38,  128 }, {  -40,  136 }, {  -39,  142 }, {    0,   64 },
    {  -35,  148 }, {  -47,  150 }, {  -22,  100 }, {  -73,  176 },
    {  -12,   91 }, {  -54,  155 }, {  -86,  203 }, {  -71,  174 },
    {  -66,  167 }, {  -41,  129 }, { -144,  285 }, { -168,  304 },
    {  -59,  148 }, { -125,  248 }, { -129,  252 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  }
};
#endif
#if !PCP_SIGMAP_SIMPLE_LAST
static const Short
INIT_LAST_FLAG[3][224][2] =
{
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {   24,  -37 }, {   35,  -52 }, {    8,   -9 }, {   14,  -11 },
    {   14,  -10 }, {   22,  -18 }, {   48,  -64 }, {   -1,   29 },
    {   20,   13 }, {  -15,   51 }, {   77, -101 }, {   22,  -20 },
    {  -34,   89 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {   33,  -47 }, {   38,  -54 }, {   25,  -32 }, {   46,  -66 },
    {   24,  -27 }, {   26,  -28 }, {   36,  -39 }, {   -6,   48 },
    {   -2,   51 }, {   -1,   44 }, {    9,   26 }, {  -36,   93 },
    {  -20,   61 }, {  -51,  130 }, {   29,   -2 }, {    0,    0 },
    {   42,  -30 }, {   43,  -46 }, {   61,  -81 }, {   48,  -51 },
    {   21,    5 }, {    8,   22 }, {    5,   38 }, {   23,   12 },
    {  102, -138 }, {  304, -440 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {   30,  -24 }, {   24,  -15 }, {   19,   -2 }, {    9,   15 },
    {    9,   16 }, {    5,   27 }, {   19,    6 }, {   11,   29 },
    {   -7,   62 }, {   -9,   66 }, {    7,   41 }, {  -11,   76 },
    {  -42,  130 }, {  -14,   91 }, {   16,   14 }, {    0,   64 },
    {   22,   14 }, {   26,   -1 }, {   13,   18 }, {    4,   37 },
    {    5,   36 }, {   20,   14 }, {   12,   29 }, {   24,   17 },
    {  -50,  133 }, { -165,  315 }, { -160,  301 }, { -217,  408 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {   31,  -12 }, {   20,    4 }, {   16,    9 }, {   16,    9 },
    {   17,    8 }, {   20,    4 }, {   23,    4 }, {   17,   18 },
    {    9,   36 }, {   10,   36 }, {   10,   39 }, {    7,   52 },
    {   -3,   75 }, {  -16,  103 }, {    0,   64 }, {    0,   64 },
    {   14,   44 }, {   -2,   64 }, {    3,   51 }, {   18,   22 },
    {   -8,   70 }, {    3,   53 }, {    7,   48 }, {   -6,   70 },
    {  -48,  133 }, {  -22,   96 }, { -111,  240 }, {  -55,  156 },
    {  -92,  215 }, { -347,  585 }, {    0,   64 }, {    0,   64 },
    {   12,   26 }, {   11,   23 }, {   16,   15 }, {   19,   17 },
    {    9,   30 }, {   14,   24 }, {   18,   21 }, {   16,   30 },
    {   13,   27 }, {   12,   33 }, {   13,   37 }, {    9,   54 },
    {   12,   37 }, {   12,   42 }, {   15,   49 }, {    0,   64 },
    {   23,   26 }, {   17,   30 }, {    0,   60 }, {    7,   53 },
    {   18,   33 }, {   15,   39 }, {   10,   48 }, {  -22,  101 },
    {   19,   34 }, {   10,   51 }, {  -35,  124 }, {  -16,   93 },
    {    2,   65 }, {   -9,   83 }, {  -24,  109 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {   17,   18 }, {   28,  -21 }, {   40,  -50 }, {    7,   20 },
    {   -9,   47 }, {  -12,   43 }, {    9,   -1 }, {   -7,   52 },
    {  -29,   89 }, {   25,  -20 }, {  -36,   82 }, {  106, -178 },
    {  248, -404 }, {   56,  -38 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {   20,   10 }, {   25,   -8 }, {   23,   -5 }, {    2,   32 },
    {    8,   15 }, {   19,   -5 }, {   15,    0 }, {    8,   27 },
    {   -1,   48 }, {    1,   42 }, {    1,   41 }, {  -48,  124 },
    {  -97,  213 }, {   32,  -21 }, {   61,  -47 }, { -480,  845 },
    {   22,   17 }, {   26,   -6 }, {   42,  -38 }, {   33,  -15 },
    {    9,   29 }, {  -19,   70 }, {  -77,  167 }, {  -80,  176 },
    {  -96,  186 }, {  171, -294 }, {    5,   45 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {   23,    2 }, {   30,  -12 }, {   20,    9 }, {   16,   16 },
    {   19,    4 }, {    8,   24 }, {   15,   13 }, {   10,   26 },
    {   12,   25 }, {  -12,   66 }, {   -7,   60 }, {   -8,   64 },
    {   -2,   63 }, {   -7,   75 }, { -116,  252 }, {    0,   64 },
    {   -4,   68 }, {   20,   16 }, {   17,   22 }, {   13,   31 },
    {    3,   48 }, {  -21,   84 }, {  -41,  117 }, {  -43,  125 },
    {    5,   40 }, {   29,    4 }, {  -24,   87 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {   14,   23 }, {   24,    4 }, {   15,   23 }, {    7,   34 },
    {   19,   14 }, {   19,   12 }, {   17,   18 }, {   11,   33 },
    {    4,   44 }, {   -2,   55 }, {    2,   54 }, {    3,   53 },
    {  -43,  137 }, {  -22,  104 }, {    0,   64 }, {    0,   64 },
    {   -5,   76 }, {    7,   50 }, {    9,   43 }, {    6,   50 },
    {    0,   59 }, {    1,   56 }, {    4,   51 }, {   22,   18 },
    {   21,   21 }, { -105,  242 }, {  -52,  145 }, {  260, -439 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   50 }, {    5,   42 }, {    4,   43 }, {   18,   20 },
    {   17,   23 }, {    0,   55 }, {   10,   39 }, {    6,   46 },
    {   10,   36 }, {    9,   41 }, {   12,   42 }, {    3,   63 },
    {   15,   30 }, {    9,   46 }, {    0,   73 }, {    0,   64 },
    {   12,   47 }, {   11,   43 }, {    5,   50 }, {    5,   53 },
    {   -2,   68 }, {  -11,   83 }, {  -18,   93 }, {  -59,  154 },
    {  -22,   99 }, {  -45,  137 }, {  -63,  167 }, {  -56,  152 },
    {  -34,  117 }, {  -45,  136 }, {  -74,  182 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {   20,    7 }, {   38,  -44 }, {   37,  -43 }, {   -9,   48 },
    {  -45,  104 }, {  -87,  174 }, {   26,  -35 }, {   51,  -63 },
    {  129, -174 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    7,   38 }, {   28,  -14 }, {   25,   -7 }, {    2,   38 },
    {  -13,   53 }, {   11,    2 }, {  -18,   59 }, {   23,   -4 },
    {   46,  -35 }, {   -9,   59 }, {  -32,   97 }, {   -5,   57 },
    {  -98,  215 }, {  -51,  141 }, {  163, -267 }, {    0,    0 },
    {    2,   48 }, {   57,  -65 }, {   70,  -78 }, {   74,  -74 },
    {   38,   -7 }, {   64,  -44 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {   12,   24 }, {   17,   17 }, {   11,   31 }, {    6,   35 },
    {    9,   25 }, {   -2,   41 }, {   11,   19 }, {   -3,   51 },
    {  -30,  100 }, {   -4,   55 }, {    2,   49 }, {   38,  -17 },
    {   43,  -28 }, {  -35,  117 }, {  -97,  214 }, {    0,   64 },
    {    7,   41 }, {   29,   -3 }, {   17,   23 }, {    6,   45 },
    {   12,   36 }, {  -22,   85 }, {    2,   54 }, {  -18,   87 },
    {    0,   65 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {   -5,   59 }, {   14,   25 }, {   13,   29 }, {    4,   42 },
    {    8,   33 }, {    5,   37 }, {   20,   11 }, {   16,   20 },
    {   15,   22 }, {  -12,   70 }, {   -9,   67 }, {   13,   35 },
    {  -15,   85 }, {  -25,  101 }, {    0,   64 }, {    0,   64 },
    {   -3,   65 }, {    6,   50 }, {    3,   56 }, {   21,   19 },
    {   -1,   64 }, {  -73,  175 }, {  -17,   76 }, {    7,   52 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {   12,   22 }, {   15,   20 }, {   18,   14 }, {   13,   28 },
    {   18,   18 }, {    0,   54 }, {  -14,   77 }, {  -34,  113 },
    {    4,   43 }, {    7,   43 }, {    2,   55 }, {   -2,   69 },
    {  -17,   82 }, {   -2,   60 }, {   -8,   81 }, {    0,   64 },
    {  -20,   92 }, {    1,   56 }, {  -35,  114 }, {  -12,   78 },
    {    6,   51 }, {  -48,  141 }, {  -77,  183 }, {  -63,  157 },
    {    7,   46 }, {  -56,  149 }, { -125,  254 }, { -134,  247 },
    {  -49,  134 }, {  -72,  167 }, { -130,  253 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }, {    0,   64 }, {    0,   64 }
  }
};
#endif

static const Short
INIT_ONE_FLAG[3][2*NUM_ONE_FLAG_CTX][2] =
{
  {
    {  -11,   87 }, {  -20,   64 }, {  -16,   68 }, {  -13,   71 },
    {  -10,   73 }, {   -5,   67 }, {   -8,   26 }, {   -8,   37 },
    {   -3,   36 }, {   -9,   56 }, {    0,   63 }, {   -5,   39 },
    {  -12,   56 }, {   -9,   57 }, {   -1,   52 }, {   -4,   72 },
    {  -19,   73 }, {  -28,   88 }, {  -23,   85 }, {   -3,   59 },
    {   -2,   72 }, {  -27,   97 }, {  -22,   89 }, {  -14,   77 },
    {   -1,   58 }, {  -10,   86 }, {  -22,   74 }, {  -13,   63 },
    {   -6,   57 }, {   -5,   63 },
    {   -4,   70 }, {   16,   -2 }, {    9,   23 }, {    5,   41 },
    {   -7,   67 }, {   -6,   63 }, {   13,   -5 }, {  -12,   42 },
    {  -18,   53 }, {  -14,   59 }, {   -5,   65 }, {   -8,   36 },
    {  -22,   67 }, {  -35,   89 }, {   -3,   47 }, {  -12,   77 },
    {  -20,   66 }, {  -51,  113 }, {  -44,  109 }, {  -23,   84 },
    {   -1,   64 }, {  -58,  127 }, {  -71,  143 }, {  -67,  134 },
    {  -91,  187 }, {   -1,   64 }, {   20,   -3 }, {   13,   21 },
    {    2,   46 }, {    5,   48 }
  },
  {
    {   -4,   71 }, {   29,   -5 }, {    1,   45 }, {   -6,   58 },
    {   -9,   67 }, {   -6,   66 }, {   23,   -7 }, {    3,   26 },
    {   -5,   42 }, {    5,   31 }, {  -12,   79 }, {   -2,   45 },
    {  -15,   67 }, {  -11,   62 }, {   -3,   54 }, {   -4,   70 },
    {  -13,   68 }, {  -31,  100 }, {  -26,   91 }, {  -18,   84 },
    {   -9,   83 }, {  -30,  103 }, {  -24,   90 }, {  -43,  122 },
    {   -6,   66 }, {    4,   59 }, {   34,  -11 }, {   21,   10 },
    {   14,   25 }, {    5,   44 },
    {   -4,   69 }, {   40,  -33 }, {   19,    8 }, {  -11,   65 },
    {   -6,   59 }, {  -11,   68 }, {   12,   -2 }, {   17,  -10 },
    {   -6,   34 }, {   11,   14 }, {  -11,   70 }, {    1,   27 },
    {  -26,   71 }, {  -28,   79 }, {   -2,   47 }, {    6,   47 },
    {  -23,   67 }, {  -47,  107 }, {  -55,  117 }, {  -21,   83 },
    {    1,   59 }, {  -38,   82 }, {  -34,   77 }, {  -45,   95 },
    {   10,   25 }, {    9,   46 }, {   41,  -31 }, {   32,  -17 },
    {   14,   19 }, {   17,   18 }
  },
  {
    {   -2,   65 }, {   18,   22 }, {   10,   33 }, {   -2,   55 },
    {   -7,   64 }, {   -6,   67 }, {   19,   11 }, {   -5,   50 },
    {   -7,   53 }, {   -4,   54 }, {  -23,   99 }, {   -3,   51 },
    {    2,   41 }, {  -32,  102 }, {  -16,   79 }, {   -8,   77 },
    {  -21,   84 }, {  -26,   91 }, {  -33,  104 }, {   -4,   61 },
    {  -31,  122 }, {  -34,  110 }, {  -25,   96 }, {  -43,  124 },
    {   -6,   70 }, {    3,   60 }, {   23,   12 }, {   12,   30 },
    {   11,   33 }, {    8,   40 },
    {   -2,   66 }, {   40,  -20 }, {    0,   46 }, {   -5,   54 },
    {    7,   37 }, {  -39,  116 }, {   31,  -27 }, {    1,   22 },
    {  -35,   82 }, {  -32,   85 }, {  -15,   72 }, {   16,    0 },
    {  -43,  102 }, {  -75,  152 }, {   -8,   55 }, {   -9,   68 },
    {  -12,   54 }, {  -84,  171 }, {  -93,  186 }, {   25,   12 },
    { -104,  222 }, {  -40,   92 }, {  -51,   93 }, {  110, -169 },
    {    3,   52 }, {   17,   33 }, {   55,  -45 }, {   28,    1 },
    {   -5,   55 }, {   29,   -4 }
  }
};

static const Short
INIT_ABS_FLAG[3][2*NUM_ABS_FLAG_CTX][2] =
{
  {
    {  -12,   72 }, {  -10,   79 }, {  -11,   87 }, {  -14,   94 },
    {  -35,  136 }, {  -10,   58 }, {   -1,   54 }, {  -17,   86 },
    {   -5,   70 }, {  -22,  105 }, {  -13,   70 }, {   -2,   59 },
    {  -13,   81 }, {  -21,   96 }, {   -3,   73 }, {  -24,   90 },
    {  -19,   88 }, {   -1,   63 }, {    1,   60 }, {  -17,   97 },
    {   -7,   69 }, {  -20,   95 }, {  -11,   84 }, {  -27,  110 },
    {  -12,   96 }, {   -9,   72 }, {   -8,   76 }, {   -4,   75 },
    {   -3,   76 }, {   -9,   94 },
    {  -11,   65 }, {   -4,   66 }, {   -9,   82 }, {  -24,  107 },
    {  -24,  111 }, {  -18,   58 }, {  -10,   61 }, {  -19,   82 },
    {  -28,   97 }, {    6,   47 }, {  -12,   50 }, {  -17,   73 },
    {   -8,   66 }, {  -44,  115 }, {  -17,   86 }, {  -26,   74 },
    {   -4,   48 }, {  -43,  115 }, {  -58,  141 }, {  -51,  137 },
    {  -51,  117 }, {  -85,  176 }, {  -47,  120 }, {  -95,  202 },
    {  -65,  159 }, {   -2,   53 }, {    7,   43 }, {   -2,   67 },
    {    3,   62 }, {    4,   67 }
  },
  {
    {   -2,   49 }, {  -13,   81 }, {  -11,   82 }, {  -13,   88 },
    {  -24,  112 }, {   -4,   44 }, {  -17,   77 }, {  -16,   82 },
    {  -19,   89 }, {  -28,  110 }, {  -12,   64 }, {  -11,   70 },
    {  -18,   88 }, {   -6,   67 }, {  -19,   97 }, {  -17,   76 },
    {  -27,  100 }, {  -18,   88 }, {  -13,   84 }, {  -17,   94 },
    {  -11,   73 }, {  -15,   83 }, {  -10,   77 }, {  -10,   80 },
    {  -12,   91 }, {   -8,   63 }, {   -7,   71 }, {   -6,   73 },
    {   -8,   80 }, {   -9,   90 },
    {   14,   20 }, {   -7,   68 }, {   -4,   66 }, {   -8,   76 },
    {  -50,  148 }, {    2,   22 }, {   11,   15 }, {   -4,   49 },
    {  -41,  117 }, {  -60,  149 }, {  -12,   49 }, {   -6,   55 },
    {    1,   44 }, {   -5,   58 }, {  -12,   73 }, {   13,    9 },
    {    5,   26 }, {  -29,  101 }, {    0,   46 }, {  -23,   92 },
    {  -12,   57 }, {    6,   30 }, {   35,   -4 }, {   42,  -24 },
    {   22,   24 }, {   24,    0 }, {   11,   34 }, {    2,   61 },
    {   -5,   75 }, {    4,   62 }
  },
  {
    {    0,   43 }, {   -6,   65 }, {   -7,   71 }, {  -31,  117 },
    {  -25,  109 }, {  -20,   76 }, {  -14,   73 }, {  -20,   88 },
    {  -21,   92 }, {   -6,   71 }, {  -19,   73 }, {  -34,  108 },
    {  -27,  101 }, {   -7,   69 }, {   -7,   75 }, {  -18,   77 },
    {   -7,   64 }, {  -20,   91 }, {   -9,   75 }, {   -7,   78 },
    {  -26,   98 }, {  -13,   81 }, {   -6,   69 }, {  -12,   83 },
    {   -2,   70 }, {   -3,   50 }, {   -6,   66 }, {   -7,   73 },
    {   -6,   75 }, {  -11,   91 },
    {   19,   10 }, {    2,   49 }, {    4,   52 }, {   -4,   69 },
    {  -32,  114 }, {   13,   -1 }, {  -29,   85 }, {  -64,  143 },
    {  -90,  186 }, {   26,    4 }, {  -24,   68 }, {    8,   30 },
    {  -10,   61 }, {  -16,   78 }, {  -27,   96 }, {   31,  -16 },
    {  -12,   54 }, {  -15,   70 }, {  -68,  158 }, {   31,   12 },
    {   -8,   44 }, {  -32,   63 }, {  -36,   81 }, {  -53,  106 },
    {   36,   12 }, {   26,   -5 }, {   19,   17 }, {  -22,  101 },
    {    5,   54 }, {    8,   53 }
  }
};

// initial probability for motion vector predictor index
static const Short
INIT_MVP_IDX[3][NUM_MVP_IDX_CTX][2] =
{
  {
    {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   64 }, {    0,   64 }
  }
};

// initial probability for ALF flag
static const Short
INIT_ALF_FLAG[3][NUM_ALF_FLAG_CTX][2] =
{
  {
    {   50,  -48 }
  },
  {
    {   27,  -20 }
  },
  {
    {  -12,   68 }
  }
};

// initial probability for ALF side information (unsigned)
static const Short
INIT_ALF_UVLC[3][NUM_ALF_UVLC_CTX][2] =
{
  {
    {    1,   66 }, {   -3,   77 }
  },
  {
    {   -5,   75 }, {  -14,   94 }
  },
  {
    {   -5,   72 }, {  -30,  122 }
  }
};

// initial probability for ALF side information (signed)
static const Short
INIT_ALF_SVLC[3][NUM_ALF_SVLC_CTX][2] =
{
  {
    {   11,   57 }, {   -1,   62 }, {    0,   64 }
  },
  {
    {    6,   66 }, {   -1,   64 }, {    0,   64 }
  },
  {
    {    1,   73 }, {    2,   61 }, {    0,   64 }
  }
};
#if MTK_SAO
// initial probability for ALF flag
static const Short
INIT_AO_FLAG[3][NUM_AO_FLAG_CTX][2] =
{
  {
    {   50,  -48 }
  },
  {
    {   27,  -20 }
  },
  {
    {  -12,   68 }
  }
};

// initial probability for ALF side information (unsigned)
static const Short
INIT_AO_UVLC[3][NUM_AO_UVLC_CTX][2] =
{
  {
    {    1,   66 }, {   -3,   77 }
  },
  {
    {   -5,   75 }, {  -14,   94 }
  },
  {
    {   -5,   72 }, {  -30,  122 }
  }
};

// initial probability for ALF side information (signed)
static const Short
INIT_AO_SVLC[3][NUM_AO_SVLC_CTX][2] =
{
  {
    {   11,   57 }, {   -1,   62 }, {    0,   64 }
  },
  {
    {    6,   66 }, {   -1,   64 }, {    0,   64 }
  },
  {
    {    1,   73 }, {    2,   61 }, {    0,   64 }
  }
};
#endif

static const Short
INIT_TRANS_SUBDIV_FLAG[3][NUM_TRANS_SUBDIV_FLAG_CTX][2] =
{
  {
    {    0,    0 }, {   12,   12 }, {   22,    4 }, {   -2,   49 },
    {    4,   46 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }
  },
  {
    {    0,   13 }, {  -28,   89 }, {  -30,   99 }, {  -34,  106 },
    {  -19,   76 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }
  },
  {
    {  -11,   38 }, {  -31,   88 }, {  -42,  118 }, {  -47,  130 },
    {  -21,   73 }, {    0,   64 }, {    0,   64 }, {    0,   64 },
    {    0,   64 }, {    0,   64 }
  }
};

#endif
#endif
