/* The copyright in this software is being made available under the BSD
 * License, included below. This software may be subject to other third party
 * and contributor rights, including patent rights, and no such rights are
 * granted under this license.
 *
 * Copyright (c) 2010-2011, ITU/ISO/IEC
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *  * Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 *  * Neither the name of the ITU/ISO/IEC nor the names of its contributors may
 *    be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

/** \file     TDecSbac.cpp
    \brief    Context-adaptive entropy decoder class
*/

#include "TDecSbac.h"


#if HHI_SYS_PIPE
#if HHI_SYS_PIPE_LC_MODE

#define DECODE_NONTAB     0
#define LOAD_8BIN         0

#define PIPE_CW_NUM_BITS  32
#define PIPE_INIT_CW      ((UInt)(1<<(PIPE_CW_NUM_BITS-1)))
#define PIPE_INIT_CW64    (((UInt64)1<<(64-1)))
#define PIPE_2BIT_CW      ((UInt)(3<<(PIPE_CW_NUM_BITS-2)))
#define PIPE_2BIT_CW64    (((UInt64)3<<(64-2)))
#define PIPE_MASK_CW      ((UInt)0xffffffff)

#define SIG_BIN( _pos ) \
{ \
  const UInt uiBlkPos = *pauiSigScan++; \
  if( uiSymbol & ( 1 << ( 31 - _pos ) ) ) \
  { \
    pcCoef[ uiBlkPos ]++; \
    auiScanSig[ uiSCntSig++ ] = uiBlkPos; \
  } \
}

#define ABS_ONE( _pos ) \
{ \
  const UInt uiBlkPos = *pauiScan--; \
  if( uiSymbol & ( 1 << ( 31 - _pos ) ) ) \
  { \
    pcCoef[ uiBlkPos ]++; \
    auiScanOne[ uiSCntOne++ ] = uiBlkPos; \
  } \
}

#define ABS_TWO( _pos ) \
{ \
  const UInt uiBlkPos = *pauiScan++; \
  if( uiSymbol & ( 1 << ( 31 - _pos ) ) ) \
  { \
    pcCoef[ uiBlkPos ]++; \
    auiScanTwo[ uiSCntTwo++ ] = uiBlkPos; \
  } \
}

#define GET_BIN( _dst, _cnt ) \
{ \
  if( uiSymbol & (0x80000000) ) \
  { \
    pcCoef[ uiBlkPos ]++; \
    (_dst)[ (_cnt)++ ] = uiBlkPos; \
  } \
  uiSymbol <<= 1; \
}
#else
#define PIPE_CW_NUM_BITS  32
#define PIPE_INIT_CW      ((UInt)(1<<(PIPE_CW_NUM_BITS-1)))
#define PIPE_INIT_CW64    (((UInt64)1<<(64-1)))
#define PIPE_2BIT_CW      ((UInt)(3<<(PIPE_CW_NUM_BITS-2)))
#define PIPE_2BIT_CW64    (((UInt64)3<<(64-2)))
#define PIPE_MASK_CW      ((UInt)0xffffffff)
#endif
#endif


#if HHI_SYS_PIPE
UInt                  TDecSbac::m_auiCounter[ 8 ];
Bool                  TDecSbac::m_bAlfCtrl          = false;
UInt                  TDecSbac::m_uiMaxAlfCtrlDepth = false;
TComInputBitstream   *TDecSbac::m_pcBitstream       = 0;



UInt * const          TDecSbac::m_puiPipe0CwCnt      = m_auiCounter;
UInt * const          TDecSbac::m_puiPipe1CwCnt      = m_auiCounter + 1;
UInt * const          TDecSbac::m_puiPipe2CwCnt      = m_auiCounter + 2;
UInt * const          TDecSbac::m_puiPipe4CwCnt      = m_auiCounter + 4;
UInt * const          TDecSbac::m_puiPipe5CwCnt      = m_auiCounter + 5;
UInt * const          TDecSbac::m_puiPipe6CwCnt      = m_auiCounter + 6;
UInt * const          TDecSbac::m_puiPipe7CwCnt      = m_auiCounter + 7;


UInt (*TDecSbac::decodePipe1Bin[8])( const UChar ) =
{
  TDecSbac::xDecode1Bin0,
  TDecSbac::xDecode1Bin1,
  TDecSbac::xDecode1Bin2,
  TDecSbac::xDecode1Bin3,
  TDecSbac::xDecode1Bin4,
  TDecSbac::xDecode1Bin5,
  TDecSbac::xDecode1Bin6,
  TDecSbac::xDecode1Bin7
};

#if HHI_SYS_PIPE_LC_MODE
UInt (*TDecSbac::decodePipeNBin[8])( const UChar, const UInt ) =
{
  TDecSbac::xDecodeNBin0,
  TDecSbac::xDecodeNBin1,
  TDecSbac::xDecodeNBin2,
  TDecSbac::xDecodeNBin3,
  TDecSbac::xDecodeNBin4,
  TDecSbac::xDecodeNBin5,
  TDecSbac::xDecodeNBin6,
  TDecSbac::xDecodeNBin7
};
#endif
#endif //HHI_SYS_PIPE

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

TDecSbac::TDecSbac()
// new structure here
#if HHI_SYS_PIPE
: m_cCUSkipFlagSCModel        ( 1,             1,               NUM_SKIP_FLAG_CTX             )
#else
: m_pcBitstream               ( 0 )
, m_pcTDecBinIf               ( NULL )
, m_bAlfCtrl                  ( false )
, m_uiMaxAlfCtrlDepth         ( 0 )
, m_cCUSkipFlagSCModel        ( 1,             1,               NUM_SKIP_FLAG_CTX             )
#endif
, m_cCUSplitFlagSCModel       ( 1,             1,               NUM_SPLIT_FLAG_CTX            )
, m_cCUMergeFlagExtSCModel    ( 1,             1,               NUM_MERGE_FLAG_EXT_CTX        )
, m_cCUMergeIdxExtSCModel     ( 1,             1,               NUM_MERGE_IDX_EXT_CTX         )
, m_cCUAlfCtrlFlagSCModel     ( 1,             1,               NUM_ALF_CTRL_FLAG_CTX         )
, m_cCUPartSizeSCModel        ( 1,             1,               NUM_PART_SIZE_CTX             )
, m_cCUPredModeSCModel        ( 1,             1,               NUM_PRED_MODE_CTX             )
, m_cCUIntraPredSCModel       ( 1,             1,               NUM_ADI_CTX                   )
#if ADD_PLANAR_MODE
, m_cPlanarFlagSCModel        ( 1,             1,               NUM_PLANARFLAG_CTX            )
#endif
, m_cCUChromaPredSCModel      ( 1,             1,               NUM_CHROMA_PRED_CTX           )
, m_cCUInterDirSCModel        ( 1,             1,               NUM_INTER_DIR_CTX             )
, m_cCURefPicSCModel          ( 1,             1,               NUM_REF_NO_CTX                )
, m_cCUMvdSCModel             ( 1,             2,               NUM_MV_RES_CTX                )
, m_cCUTransSubdivFlagSCModel ( 1,             1,               NUM_TRANS_SUBDIV_FLAG_CTX     )
, m_cCUQtRootCbfSCModel       ( 1,             1,               NUM_QT_ROOT_CBF_CTX           )
, m_cCUDeltaQpSCModel         ( 1,             1,               NUM_DELTA_QP_CTX              )
, m_cCUQtCbfSCModel           ( 1,             3,               NUM_QT_CBF_CTX                )
#if SIMPLE_CONTEXT_SIG || HHI_SYS_PIPE_LC_MODE
, m_cCUSigSCModel             ( 1,             2,               NUM_SIG_FLAG_CTX              )
#else
, m_cCUSigSCModel             ( MAX_CU_DEPTH,  2,               NUM_SIG_FLAG_CTX              )
#endif

#if PCP_SIGMAP_SIMPLE_LAST
, m_cCuCtxLastX               ( 1,             2,               NUM_CTX_LAST_FLAG_XY          )
, m_cCuCtxLastY               ( 1,             2,               NUM_CTX_LAST_FLAG_XY          )
#else
, m_cCULastSCModel            ( MAX_CU_DEPTH,  2,               NUM_LAST_FLAG_CTX             )
#endif
, m_cCUOneSCModel             ( 1,             2,               NUM_ONE_FLAG_CTX              )
, m_cCUAbsSCModel             ( 1,             2,               NUM_ABS_FLAG_CTX              )
, m_cMVPIdxSCModel            ( 1,             1,               NUM_MVP_IDX_CTX               )
, m_cALFFlagSCModel           ( 1,             1,               NUM_ALF_FLAG_CTX              )
, m_cALFUvlcSCModel           ( 1,             1,               NUM_ALF_UVLC_CTX              )
, m_cALFSvlcSCModel           ( 1,             1,               NUM_ALF_SVLC_CTX              )
#if MTK_SAO
, m_cAOFlagSCModel            ( 1,             1,               NUM_AO_FLAG_CTX              )
, m_cAOUvlcSCModel            ( 1,             1,               NUM_AO_UVLC_CTX              )
, m_cAOSvlcSCModel            ( 1,             1,               NUM_AO_SVLC_CTX              )
#endif
{
#if FINE_GRANULARITY_SLICES && MTK_NONCROSS_INLOOP_FILTER
  m_iSliceGranularity = 0;
#endif
#if HHI_SYS_PIPE_LC_MODE
  m_initQp = -1;
  m_initSliceType = SliceType( -1 );
#endif
}

TDecSbac::~TDecSbac()
{
}

// ====================================================================================================================
// Public member functions
// ====================================================================================================================

Void TDecSbac::resetEntropy          (TComSlice* pcSlice)
{
  Int  iQp              = pcSlice->getSliceQp();
  SliceType eSliceType  = pcSlice->getSliceType();
#if HHI_SYS_PIPE_LC_MODE
  if( m_initQp != iQp || m_initSliceType != eSliceType )
  {
    m_initQp = iQp;
    m_initSliceType = eSliceType;
#endif

#if HHI_SYS_PIPE_8BIT_INIT
  INIT_CONTEXT_MODEL_DEC( m_cCUSkipFlagSCModel,        INIT_SKIP_FLAG )
  INIT_CONTEXT_MODEL_DEC( m_cCUSplitFlagSCModel,       INIT_SPLIT_FLAG )
  INIT_CONTEXT_MODEL_DEC( m_cCUSkipFlagSCModel,        INIT_SKIP_FLAG )
  INIT_CONTEXT_MODEL_DEC( m_cCUMergeFlagExtSCModel,    INIT_MERGE_FLAG_EXT )
  INIT_CONTEXT_MODEL_DEC( m_cCUMergeIdxExtSCModel,     INIT_MERGE_IDX_EXT )
  INIT_CONTEXT_MODEL_DEC( m_cCUAlfCtrlFlagSCModel,     INIT_ALF_CTRL_FLAG )
  INIT_CONTEXT_MODEL_DEC( m_cCUPartSizeSCModel,        INIT_PART_SIZE )
  INIT_CONTEXT_MODEL_DEC( m_cCUPredModeSCModel,        INIT_PRED_MODE )
  INIT_CONTEXT_MODEL_DEC( m_cCUIntraPredSCModel,       INIT_INTRA_PRED_MODE )
#if ADD_PLANAR_MODE
  INIT_CONTEXT_MODEL_DEC( m_cPlanarFlagSCModel,        INIT_PLANARFLAG )
#endif
  INIT_CONTEXT_MODEL_DEC( m_cCUChromaPredSCModel,      INIT_CHROMA_PRED_MODE )
  INIT_CONTEXT_MODEL_DEC( m_cCUInterDirSCModel,        INIT_INTER_DIR )
  INIT_CONTEXT_MODEL_DEC( m_cCUMvdSCModel,             INIT_MVD )
  INIT_CONTEXT_MODEL_DEC( m_cCURefPicSCModel,          INIT_REF_PIC )
  INIT_CONTEXT_MODEL_DEC( m_cCUDeltaQpSCModel,         INIT_DQP )
  INIT_CONTEXT_MODEL_DEC( m_cCUQtCbfSCModel,           INIT_QT_CBF )
  INIT_CONTEXT_MODEL_DEC( m_cCUQtRootCbfSCModel,       INIT_QT_ROOT_CBF )
  INIT_CONTEXT_MODEL_DEC( m_cCUSigSCModel,             INIT_SIG_FLAG )
#if PCP_SIGMAP_SIMPLE_LAST
  INIT_CONTEXT_MODEL_DEC( m_cCuCtxLastX,               INIT_LAST_X )
  INIT_CONTEXT_MODEL_DEC( m_cCuCtxLastY,               INIT_LAST_Y )
#else
#error "no init values available"!
  INIT_CONTEXT_MODEL_DEC( m_cCULastSCModel,            INIT_LAST_FLAG )
#endif
  INIT_CONTEXT_MODEL_DEC( m_cCUOneSCModel,             INIT_ONE_FLAG )
  INIT_CONTEXT_MODEL_DEC( m_cCUAbsSCModel,             INIT_ABS_FLAG )
  INIT_CONTEXT_MODEL_DEC( m_cMVPIdxSCModel,            INIT_MVP_IDX )
  INIT_CONTEXT_MODEL_DEC( m_cALFFlagSCModel,           INIT_ALF_FLAG )
  INIT_CONTEXT_MODEL_DEC( m_cALFUvlcSCModel,           INIT_ALF_UVLC )
  INIT_CONTEXT_MODEL_DEC( m_cALFSvlcSCModel,           INIT_ALF_SVLC )
#if MTK_SAO
  INIT_CONTEXT_MODEL_DEC( m_cAOFlagSCModel,            INIT_AO_FLAG )
  INIT_CONTEXT_MODEL_DEC( m_cAOUvlcSCModel,            INIT_AO_UVLC )
  INIT_CONTEXT_MODEL_DEC( m_cAOSvlcSCModel,            INIT_AO_SVLC )
#endif
  INIT_CONTEXT_MODEL_DEC( m_cCUTransSubdivFlagSCModel, INIT_TRANS_SUBDIV_FLAG )
#else
  m_cCUSplitFlagSCModel.initBuffer       ( eSliceType, iQp, (Short*)INIT_SPLIT_FLAG );
  m_cCUSkipFlagSCModel.initBuffer        ( eSliceType, iQp, (Short*)INIT_SKIP_FLAG );
  m_cCUMergeFlagExtSCModel.initBuffer    ( eSliceType, iQp, (Short*)INIT_MERGE_FLAG_EXT );
  m_cCUMergeIdxExtSCModel.initBuffer     ( eSliceType, iQp, (Short*)INIT_MERGE_IDX_EXT );
  m_cCUAlfCtrlFlagSCModel.initBuffer     ( eSliceType, iQp, (Short*)INIT_ALF_CTRL_FLAG );
  m_cCUPartSizeSCModel.initBuffer        ( eSliceType, iQp, (Short*)INIT_PART_SIZE );
  m_cCUPredModeSCModel.initBuffer        ( eSliceType, iQp, (Short*)INIT_PRED_MODE );
  m_cCUIntraPredSCModel.initBuffer       ( eSliceType, iQp, (Short*)INIT_INTRA_PRED_MODE );
#if ADD_PLANAR_MODE
  m_cPlanarFlagSCModel.initBuffer        ( eSliceType, iQp, (Short*)INIT_PLANARFLAG );
#endif
  m_cCUChromaPredSCModel.initBuffer      ( eSliceType, iQp, (Short*)INIT_CHROMA_PRED_MODE );
  m_cCUInterDirSCModel.initBuffer        ( eSliceType, iQp, (Short*)INIT_INTER_DIR );
  m_cCUMvdSCModel.initBuffer             ( eSliceType, iQp, (Short*)INIT_MVD );
  m_cCURefPicSCModel.initBuffer          ( eSliceType, iQp, (Short*)INIT_REF_PIC );
  m_cCUDeltaQpSCModel.initBuffer         ( eSliceType, iQp, (Short*)INIT_DQP );
  m_cCUQtCbfSCModel.initBuffer           ( eSliceType, iQp, (Short*)INIT_QT_CBF );
  m_cCUQtRootCbfSCModel.initBuffer       ( eSliceType, iQp, (Short*)INIT_QT_ROOT_CBF );
  m_cCUSigSCModel.initBuffer             ( eSliceType, iQp, (Short*)INIT_SIG_FLAG );
#if PCP_SIGMAP_SIMPLE_LAST
  m_cCuCtxLastX.initBuffer               ( eSliceType, iQp, (Short*)INIT_LAST_X );
  m_cCuCtxLastY.initBuffer               ( eSliceType, iQp, (Short*)INIT_LAST_Y );
#else
  m_cCULastSCModel.initBuffer            ( eSliceType, iQp, (Short*)INIT_LAST_FLAG );
#endif
  m_cCUOneSCModel.initBuffer             ( eSliceType, iQp, (Short*)INIT_ONE_FLAG );
  m_cCUAbsSCModel.initBuffer             ( eSliceType, iQp, (Short*)INIT_ABS_FLAG );
  m_cMVPIdxSCModel.initBuffer            ( eSliceType, iQp, (Short*)INIT_MVP_IDX );
  m_cALFFlagSCModel.initBuffer           ( eSliceType, iQp, (Short*)INIT_ALF_FLAG );
  m_cALFUvlcSCModel.initBuffer           ( eSliceType, iQp, (Short*)INIT_ALF_UVLC );
  m_cALFSvlcSCModel.initBuffer           ( eSliceType, iQp, (Short*)INIT_ALF_SVLC );
#if MTK_SAO
  m_cAOFlagSCModel.initBuffer           ( eSliceType, iQp, (Short*)INIT_AO_FLAG );
  m_cAOUvlcSCModel.initBuffer           ( eSliceType, iQp, (Short*)INIT_AO_UVLC );
  m_cAOSvlcSCModel.initBuffer           ( eSliceType, iQp, (Short*)INIT_AO_SVLC );
#endif
  m_cCUTransSubdivFlagSCModel.initBuffer ( eSliceType, iQp, (Short*)INIT_TRANS_SUBDIV_FLAG );

#if !HHI_SYS_PIPE
  m_uiLastDQpNonZero  = 0;

  // new structure
  m_uiLastQp          = iQp;
#endif
#endif

#if HHI_SYS_PIPE_LC_MODE
  }
#endif

#if HHI_SYS_PIPE
  xStart();
#else
  m_pcTDecBinIf->start();
#endif
}


#if HHI_SYS_PIPE
Void
TDecSbac::parseTerminatingBit( UInt& ruiBit )
{
  ruiBit = xDecodeBinTrm();
  return;
}
#else
Void TDecSbac::parseTerminatingBit( UInt& ruiBit )
{
  m_pcTDecBinIf->decodeBinTrm( ruiBit );
}
#endif

#if HHI_SYS_PIPE
UInt
#if HHI_SYS_PIPE_LC_MODE
TDecSbac::xReadUnaryMaxSymbol( const ContextModel* pcSCModel, const Int iOffset, const UInt uiMaxSymbol )
#else
TDecSbac::xReadUnaryMaxSymbol( ContextModel* pcSCModel, const Int iOffset, const UInt uiMaxSymbol )
#endif
{
  if( uiMaxSymbol == 0 )
  {
    return 0;
  }

  UInt uiSymbol = xDecodeBin( *pcSCModel );

  if( uiSymbol == 0 || uiMaxSymbol == 1 )
  {
    return uiSymbol;
  }

  uiSymbol           = 0;
  UInt uiCount       = 0;
  const UInt uiMaxM1 = uiMaxSymbol - 1;
  pcSCModel         += iOffset;

  do
  {
    uiCount = xDecodeBin( *pcSCModel );
    uiSymbol++;
  }
  while( uiCount && uiSymbol < uiMaxM1 );

  if( uiCount && ( uiSymbol == uiMaxM1 ) )
  {
    uiSymbol++;
  }

  return uiSymbol;
}
#else
Void TDecSbac::xReadUnaryMaxSymbol( UInt& ruiSymbol, ContextModel* pcSCModel, Int iOffset, UInt uiMaxSymbol )
{
  if (uiMaxSymbol == 0)
  {
    ruiSymbol = 0;
    return;
  }

  m_pcTDecBinIf->decodeBin( ruiSymbol, pcSCModel[0] );

  if( ruiSymbol == 0 || uiMaxSymbol == 1 )
  {
    return;
  }

  UInt uiSymbol = 0;
  UInt uiCont;

  do
  {
    m_pcTDecBinIf->decodeBin( uiCont, pcSCModel[ iOffset ] );
    uiSymbol++;
  }
  while( uiCont && ( uiSymbol < uiMaxSymbol - 1 ) );

  if( uiCont && ( uiSymbol == uiMaxSymbol - 1 ) )
  {
    uiSymbol++;
  }

  ruiSymbol = uiSymbol;
}
#endif

#if HHI_SYS_PIPE
Int
TDecSbac::xReadMvd( const UInt uiAbsSumL, const UInt uiAbsSumA, const UInt uiCtx )
{
   Int uiMvdComp  = 0;
#if HHI_NO_GET_CTX_CALL
  ContextModel *pCtx = m_cCUMvdSCModel.get( 0, uiCtx );
  UInt uiSymbol = xDecodeBin( *( pCtx ) );
#else
  UInt uiLocalCtx = 0;
  ContextModel *pCtx = m_cCUMvdSCModel.get( 0, uiCtx );

  uiLocalCtx += ( uiAbsSumA > 16 ) ? 1 : 0;
  uiLocalCtx += ( uiAbsSumL > 16 ) ? 1 : 0;

  UInt uiSymbol = xDecodeBin( *( pCtx + uiLocalCtx ) );
#endif
  if( !uiSymbol )
  {
    return 0;
  }

  uiSymbol = xReadExGolombMvd( pCtx + 3, 3 );
  uiSymbol++;

  const UInt uiSign = xDecodeEPBin();

  uiMvdComp = uiSign ? -(Int)uiSymbol : (Int)uiSymbol;

  return uiMvdComp;
}
#else
#if MVD_CTX
/** Decode a motion vector difference
 * \param riMvdComp motion vector difference
 * \param uiAbsSumL motion vector difference of left PU
 * \param uiAbsSumA motion vector difference of above PU
 * \param uiCtx index for context set based on vertical or horizontal component
 */
Void TDecSbac::xReadMvd( Int& riMvdComp, UInt uiAbsSumL, UInt uiAbsSumA, UInt uiCtx )
#else
Void TDecSbac::xReadMvd( Int& riMvdComp, UInt uiAbsSum, UInt uiCtx )
#endif
{
  UInt uiLocalCtx = 0;

#if MVD_CTX
  uiLocalCtx += (uiAbsSumA>16) ? 1 : 0;
  uiLocalCtx += (uiAbsSumL>16) ? 1 : 0;
#else
  if( uiAbsSum >= 3 )
  {
    uiLocalCtx += ( uiAbsSum > 32) ? 2 : 1;
  }
#endif

  riMvdComp = 0;

  UInt uiSymbol;
  m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUMvdSCModel.get( 0, uiCtx, uiLocalCtx ) );

  if (!uiSymbol)
  {
    return;
  }

  xReadExGolombMvd( uiSymbol, &m_cCUMvdSCModel.get( 0, uiCtx, 3 ), 3 );
  uiSymbol++;

  UInt uiSign;
  m_pcTDecBinIf->decodeBinEP( uiSign );

  riMvdComp = ( 0 != uiSign ) ? -(Int)uiSymbol : (Int)uiSymbol;

  return;
}
#endif

#if HHI_SYS_PIPE
UInt
#if HHI_SYS_PIPE_LC_MODE
TDecSbac::xReadExGolombMvd( const ContextModel* pcSCModel, const UInt uiMaxBin )
#else
TDecSbac::xReadExGolombMvd( ContextModel* pcSCModel, const UInt uiMaxBin )
#endif
{
  UInt uiMvd    = 0;
  UInt uiSymbol = xDecodeBin( *pcSCModel++ );

  if( !uiSymbol )
  {
    return 0;
  }

  uiSymbol = xDecodeBin( *pcSCModel );

  if( !uiSymbol )
  {
    return 1;
  }

  pcSCModel++;
  UInt uiCount = 2;

  do
  {
    if( uiMaxBin == uiCount )
    {
      pcSCModel++;
    }
    uiSymbol = xDecodeBin( *pcSCModel );
    uiCount++;
  }
  while( uiSymbol && ( uiCount != 8 ) );

  uiMvd = uiCount - 1;

  if( uiSymbol )
  {
    uiSymbol = xReadEpExGolomb( 3 ) + 1;
    uiMvd   += uiSymbol;
  }
  return uiMvd;
}
#else
Void TDecSbac::xReadExGolombMvd( UInt& ruiSymbol, ContextModel* pcSCModel, UInt uiMaxBin )
{
  UInt uiSymbol;

  m_pcTDecBinIf->decodeBin( ruiSymbol, pcSCModel[0] );

  if( !ruiSymbol )
  {
    return;
  }

  m_pcTDecBinIf->decodeBin( uiSymbol, pcSCModel[1] );

  ruiSymbol = 1;

  if( !uiSymbol )
  {
    return;
  }

  pcSCModel += 2;
  UInt uiCount = 2;

  do
  {
    if( uiMaxBin == uiCount )
    {
      pcSCModel++;
    }
    m_pcTDecBinIf->decodeBin( uiSymbol, *pcSCModel );
    uiCount++;
  }
  while( uiSymbol && ( uiCount != 8 ) );

  ruiSymbol = uiCount - 1;

  if( uiSymbol )
  {
    xReadEpExGolomb( uiSymbol, 3 );
    ruiSymbol += uiSymbol + 1;
  }

  return;
}
#endif

#if HHI_SYS_PIPE
UInt
TDecSbac::xReadEpExGolomb( UInt uiCount )
{
  UInt uiBit    = 1;
  UInt uiSymbol = 0;

  while( uiBit )
  {
    uiBit = xDecodeEPBin();
    uiSymbol += uiBit << uiCount++;
  }

  uiCount--;
#if HHI_SYS_PIPE_LC_MODE
  if( uiCount )
  {
    uiSymbol += xDecodeEPBin( uiCount );
  }
#else
  while( uiCount-- )
  {
    uiBit = xDecodeEPBin();
    uiSymbol += uiBit << uiCount;
  }
#endif

  return uiSymbol;
}
#else
Void TDecSbac::xReadEpExGolomb( UInt& ruiSymbol, UInt uiCount )
{
  UInt uiSymbol = 0;
  UInt uiBit = 1;

  while( uiBit )
  {
    m_pcTDecBinIf->decodeBinEP( uiBit );
    uiSymbol += uiBit << uiCount++;
  }

  uiCount--;
  while( uiCount-- )
  {
    m_pcTDecBinIf->decodeBinEP( uiBit );
    uiSymbol += uiBit << uiCount;
  }

  ruiSymbol = uiSymbol;

  return;
}
#endif

#if HHI_SYS_PIPE
UInt
#if HHI_SYS_PIPE_LC_MODE
TDecSbac::xReadUnarySymbol( const ContextModel* pcSCModel, const Int iOffset )
#else
TDecSbac::xReadUnarySymbol( ContextModel* pcSCModel, const Int iOffset )
#endif
{
  UInt uiSymbol = xDecodeBin( *pcSCModel );

  if( !uiSymbol )
  {
    return 0;
  }

  uiSymbol     = 0;
  UInt uiCount = 0;
  pcSCModel   += iOffset;

  do
  {
    uiCount = xDecodeBin( *pcSCModel );
    uiSymbol++;
  }
  while( uiCount );

  return uiSymbol;
}
#else
Void TDecSbac::xReadUnarySymbol( UInt& ruiSymbol, ContextModel* pcSCModel, Int iOffset )
{
  m_pcTDecBinIf->decodeBin( ruiSymbol, pcSCModel[0] );

  if( !ruiSymbol )
  {
    return;
  }

  UInt uiSymbol = 0;
  UInt uiCont;

  do
  {
    m_pcTDecBinIf->decodeBin( uiCont, pcSCModel[ iOffset ] );
    uiSymbol++;
  }
  while( uiCont );

  ruiSymbol = uiSymbol;
}
#endif

#if HHI_SYS_PIPE
#if HHI_SYS_PIPE_LC_MODE
Void
TDecSbac::xReadGoRiceExGolomb( TCoeff* pcCoef, const UInt* pauiScan, const UInt uiScanPos )
{
  UInt uiRice = 0;
  UInt uiLeft = uiScanPos;

  while( uiLeft-- )
  {
    const UInt uiBlkPos     = *pauiScan++;
    const UInt uiMaxVlc     = g_auiGoRiceRange[ uiRice ] + 1;
    const UInt uiMaxPreLen  = g_auiGoRicePrefixLen[ uiRice ];
    UInt       uiCodeWord   = 0;
    UInt       uiQuotient   = 0;

    do
    {
      uiQuotient++;
      uiCodeWord = xDecodeEPBin();
    }
    while( uiCodeWord && uiQuotient < uiMaxPreLen );

    uiQuotient            -= 1-uiCodeWord;
    const UInt uiRemainder = uiRice > 0 ? xDecodeEPBin( uiRice ) : 0;
    const UInt uiPrefix    = uiRemainder + ( uiQuotient << uiRice );
    const UInt uiSymbol    = uiPrefix + ( uiPrefix == uiMaxVlc ? xReadEpExGolomb( 0 ) : 0 );

    if( uiSymbol )
    {
      pcCoef[ uiBlkPos ] = uiSymbol + 3;
    }

    uiRice = g_aauiGoRiceUpdate[ uiRice ][ uiSymbol > 15 ? 15 : uiSymbol ];
  }
}
#else
Void TDecSbac::xReadGoRiceExGolomb( UInt &ruiSymbol, UInt &ruiGoRiceParam )
{
  Bool bExGolomb    = false;
  UInt uiCodeWord   = 0;
  UInt uiQuotient   = 0;
  UInt uiRemainder  = 0;
  UInt uiMaxVlc     = g_auiGoRiceRange[ ruiGoRiceParam ];
  UInt uiMaxPreLen  = g_auiGoRicePrefixLen[ ruiGoRiceParam ];

  do
  {
    uiQuotient++;
    uiCodeWord = xDecodeEPBin();
  }
  while( uiCodeWord && uiQuotient < uiMaxPreLen );

  uiCodeWord  = 1 - uiCodeWord;
  uiQuotient -= uiCodeWord;

  for( UInt ui = 0; ui < ruiGoRiceParam; ui++ )
  {
    uiCodeWord = xDecodeEPBin();
    if( uiCodeWord )
    {
      uiRemainder |= ( 1 << ( ruiGoRiceParam - ui - 1 ) );
    }
  }

  ruiSymbol      = uiRemainder + ( uiQuotient << ruiGoRiceParam );
  bExGolomb      = ruiSymbol == ( uiMaxVlc + 1 );

  if( bExGolomb )
  {
    uiCodeWord = xReadEpExGolomb( 0 );
    ruiSymbol += uiCodeWord;
  }

  ruiGoRiceParam = g_aauiGoRiceUpdate[ ruiGoRiceParam ][ min<UInt>( ruiSymbol, 15 ) ];

  return;
}
#endif
#else
#if E253
/** Parsing of coeff_abs_level_minus3
 * \param ruiSymbol reference to coeff_abs_level_minus3
 * \param ruiGoRiceParam reference to Rice parameter
 * \returns Void
 */
Void TDecSbac::xReadGoRiceExGolomb( UInt &ruiSymbol, UInt &ruiGoRiceParam )
{
  Bool bExGolomb    = false;
  UInt uiCodeWord   = 0;
  UInt uiQuotient   = 0;
  UInt uiRemainder  = 0;
  UInt uiMaxVlc     = g_auiGoRiceRange[ ruiGoRiceParam ];
  UInt uiMaxPreLen  = g_auiGoRicePrefixLen[ ruiGoRiceParam ];

  do
  {
    uiQuotient++;
    m_pcTDecBinIf->decodeBinEP( uiCodeWord );
  }
  while( uiCodeWord && uiQuotient < uiMaxPreLen );

  uiCodeWord  = 1 - uiCodeWord;
  uiQuotient -= uiCodeWord;

  for( UInt ui = 0; ui < ruiGoRiceParam; ui++ )
  {
    m_pcTDecBinIf->decodeBinEP( uiCodeWord );
    if( uiCodeWord )
    {
      uiRemainder += 1 << ui;
    }
  }

  ruiSymbol      = uiRemainder + ( uiQuotient << ruiGoRiceParam );
  bExGolomb      = ruiSymbol == ( uiMaxVlc + 1 );

  if( bExGolomb )
  {
    xReadEpExGolomb( uiCodeWord, 0 );
    ruiSymbol += uiCodeWord;
  }

  ruiGoRiceParam = g_aauiGoRiceUpdate[ ruiGoRiceParam ][ min<UInt>( ruiSymbol, 15 ) ];

  return;
}
#else
Void TDecSbac::xReadExGolombLevel( UInt& ruiSymbol, ContextModel& rcSCModel  )
{
  UInt uiSymbol;
  UInt uiCount = 0;
  do
  {
    m_pcTDecBinIf->decodeBin( uiSymbol, rcSCModel );
    uiCount++;
  }
  while( uiSymbol && ( uiCount != 13 ) );

  ruiSymbol = uiCount - 1;

  if( uiSymbol )
  {
    xReadEpExGolomb( uiSymbol, 0 );
    ruiSymbol += uiSymbol + 1;
  }

  return;
}
#endif
#endif

#if E057_INTRA_PCM
/** Parse I_PCM information.
 * \param pcCU
 * \param uiAbsPartIdx
 * \param uiDepth
 * \returns Void
 *
 * If I_PCM flag indicates that the CU is I_PCM, parse its PCM alignment bits and codes.
 */
Void TDecSbac::parseIPCMInfo ( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
  UInt uiSymbol;

  m_pcTDecBinIf->decodeBinTrm(uiSymbol);

  if (uiSymbol)
  {
    Bool bIpcmFlag = true;

    m_pcTDecBinIf->decodePCMAlignBits();

    pcCU->setPartSizeSubParts  ( SIZE_2Nx2N, uiAbsPartIdx, uiDepth );
    pcCU->setSizeSubParts      ( g_uiMaxCUWidth>>uiDepth, g_uiMaxCUHeight>>uiDepth, uiAbsPartIdx, uiDepth );
    pcCU->setIPCMFlagSubParts  ( bIpcmFlag, uiAbsPartIdx, uiDepth );

    UInt uiMinCoeffSize = pcCU->getPic()->getMinCUWidth()*pcCU->getPic()->getMinCUHeight();
    UInt uiLumaOffset   = uiMinCoeffSize*uiAbsPartIdx;
    UInt uiChromaOffset = uiLumaOffset>>2;

    Pel* piPCMSample;
    UInt uiWidth;
    UInt uiHeight;
    UInt uiSampleBits;
    UInt uiX, uiY;

    piPCMSample = pcCU->getPCMSampleY() + uiLumaOffset;
    uiWidth = pcCU->getWidth(uiAbsPartIdx);
    uiHeight = pcCU->getHeight(uiAbsPartIdx);
#if E192_SPS_PCM_BIT_DEPTH_SYNTAX
    uiSampleBits = pcCU->getSlice()->getSPS()->getPCMBitDepthLuma();
#else
    uiSampleBits = g_uiBitDepth;
#endif

    for(uiY = 0; uiY < uiHeight; uiY++)
    {
      for(uiX = 0; uiX < uiWidth; uiX++)
      {
        UInt uiSample;
        m_pcTDecBinIf->xReadPCMCode(uiSampleBits, uiSample);
        piPCMSample[uiX] = uiSample;
      }
      piPCMSample += uiWidth;
    }

    piPCMSample = pcCU->getPCMSampleCb() + uiChromaOffset;
    uiWidth = pcCU->getWidth(uiAbsPartIdx)/2;
    uiHeight = pcCU->getHeight(uiAbsPartIdx)/2;
#if E192_SPS_PCM_BIT_DEPTH_SYNTAX
    uiSampleBits = pcCU->getSlice()->getSPS()->getPCMBitDepthChroma();
#else
    uiSampleBits = g_uiBitDepth;
#endif

    for(uiY = 0; uiY < uiHeight; uiY++)
    {
      for(uiX = 0; uiX < uiWidth; uiX++)
      {
        UInt uiSample;
        m_pcTDecBinIf->xReadPCMCode(uiSampleBits, uiSample);
        piPCMSample[uiX] = uiSample;
      }
      piPCMSample += uiWidth;
    }

    piPCMSample = pcCU->getPCMSampleCr() + uiChromaOffset;
    uiWidth = pcCU->getWidth(uiAbsPartIdx)/2;
    uiHeight = pcCU->getHeight(uiAbsPartIdx)/2;
#if E192_SPS_PCM_BIT_DEPTH_SYNTAX
    uiSampleBits = pcCU->getSlice()->getSPS()->getPCMBitDepthChroma();
#else
    uiSampleBits = g_uiBitDepth;
#endif

    for(uiY = 0; uiY < uiHeight; uiY++)
    {
      for(uiX = 0; uiX < uiWidth; uiX++)
      {
        UInt uiSample;
        m_pcTDecBinIf->xReadPCMCode(uiSampleBits, uiSample);
        piPCMSample[uiX] = uiSample;
      }
      piPCMSample += uiWidth;
    }

    m_pcTDecBinIf->resetBac();
  }
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseAlfCtrlDepth( UInt& ruiAlfCtrlDepth )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cALFUvlcSCModel.get();
#else
  ContextModel *pCtx = m_cALFUvlcSCModel.get();
#endif
  ruiAlfCtrlDepth = xReadUnaryMaxSymbol( pCtx, 1, g_uiMaxCUDepth - 1 );
  return;
}
#else
Void TDecSbac::parseAlfCtrlDepth( UInt& ruiAlfCtrlDepth )
{
  UInt uiSymbol;
  xReadUnaryMaxSymbol( uiSymbol, m_cALFUvlcSCModel.get( 0 ), 1, g_uiMaxCUDepth - 1 );
  ruiAlfCtrlDepth = uiSymbol;
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseAlfCtrlFlag( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
  if( !m_bAlfCtrl )
  {
    return;
  }

  if( uiDepth > m_uiMaxAlfCtrlDepth && !pcCU->isFirstAbsZorderIdxInDepth( uiAbsPartIdx, m_uiMaxAlfCtrlDepth ) )
  {
    return;
  }

#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cCUAlfCtrlFlagSCModel.get();
#else
  ContextModel *pCtx = m_cCUAlfCtrlFlagSCModel.get();
#endif
#if HHI_NO_GET_CTX_CALL
  const UInt uiSymbol = xDecodeBin( *pCtx );
#else
  const UInt uiSymbol = xDecodeBin( *( pCtx + pcCU->getCtxAlfCtrlFlag( uiAbsPartIdx ) ) );
#endif

  if( uiDepth > m_uiMaxAlfCtrlDepth )
  {
    pcCU->setAlfCtrlFlagSubParts( uiSymbol, uiAbsPartIdx, m_uiMaxAlfCtrlDepth );
  }
  else
  {
    pcCU->setAlfCtrlFlagSubParts( uiSymbol, uiAbsPartIdx, uiDepth );
  }
  return;
}
#else
Void TDecSbac::parseAlfCtrlFlag( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
  if( !m_bAlfCtrl )
  {
    return;
  }

  if( uiDepth > m_uiMaxAlfCtrlDepth && !pcCU->isFirstAbsZorderIdxInDepth( uiAbsPartIdx, m_uiMaxAlfCtrlDepth ) )
  {
    return;
  }

  UInt uiSymbol;
  m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUAlfCtrlFlagSCModel.get( 0, 0, pcCU->getCtxAlfCtrlFlag( uiAbsPartIdx ) ) );
  if( uiDepth > m_uiMaxAlfCtrlDepth )
  {
    pcCU->setAlfCtrlFlagSubParts( uiSymbol, uiAbsPartIdx, m_uiMaxAlfCtrlDepth );
  }
  else
  {
    pcCU->setAlfCtrlFlagSubParts( uiSymbol, uiAbsPartIdx, uiDepth );
  }
}
#endif

/** parse skip flag
 * \param pcCU
 * \param uiAbsPartIdx
 * \param uiDepth
 * \returns Void
 */
#if HHI_SYS_PIPE
Void
TDecSbac::parseSkipFlag( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
  if( pcCU->getSlice()->isIntra() )
  {
    return;
  }

#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cCUSkipFlagSCModel.get();
#else
  ContextModel *pCtx = m_cCUSkipFlagSCModel.get();
#endif
#if HHI_NO_GET_CTX_CALL
  const UInt uiSymbol = xDecodeBin( *pCtx );
#else
  const UInt uiSymbol = xDecodeBin( *( pCtx + pcCU->getCtxSkipFlag( uiAbsPartIdx ) ) );
#endif

  //DTRACE_CABAC_V( g_nSymbolCounter++ );
  //DTRACE_CABAC_T( "\tSkipFlag" );
  //DTRACE_CABAC_T( "\tuiCtxSkip: ");
  //DTRACE_CABAC_V( uiCtxSkip );
  //DTRACE_CABAC_T( "\tuiSymbol: ");
  //DTRACE_CABAC_V( uiSymbol );
  //DTRACE_CABAC_T( "\n");

  if( uiSymbol )
  {
    pcCU->setPredModeSubParts( MODE_SKIP,  uiAbsPartIdx, uiDepth );
    pcCU->setPartSizeSubParts( SIZE_2Nx2N, uiAbsPartIdx, uiDepth );
    pcCU->setSizeSubParts( g_uiMaxCUWidth>>uiDepth, g_uiMaxCUHeight>>uiDepth, uiAbsPartIdx, uiDepth );
    pcCU->setMergeFlagSubParts( true , uiAbsPartIdx, 0, uiDepth );
  }
  return;
}
#else
Void TDecSbac::parseSkipFlag( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
  if( pcCU->getSlice()->isIntra() )
  {
    return;
  }

  UInt uiSymbol = 0;
  UInt uiCtxSkip = pcCU->getCtxSkipFlag( uiAbsPartIdx );
  m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUSkipFlagSCModel.get( 0, 0, uiCtxSkip ) );
  DTRACE_CABAC_V( g_nSymbolCounter++ );
  DTRACE_CABAC_T( "\tSkipFlag" );
  DTRACE_CABAC_T( "\tuiCtxSkip: ");
  DTRACE_CABAC_V( uiCtxSkip );
  DTRACE_CABAC_T( "\tuiSymbol: ");
  DTRACE_CABAC_V( uiSymbol );
  DTRACE_CABAC_T( "\n");

  if( uiSymbol )
  {
    pcCU->setPredModeSubParts( MODE_SKIP,  uiAbsPartIdx, uiDepth );
    pcCU->setPartSizeSubParts( SIZE_2Nx2N, uiAbsPartIdx, uiDepth );
    pcCU->setSizeSubParts( g_uiMaxCUWidth>>uiDepth, g_uiMaxCUHeight>>uiDepth, uiAbsPartIdx, uiDepth );

#if HHI_MRG_SKIP
    if ( pcCU->getSlice()->getSPS()->getUseMRG() )
    {
    pcCU->setMergeFlagSubParts( true , uiAbsPartIdx, 0, uiDepth );
    }
    else
#endif // HHI_MRG_SKIP
    {
      TComMv cZeroMv(0,0);
      pcCU->getCUMvField( REF_PIC_LIST_0 )->setAllMvd    ( cZeroMv, SIZE_2Nx2N, uiAbsPartIdx, uiDepth );
      pcCU->getCUMvField( REF_PIC_LIST_1 )->setAllMvd    ( cZeroMv, SIZE_2Nx2N, uiAbsPartIdx, uiDepth );

      pcCU->setTrIdxSubParts( 0, uiAbsPartIdx, uiDepth );
      pcCU->setCbfSubParts  ( 0, 0, 0, uiAbsPartIdx, uiDepth );

      if ( pcCU->getSlice()->isInterP() )
      {
        pcCU->setInterDirSubParts( 1, uiAbsPartIdx, 0, uiDepth );

        if ( pcCU->getSlice()->getNumRefIdx( REF_PIC_LIST_0 ) > 0 )
          pcCU->getCUMvField( REF_PIC_LIST_0 )->setAllRefIdx(  0, SIZE_2Nx2N, uiAbsPartIdx, uiDepth );
        if ( pcCU->getSlice()->getNumRefIdx( REF_PIC_LIST_1 ) > 0 )
          pcCU->getCUMvField( REF_PIC_LIST_1 )->setAllRefIdx( NOT_VALID, SIZE_2Nx2N, uiAbsPartIdx, uiDepth );
      }
      else
      {
        pcCU->setInterDirSubParts( 3, uiAbsPartIdx, 0, uiDepth );

        if ( pcCU->getSlice()->getNumRefIdx( REF_PIC_LIST_0 ) > 0 )
          pcCU->getCUMvField( REF_PIC_LIST_0 )->setAllRefIdx(  0, SIZE_2Nx2N, uiAbsPartIdx, uiDepth );
        if ( pcCU->getSlice()->getNumRefIdx( REF_PIC_LIST_1 ) > 0 )
          pcCU->getCUMvField( REF_PIC_LIST_1 )->setAllRefIdx( 0, SIZE_2Nx2N, uiAbsPartIdx, uiDepth );
      }
    }
  }
}
#endif

/** parse merge flag
 * \param pcCU
 * \param uiAbsPartIdx
 * \param uiDepth
 * \param uiPUIdx
 * \returns Void
 */
#if HHI_SYS_PIPE
Void
TDecSbac::parseMergeFlag ( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth, UInt uiPUIdx )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cCUMergeFlagExtSCModel.get();
#else
  ContextModel *pCtx = m_cCUMergeFlagExtSCModel.get();
#endif

#if HHI_NO_GET_CTX_CALL
  const UInt uiSymbol = xDecodeBin( *pCtx );
#else
#if CHANGE_MERGE_CONTEXT
  const UInt uiCtx = pcCU->getCtxMergeFlag( uiAbsPartIdx );
#else
  UInt uiCtx = 0;

  for(UInt uiIter = 0; uiIter < MRG_MAX_NUM_CANDS; uiIter++ )
  {
    if( pcCU->getNeighbourCandIdx( uiIter, uiAbsPartIdx ) == uiIter + 1 )
    {
      if( uiIter == 0 )
      {
        uiCtx++;
      }
      else if( uiIter == 1 )
      {
        uiCtx++;
      }
    }
  }
#endif
  const UInt uiSymbol = xDecodeBin( *( pCtx + uiCtx ) );
#endif
  pcCU->setMergeFlagSubParts( uiSymbol ? true : false, uiAbsPartIdx, uiPUIdx, uiDepth );

//  DTRACE_CABAC_V( g_nSymbolCounter++ );
//  DTRACE_CABAC_T( "\tMergeFlag: " );
//  DTRACE_CABAC_V( uiSymbol );
//  DTRACE_CABAC_T( "\tAddress: " );
//  DTRACE_CABAC_V( pcCU->getAddr() );
//  DTRACE_CABAC_T( "\tuiAbsPartIdx: " );
//  DTRACE_CABAC_V( uiAbsPartIdx );
//#if ENC_DEC_TRACE
//  for( UInt ui = 0; ui < MRG_MAX_NUM_CANDS; ui++ )
//  {
//    DTRACE_CABAC_T( "\tNumMrgCand: " );
//    DTRACE_CABAC_V( ui );
//    DTRACE_CABAC_T( "\t==\t" );
//    DTRACE_CABAC_V( UInt( pcCU->getNeighbourCandIdx( ui, uiAbsPartIdx ) ) );
//  }
//#endif
//  DTRACE_CABAC_T( "\n" );
  return;
}
#else
Void TDecSbac::parseMergeFlag ( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth, UInt uiPUIdx )
{
  UInt uiSymbol;
  UInt uiCtx = 0;
#if CHANGE_MERGE_CONTEXT
  uiCtx = pcCU->getCtxMergeFlag( uiAbsPartIdx );
#else
  for(UInt uiIter = 0; uiIter < MRG_MAX_NUM_CANDS; uiIter++ )
  {
    if( pcCU->getNeighbourCandIdx( uiIter, uiAbsPartIdx ) == uiIter + 1 )
    {
      if( uiIter == 0 )
      {
        uiCtx++;
      }
      else if( uiIter == 1 )
      {
        uiCtx++;
      }
    }
  }
#endif
  m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUMergeFlagExtSCModel.get( 0, 0, uiCtx ) );
  pcCU->setMergeFlagSubParts( uiSymbol ? true : false, uiAbsPartIdx, uiPUIdx, uiDepth );

  DTRACE_CABAC_V( g_nSymbolCounter++ );
  DTRACE_CABAC_T( "\tMergeFlag: " );
  DTRACE_CABAC_V( uiSymbol );
  DTRACE_CABAC_T( "\tAddress: " );
  DTRACE_CABAC_V( pcCU->getAddr() );
  DTRACE_CABAC_T( "\tuiAbsPartIdx: " );
  DTRACE_CABAC_V( uiAbsPartIdx );
  for( UInt ui = 0; ui < MRG_MAX_NUM_CANDS; ui++ )
  {
    DTRACE_CABAC_T( "\tNumMrgCand: " );
    DTRACE_CABAC_V( ui );
    DTRACE_CABAC_T( "\t==\t" );
    DTRACE_CABAC_V( UInt( pcCU->getNeighbourCandIdx( ui, uiAbsPartIdx ) ) );
  }
  DTRACE_CABAC_T( "\n" );
}
#endif

#if HHI_SYS_PIPE
Void TDecSbac::parseMergeIndex ( TComDataCU* pcCU, UInt& ruiMergeIndex, UInt uiAbsPartIdx, UInt uiDepth )
{
  Bool bLeftInvolved       = false;
  Bool bAboveInvolved      = false;
  Bool bCollocatedInvolved = false;
  Bool bCornerInvolved     = false;
  Bool bCornerBLInvolved   = false;
  UInt uiNumCand           = 0;

  for( UInt uiIter = 0; uiIter < MRG_MAX_NUM_CANDS; ++uiIter )
  {
    if( pcCU->getNeighbourCandIdx( uiIter, uiAbsPartIdx ) == uiIter + 1 )
    {
      uiNumCand++;

      switch( uiIter )
      {
      case 0:
        bLeftInvolved       = true;
        break;
      case 1:
        bAboveInvolved      = true;
        break;
      case 2:
        bCollocatedInvolved = true;
        break;
      case 3:
        bCornerInvolved     = true;
        break;
      default:
        bCornerBLInvolved   = true;
        break;
      }
    }
  }

  UInt auiCtx[4] =
  {
    0, 0, 0, 3
  };

  if( bLeftInvolved && bAboveInvolved )
  {
    auiCtx[ 0 ] = 0;
  }
  else if( bLeftInvolved || bAboveInvolved )
  {
    auiCtx[ 0 ] = bCollocatedInvolved ? 1 : 2;
  }
  else
  {
    auiCtx[ 0 ] = bCollocatedInvolved ? 2 : 3;
  }

  if( uiNumCand >= 3 )
  {
    if( bAboveInvolved )
    {
      auiCtx[ 1 ] = bCollocatedInvolved ? 1 : 2;
    }
    else
    {
      auiCtx[ 1 ] = bCollocatedInvolved ? 2 : 3;
    }
  }

  if( uiNumCand >= 4 )
  {
    auiCtx[ 2 ] =  bCollocatedInvolved ? 2 : 3;
  }

  UInt       uiUnaryIdx = 0;
  const UInt uiNumTest  = uiNumCand - 1;
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cCUMergeIdxExtSCModel.get();
#else
  ContextModel *pCtx = m_cCUMergeIdxExtSCModel.get();
#endif

  for( ; uiUnaryIdx < uiNumTest; ++uiUnaryIdx )
  {
    const UInt uiSymbol = xDecodeBin( *( pCtx + auiCtx[ uiUnaryIdx ] ) );

    if( !uiSymbol )
    {
      break;
    }
  }

  if( !bLeftInvolved )
  {
    ++uiUnaryIdx;
  }
  if( !bAboveInvolved && uiUnaryIdx >= 1 )
  {
    ++uiUnaryIdx;
  }
  if( !bCollocatedInvolved && uiUnaryIdx >= 2 )
  {
    ++uiUnaryIdx;
  }
  if( !bCornerInvolved && uiUnaryIdx >= 3 )
  {
    ++uiUnaryIdx;
  }

  ruiMergeIndex = uiUnaryIdx;

  DTRACE_CABAC_V( g_nSymbolCounter++ )
  DTRACE_CABAC_T( "\tparseMergeIndex()" )
  DTRACE_CABAC_T( "\tuiMRGIdx= " )
  DTRACE_CABAC_V( ruiMergeIndex )
  DTRACE_CABAC_T( "\tuiNumCand= " )
  DTRACE_CABAC_V( uiNumCand )
  DTRACE_CABAC_T( "\tbLeftInvolved= " )
  DTRACE_CABAC_V( bLeftInvolved )
  DTRACE_CABAC_T( "\tbAboveInvolved= " )
  DTRACE_CABAC_V( bAboveInvolved )
  DTRACE_CABAC_T( "\tbCollocatedInvolved= " )
  DTRACE_CABAC_V( bCollocatedInvolved )
  DTRACE_CABAC_T( "\tbCornerRTInvolved= " )
  DTRACE_CABAC_V( bCornerInvolved )
  DTRACE_CABAC_T( "\tbCornerBLInvolved= " )
  DTRACE_CABAC_V( bCornerBLInvolved )
  DTRACE_CABAC_T( "\n" )
  return;
}
#else
Void TDecSbac::parseMergeIndex ( TComDataCU* pcCU, UInt& ruiMergeIndex, UInt uiAbsPartIdx, UInt uiDepth )
{
  Bool bLeftInvolved = false;
  Bool bAboveInvolved = false;
  Bool bCollocatedInvolved = false;
  Bool bCornerInvolved = false;
  UInt uiNumCand = 0;
  for( UInt uiIter = 0; uiIter < MRG_MAX_NUM_CANDS; ++uiIter )
  {
    if( pcCU->getNeighbourCandIdx( uiIter, uiAbsPartIdx ) == uiIter + 1 )
    {
      uiNumCand++;
      if( uiIter == 0 )
      {
        bLeftInvolved = true;
      }
      else if( uiIter == 1 )
      {
        bAboveInvolved = true;
      }
      else if( uiIter == 2 )
      {
        bCollocatedInvolved = true;
      }
      else if( uiIter == 3 )
      {
        bCornerInvolved = true;
      }
    }
  }
  assert( uiNumCand > 1 );

  UInt auiCtx[4] = { 0, 0, 0, 3 };
  if( bLeftInvolved && bAboveInvolved )
  {
    auiCtx[0] = 0;
  }
  else if( bLeftInvolved || bAboveInvolved )
  {
    auiCtx[0] = bCollocatedInvolved ? 1 : 2;
  }
  else
  {
    auiCtx[0] = bCollocatedInvolved ? 2 : 3;
  }

  if( uiNumCand >= 3 )
  {
    if( bAboveInvolved )
    {
      auiCtx[1] = bCollocatedInvolved ? 1 : 2;
    }
    else
    {
      auiCtx[1] = bCollocatedInvolved ? 2 : 3;
    }
  }

  if( uiNumCand >= 4 )
  {
    auiCtx[2] =  bCollocatedInvolved ? 2 : 3;
  }

  UInt uiUnaryIdx = 0;
  for( ; uiUnaryIdx < uiNumCand - 1; ++uiUnaryIdx )
  {
    UInt uiSymbol = 0;
    m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUMergeIdxExtSCModel.get( 0, 0, auiCtx[uiUnaryIdx] ) );
    if( uiSymbol == 0 )
    {
      break;
    }
  }

  if( !bLeftInvolved )
  {
    ++uiUnaryIdx;
  }
  if( !bAboveInvolved && uiUnaryIdx >= 1 )
  {
    ++uiUnaryIdx;
  }
  if( !bCollocatedInvolved && uiUnaryIdx >= 2 )
  {
    ++uiUnaryIdx;
  }
  if( !bCornerInvolved && uiUnaryIdx >= 3 )
  {
    ++uiUnaryIdx;
  }

  ruiMergeIndex = uiUnaryIdx;

  DTRACE_CABAC_V( g_nSymbolCounter++ )
  DTRACE_CABAC_T( "\tparseMergeIndex()" )
  DTRACE_CABAC_T( "\tuiMRGIdx= " )
  DTRACE_CABAC_V( ruiMergeIndex )
  DTRACE_CABAC_T( "\tuiNumCand= " )
  DTRACE_CABAC_V( uiNumCand )
  DTRACE_CABAC_T( "\tbLeftInvolved= " )
  DTRACE_CABAC_V( bLeftInvolved )
  DTRACE_CABAC_T( "\tbAboveInvolved= " )
  DTRACE_CABAC_V( bAboveInvolved )
  DTRACE_CABAC_T( "\tbCollocatedInvolved= " )
  DTRACE_CABAC_V( bCollocatedInvolved )
  DTRACE_CABAC_T( "\tbCornerRTInvolved= " )
  DTRACE_CABAC_V( bCornerInvolved )
  DTRACE_CABAC_T( "\n" )
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseMVPIdx( TComDataCU* pcCU, Int& riMVPIdx, Int iMVPNum, UInt uiAbsPartIdx, UInt uiDepth, RefPicList eRefList )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cMVPIdxSCModel.get();
#else
  ContextModel *pCtx = m_cMVPIdxSCModel.get();
#endif
  riMVPIdx = xReadUnaryMaxSymbol( pCtx, 1, iMVPNum-1 );
  return;
}
#else
Void TDecSbac::parseMVPIdx      ( TComDataCU* pcCU, Int& riMVPIdx, Int iMVPNum, UInt uiAbsPartIdx, UInt uiDepth, RefPicList eRefList )
{
  UInt uiSymbol;
  xReadUnaryMaxSymbol(uiSymbol, m_cMVPIdxSCModel.get(0), 1, iMVPNum-1);

  riMVPIdx = uiSymbol;
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseSplitFlag( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
  if( uiDepth == g_uiMaxCUDepth - g_uiAddCUDepth )
  {
    pcCU->setDepthSubParts( uiDepth, uiAbsPartIdx );
    return;
  }

#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cCUSplitFlagSCModel.get();
#else
  ContextModel *pCtx = m_cCUSplitFlagSCModel.get();
#endif
#if HHI_NO_GET_CTX_CALL
  const UInt uiSymbol = xDecodeBin( *pCtx );
#else
  const UInt uiSymbol = xDecodeBin( *( pCtx + pcCU->getCtxSplitFlag( uiAbsPartIdx, uiDepth ) ) );
#endif
  DTRACE_CABAC_V( g_nSymbolCounter++ )
  DTRACE_CABAC_T( "\tSplitFlag\n" )
  pcCU->setDepthSubParts( uiDepth + uiSymbol, uiAbsPartIdx );
  return;
}
#else
Void TDecSbac::parseSplitFlag     ( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
  if( uiDepth == g_uiMaxCUDepth - g_uiAddCUDepth )
  {
    pcCU->setDepthSubParts( uiDepth, uiAbsPartIdx );
    return;
  }

  UInt uiSymbol;
  m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUSplitFlagSCModel.get( 0, 0, pcCU->getCtxSplitFlag( uiAbsPartIdx, uiDepth ) ) );
  DTRACE_CABAC_V( g_nSymbolCounter++ )
  DTRACE_CABAC_T( "\tSplitFlag\n" )
  pcCU->setDepthSubParts( uiDepth + uiSymbol, uiAbsPartIdx );

  return;
}
#endif

/** parse partition size
 * \param pcCU
 * \param uiAbsPartIdx
 * \param uiDepth
 * \returns Void
 */
#if HHI_SYS_PIPE
Void
TDecSbac::parsePartSize( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
  UInt     uiMode    = 0;
  PartSize eMode     = SIZE_2Nx2N;
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cCUPartSizeSCModel.get();
#else
  ContextModel *pCtx = m_cCUPartSizeSCModel.get();
#endif

  if( pcCU->isIntra( uiAbsPartIdx ) )
  {
    if( uiDepth == g_uiMaxCUDepth - g_uiAddCUDepth )
    {
      const UInt uiSymbol = xDecodeBin( *pCtx );
      eMode = uiSymbol ? SIZE_2Nx2N : SIZE_NxN;
    }
  }
  else
  {
    const UInt uiMaxNumBits = 3;
    for ( UInt ui = 0; ui < uiMaxNumBits; ui++ )
    {
      const UInt uiSymbol = xDecodeBin( pCtx[ ui ] );

      if( uiSymbol )
      {
        break;
      }
      uiMode++;
    }
    eMode = (PartSize)uiMode;

    if( pcCU->getSlice()->isInterB() && uiMode == 3 )
    {
      UInt uiSymbol = 0;

      if( uiDepth == g_uiMaxCUDepth - g_uiAddCUDepth )
      {
        uiSymbol = xDecodeBin( pCtx[ 3 ] );
      }

      if( uiSymbol == 0 )
      {
        pcCU->setPredModeSubParts( MODE_INTRA, uiAbsPartIdx, uiDepth );

        if( uiDepth == g_uiMaxCUDepth - g_uiAddCUDepth )
        {
          uiSymbol = xDecodeBin( pCtx[ 4 ] );
        }

        if( uiSymbol == 0 )
        {
          eMode = SIZE_2Nx2N;
        }
      }
    }
  }

  pcCU->setPartSizeSubParts( eMode, uiAbsPartIdx, uiDepth );
  pcCU->setSizeSubParts( g_uiMaxCUWidth>>uiDepth, g_uiMaxCUHeight>>uiDepth, uiAbsPartIdx, uiDepth );

  const UInt uiWidthInBit  = g_aucConvertToBit[ pcCU->getWidth( uiAbsPartIdx ) ] + 2;
  const UInt uiTrSizeInBit = g_aucConvertToBit[ pcCU->getSlice()->getSPS()->getMaxTrSize() ] + 2;
  const UInt uiTrLevel     = uiWidthInBit >= uiTrSizeInBit ? uiWidthInBit - uiTrSizeInBit : 0;

  if( pcCU->getPredictionMode(uiAbsPartIdx) == MODE_INTRA )
  {
    if( pcCU->getPartitionSize( uiAbsPartIdx ) == SIZE_NxN )
    {
      pcCU->setTrIdxSubParts( 1+uiTrLevel, uiAbsPartIdx, uiDepth );
    }
    else
    {
      pcCU->setTrIdxSubParts( uiTrLevel, uiAbsPartIdx, uiDepth );
    }
  }
  return;
}
#else
Void TDecSbac::parsePartSize( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
  UInt uiSymbol, uiMode = 0;
  PartSize eMode;

  if ( pcCU->isIntra( uiAbsPartIdx ) )
  {
#if MTK_DISABLE_INTRA_NxN_SPLIT
    uiSymbol = 1;
    if( uiDepth == g_uiMaxCUDepth - g_uiAddCUDepth )
#endif
    {
      m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUPartSizeSCModel.get( 0, 0, 0) );
    }
    eMode = uiSymbol ? SIZE_2Nx2N : SIZE_NxN;
  }
  else
  {
#if HHI_RMP_SWITCH
    if ( !pcCU->getSlice()->getSPS()->getUseRMP())
    {
      m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUPartSizeSCModel.get( 0, 0, 0) );
      if( uiSymbol )
        uiMode = 0;
      else
        uiMode = 3;
    }
    else
#endif
    {
      UInt uiMaxNumBits = 3;
      for ( UInt ui = 0; ui < uiMaxNumBits; ui++ )
      {
        m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUPartSizeSCModel.get( 0, 0, ui) );
        if ( uiSymbol )
        {
          break;
        }
        uiMode++;
      }
    }
    eMode = (PartSize) uiMode;

    if (pcCU->getSlice()->isInterB() && uiMode == 3)
    {
#if HHI_DISABLE_INTER_NxN_SPLIT
      uiSymbol = 0;
      if( uiDepth == g_uiMaxCUDepth - g_uiAddCUDepth )
#endif
      {
        m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUPartSizeSCModel.get( 0, 0, 3) );
      }

      if (uiSymbol == 0)
      {
        pcCU->setPredModeSubParts( MODE_INTRA, uiAbsPartIdx, uiDepth );
#if MTK_DISABLE_INTRA_NxN_SPLIT
        if( uiDepth == g_uiMaxCUDepth - g_uiAddCUDepth )
#endif
        {
          m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUPartSizeSCModel.get( 0, 0, 4) );
        }
        if (uiSymbol == 0)
          eMode = SIZE_2Nx2N;
      }
    }
  }

  pcCU->setPartSizeSubParts( eMode, uiAbsPartIdx, uiDepth );
  pcCU->setSizeSubParts( g_uiMaxCUWidth>>uiDepth, g_uiMaxCUHeight>>uiDepth, uiAbsPartIdx, uiDepth );

  UInt uiTrLevel = 0;

  UInt uiWidthInBit  = g_aucConvertToBit[pcCU->getWidth(uiAbsPartIdx)]+2;
  UInt uiTrSizeInBit = g_aucConvertToBit[pcCU->getSlice()->getSPS()->getMaxTrSize()]+2;
  uiTrLevel          = uiWidthInBit >= uiTrSizeInBit ? uiWidthInBit - uiTrSizeInBit : 0;

  if( pcCU->getPredictionMode(uiAbsPartIdx) == MODE_INTRA )
  {
    if( pcCU->getPartitionSize( uiAbsPartIdx ) == SIZE_NxN )
    {
      pcCU->setTrIdxSubParts( 1+uiTrLevel, uiAbsPartIdx, uiDepth );
    }
    else
    {
      pcCU->setTrIdxSubParts( uiTrLevel, uiAbsPartIdx, uiDepth );
    }
  }
}
#endif
/** parse prediction mode
 * \param pcCU
 * \param uiAbsPartIdx
 * \param uiDepth
 * \returns Void
 */
#if HHI_SYS_PIPE
Void
TDecSbac::parsePredMode( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
  if( pcCU->getSlice()->isIntra() )
  {
    pcCU->setPredModeSubParts( MODE_INTRA, uiAbsPartIdx, uiDepth );
    return;
  }

  if ( pcCU->getSlice()->isInterB() )
  {
    pcCU->setPredModeSubParts( MODE_INTER, uiAbsPartIdx, uiDepth );
    return;
  }

#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cCUPredModeSCModel.get() + 1;
#else
  ContextModel *pCtx = m_cCUPredModeSCModel.get() + 1;
#endif
  const UInt uiSymbol = xDecodeBin( *pCtx );
  pcCU->setPredModeSubParts( (PredMode)( MODE_INTER + uiSymbol ), uiAbsPartIdx, uiDepth );
  return;
}
#else
Void TDecSbac::parsePredMode( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
  if( pcCU->getSlice()->isIntra() )
  {
    pcCU->setPredModeSubParts( MODE_INTRA, uiAbsPartIdx, uiDepth );
    return;
  }

  UInt uiSymbol;
  Int  iPredMode = MODE_INTER;
  if ( pcCU->getSlice()->isInterB() )
  {
    pcCU->setPredModeSubParts( (PredMode)iPredMode, uiAbsPartIdx, uiDepth );
    return;
  }
  m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUPredModeSCModel.get( 0, 0, 1 ) );
  iPredMode += uiSymbol;
  pcCU->setPredModeSubParts( (PredMode)iPredMode, uiAbsPartIdx, uiDepth );
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseIntraDirLumaAng( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cCUIntraPredSCModel.get();
#else
  ContextModel *pCtx = m_cCUIntraPredSCModel.get();
#endif
  const UInt uiIntraIdx = pcCU->getIntraSizeIdx( uiAbsPartIdx );

  UInt uiIPredMode = 0;
  Int  iPreds[2]   = { -1, -1 };
  Int  iPredNum    = pcCU->getIntraDirLumaPredictor( uiAbsPartIdx, iPreds );

  if ( xDecodeBin( *pCtx ) )
  {
    if( iPredNum == 1 )
    {
      uiIPredMode = iPreds[ 0 ];
    }
    else
    {
      const UInt uiSymbol = xDecodeBin( pCtx[ 2 ] );
      uiIPredMode = iPreds[ uiSymbol ];
    }
  }
  else
  {
    if( g_aucIntraModeBitsAng[ uiIntraIdx ] < 6 )
    {
      {
        const UInt uiSymbol = xDecodeBin( pCtx[ 1 ] ); uiIPredMode  = uiSymbol;
      }

      if( g_aucIntraModeBitsAng[ uiIntraIdx ] > 2 ) { const UInt uiSymbol = xDecodeBin( pCtx[ 1 ] ); uiIPredMode |= uiSymbol << 1; }
      if( g_aucIntraModeBitsAng[ uiIntraIdx ] > 3 ) { const UInt uiSymbol = xDecodeBin( pCtx[ 1 ] ); uiIPredMode |= uiSymbol << 2; }
      if( g_aucIntraModeBitsAng[ uiIntraIdx ] > 4 ) { const UInt uiSymbol = xDecodeBin( pCtx[ 1 ] ); uiIPredMode |= uiSymbol << 3; }
    }
    else
    {
      for( UInt ui = 0; ui < 5; ui++ )
      {
        const UInt uiSymbol = xDecodeBin( pCtx[ 1 ] );
        uiIPredMode |= uiSymbol << ui;
      }

      if( uiIPredMode == 31 )
      { // Escape coding for the last two modes
        const UInt uiSymbol = xDecodeBin( pCtx[ 1 ] );
        uiIPredMode = uiSymbol ? 32 : 31;
      }
    }

    for( UInt ui = 0; ui < iPredNum; ui++ )
    {
      if( uiIPredMode >= iPreds[ ui ] )
      {
        uiIPredMode++;
      }
    }
  }

  if( uiIPredMode == 2 )
  {
    const UInt uiPlanarFlag = xDecodeBin( *m_cPlanarFlagSCModel.get() );
    if( uiPlanarFlag )
    {
      uiIPredMode = PLANAR_IDX;
    }
  }

  pcCU->setLumaIntraDirSubParts( uiIPredMode, uiAbsPartIdx, uiDepth );
  return;
}
#else
#if MTK_DCM_MPM
Void TDecSbac::parseIntraDirLumaAng  ( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{

  Int iIntraIdx = pcCU->getIntraSizeIdx(uiAbsPartIdx);

  UInt uiSymbol;
  Int  uiIPredMode;

  Int uiPreds[2] = {-1, -1};
  Int uiPredNum = pcCU->getIntraDirLumaPredictor(uiAbsPartIdx, uiPreds);


  m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 0) );

  if ( uiSymbol )
  {
    if(uiPredNum == 1)
    {
      uiIPredMode = uiPreds[0];
    }
    else
    {
      m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 2) );
      uiIPredMode = uiPreds[uiSymbol];
    }
  }
  else
  {
    if ( g_aucIntraModeBitsAng[iIntraIdx] < 6 )
    {
      m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode  = uiSymbol;
      if ( g_aucIntraModeBitsAng[iIntraIdx] > 2 ) { m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode |= uiSymbol << 1; }
      if ( g_aucIntraModeBitsAng[iIntraIdx] > 3 ) { m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode |= uiSymbol << 2; }
      if ( g_aucIntraModeBitsAng[iIntraIdx] > 4 ) { m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode |= uiSymbol << 3; }
    }
    else
    {
    m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode  = uiSymbol;
    m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode |= uiSymbol << 1;
    m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode |= uiSymbol << 2;
    m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode |= uiSymbol << 3;
    m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode |= uiSymbol << 4;

    if (uiIPredMode == 31){ // Escape coding for the last two modes
      m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) );
      uiIPredMode = uiSymbol ? 32 : 31;
    }

  }

    for(UInt i = 0; i < uiPredNum; i++)
    {
      if(uiIPredMode >= uiPreds[i]) {  uiIPredMode ++; }
    }
  }
#if ADD_PLANAR_MODE
  if (uiIPredMode == 2)
  {
    UInt planarFlag;
    m_pcTDecBinIf->decodeBin( planarFlag, m_cPlanarFlagSCModel.get(0,0,0) );
    if ( planarFlag )
    {
      uiIPredMode = PLANAR_IDX;
    }
  }
#endif

  pcCU->setLumaIntraDirSubParts( (UChar)uiIPredMode, uiAbsPartIdx, uiDepth );
}
#else
Void TDecSbac::parseIntraDirLumaAng  ( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
  UInt uiSymbol;
  Int  uiIPredMode;
  Int  iMostProbable = pcCU->getMostProbableIntraDirLuma( uiAbsPartIdx );

  m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 0) );

  if ( uiSymbol )
    uiIPredMode = iMostProbable;
  else
  {
    Int iIntraIdx = pcCU->getIntraSizeIdx(uiAbsPartIdx);
    if ( g_aucIntraModeBitsAng[iIntraIdx] < 6 )
    {
      m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode  = uiSymbol;
      if ( g_aucIntraModeBitsAng[iIntraIdx] > 2 ) { m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode |= uiSymbol << 1; }
      if ( g_aucIntraModeBitsAng[iIntraIdx] > 3 ) { m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode |= uiSymbol << 2; }
      if ( g_aucIntraModeBitsAng[iIntraIdx] > 4 ) { m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode |= uiSymbol << 3; }
    }
    else
    {
      m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode  = uiSymbol;
      m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode |= uiSymbol << 1;
      m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode |= uiSymbol << 2;
      m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode |= uiSymbol << 3;
      m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) ); uiIPredMode |= uiSymbol << 4;

      if (uiIPredMode == 31)
      { // Escape coding for the last two modes
        m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUIntraPredSCModel.get( 0, 0, 1 ) );
        uiIPredMode = uiSymbol ? 32 : 31;
      }
    }

    if (uiIPredMode >= iMostProbable)
      uiIPredMode++;
  }

#if ADD_PLANAR_MODE
  if (uiIPredMode == 2)
  {
    UInt planarFlag;
    m_pcTDecBinIf->decodeBin( planarFlag, m_cPlanarFlagSCModel.get(0,0,0) );
    if ( planarFlag )
    {
      uiIPredMode = PLANAR_IDX;
    }
  }
#endif
  pcCU->setLumaIntraDirSubParts( (UChar)uiIPredMode, uiAbsPartIdx, uiDepth );
}
#endif
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseIntraDirChroma( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cCUChromaPredSCModel.get();
#else
  ContextModel *pCtx = m_cCUChromaPredSCModel.get();
#endif
#if CHROMA_CODEWORD
  UInt uiMode = pcCU->getLumaIntraDir( uiAbsPartIdx );

#if ADD_PLANAR_MODE
  if( ( uiMode == 2 ) || ( uiMode == PLANAR_IDX ) )
  {
    uiMode = 4;
  }
#endif

#if LM_CHROMA
  const UInt uiMaxMode = pcCU->getSlice()->getSPS()->getUseLMChroma() ? 3 : 4;
  const UInt uiMax     = uiMode < uiMaxMode ? 2 : 3;

#if HHI_NO_GET_CTX_CALL
  UInt uiSymbol = xDecodeBin( *pCtx );
#else
  UInt uiSymbol = xDecodeBin( *( pCtx + pcCU->getCtxIntraDirChroma( uiAbsPartIdx ) ) );
#endif

  if( uiSymbol )
  {
    uiSymbol = xReadUnaryMaxSymbol( pCtx + 3, 0, uiMax );
    uiSymbol++;
  }

  if( !uiSymbol )
  {
    uiSymbol = 4;
  }
  else if( uiSymbol == 1 && pcCU->getSlice()->getSPS()->getUseLMChroma() )
  {
    uiSymbol = 3;
  }
  else
  {
#if CHROMA_CODEWORD_SWITCH
    uiSymbol = ChromaMapping[ uiMax-2 ][ uiSymbol ];
#endif

    if( pcCU->getSlice()->getSPS()->getUseLMChroma() )
    {
      uiSymbol--;
    }

    if( uiSymbol <= uiMode )
    {
      uiSymbol--;
    }
  }
#else // <-- LM_CHROMA
  const UInt uiMax = uiMode < 4 ? 2 : 3;
#if HHI_NO_GET_CTX_CALL
  uiSymbol = xDecodeBin( *pCtx );
#else
  uiSymbol = xDecodeBin( *( pCtx + pcCU->getCtxIntraDirChroma( uiAbsPartIdx ) ) );
#endif
  if ( uiSymbol )
  {
    xReadUnaryMaxSymbol( uiSymbol, pCtx + 3, 0, uiMax );
    uiSymbol++;
  }

  //switch codeword
  if (uiSymbol == 0)
  {
    uiSymbol = 4;
  }
#if CHROMA_CODEWORD_SWITCH
  else
  {
    uiSymbol = ChromaMapping[ uiMax-2 ][ uiSymbol ];
    if( uiSymbol <= uiMode )
    {
      uiSymbol --;
    }
  }
#else
  else if( uiSymbol <= uiMode )
  {
    uiSymbol --;
  }
#endif
#endif // <-- LM_CHROMA

#else // CHROMA_CODEWORD
#if HHI_NO_GET_CTX_CALL
  uiSymbol = xDecodeBin( *pCtx );
#else
  uiSymbol = xDecodeBin( *( pCtx + pcCU->getCtxIntraDirChroma( uiAbsPartIdx ) ) );
#endif

  if ( uiSymbol )
  {
    xReadUnaryMaxSymbol( uiSymbol, pCtx + 3, 0, 3 );
    uiSymbol++;
  }
#endif

#if ADD_PLANAR_MODE
  if( uiSymbol == 2 )
  {
#if CHROMA_CODEWORD
    uiMode = pcCU->getLumaIntraDir(uiAbsPartIdx);

    if( uiMode == 2 )
    {
      uiSymbol = PLANAR_IDX;
    }
    else if( uiMode != PLANAR_IDX )
#endif
    {
      const UInt uiPlanarFlag = xDecodeBin( *( m_cPlanarFlagSCModel.get() + 1 ) );

      if( uiPlanarFlag )
      {
        uiSymbol = PLANAR_IDX;
      }
    }
  }
#endif

  pcCU->setChromIntraDirSubParts( uiSymbol, uiAbsPartIdx, uiDepth );
  return;
}
#else
Void TDecSbac::parseIntraDirChroma( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
  UInt uiSymbol;

#if CHROMA_CODEWORD
  UInt uiMode = pcCU->getLumaIntraDir(uiAbsPartIdx);
#if ADD_PLANAR_MODE
  if ( (uiMode == 2 ) || (uiMode == PLANAR_IDX) )
  {
    uiMode = 4;
  }
#endif

#if LM_CHROMA
  Int  iMaxMode = pcCU->getSlice()->getSPS()->getUseLMChroma() ? 3 : 4;
  Int  iMax = uiMode < iMaxMode ? 2 : 3;

  m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUChromaPredSCModel.get( 0, 0, pcCU->getCtxIntraDirChroma( uiAbsPartIdx ) ) );

  if ( uiSymbol )
  {
    xReadUnaryMaxSymbol( uiSymbol, m_cCUChromaPredSCModel.get( 0, 0 ) + 3, 0, iMax );
    uiSymbol++;
  }

  //switch codeword
  if (uiSymbol == 0)
  {
    uiSymbol = 4;
  }
  else if (uiSymbol == 1 && pcCU->getSlice()->getSPS()->getUseLMChroma())
  {
    uiSymbol = 3;
  }
  else
  {
#if CHROMA_CODEWORD_SWITCH
    uiSymbol = ChromaMapping[iMax-2][uiSymbol];
#endif

    if (pcCU->getSlice()->getSPS()->getUseLMChroma())
       uiSymbol --;

    if (uiSymbol <= uiMode)
       uiSymbol --;
  }
#else // <-- LM_CHROMA
  Int  iMax = uiMode < 4 ? 2 : 3;

  m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUChromaPredSCModel.get( 0, 0, pcCU->getCtxIntraDirChroma( uiAbsPartIdx ) ) );

  if ( uiSymbol )
  {
    xReadUnaryMaxSymbol( uiSymbol, m_cCUChromaPredSCModel.get( 0, 0 ) + 3, 0, iMax );
    uiSymbol++;
  }

  //switch codeword
  if (uiSymbol == 0)
  {
    uiSymbol = 4;
  }
#if CHROMA_CODEWORD_SWITCH
  else
  {
    uiSymbol = ChromaMapping[iMax-2][uiSymbol];
    if (uiSymbol <= uiMode)
    {
      uiSymbol --;
    }
  }
#else
  else if (uiSymbol <= uiMode)
  {
    uiSymbol --;
  }
#endif
#endif // <-- LM_CHROMA

#else // CHROMA_CODEWORD
  m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUChromaPredSCModel.get( 0, 0, pcCU->getCtxIntraDirChroma( uiAbsPartIdx ) ) );

  if ( uiSymbol )
  {
    xReadUnaryMaxSymbol( uiSymbol, m_cCUChromaPredSCModel.get( 0, 0 ) + 3, 0, 3 );
    uiSymbol++;
  }
#endif

#if ADD_PLANAR_MODE
  if (uiSymbol == 2)
  {
#if CHROMA_CODEWORD
    uiMode = pcCU->getLumaIntraDir(uiAbsPartIdx);
    if (uiMode == 2)
    {
      uiSymbol = PLANAR_IDX;
    }
    else if (uiMode != PLANAR_IDX)
#endif
    {
      UInt planarFlag;
      m_pcTDecBinIf->decodeBin( planarFlag, m_cPlanarFlagSCModel.get(0,0,1) );
      if ( planarFlag )
      {
        uiSymbol = PLANAR_IDX;
      }
    }
  }
#endif
  pcCU->setChromIntraDirSubParts( uiSymbol, uiAbsPartIdx, uiDepth );

  return;
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseInterDir( TComDataCU* pcCU, UInt& ruiInterDir, UInt uiAbsPartIdx, UInt uiDepth )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cCUInterDirSCModel.get();
#else
  ContextModel *pCtx = m_cCUInterDirSCModel.get() + pcCU->getCtxInterDir( uiAbsPartIdx );
#endif

  ruiInterDir = xDecodeBin( *pCtx );

  if ( ruiInterDir )
  {
    ruiInterDir = 3;
  }
  else if( pcCU->getSlice()->getNumRefIdx( REF_PIC_LIST_C ) > 0 )
  {
    ruiInterDir = 1;
  }
  else if( pcCU->getSlice()->getNoBackPredFlag() )
  {
    ruiInterDir = 1;
  }
  else
  {
    ruiInterDir = xDecodeBin( pCtx[ 3 ] ) + 1;
  }
  return;
}
#else
Void TDecSbac::parseInterDir( TComDataCU* pcCU, UInt& ruiInterDir, UInt uiAbsPartIdx, UInt uiDepth )
{
  UInt uiSymbol;
  UInt uiCtx = pcCU->getCtxInterDir( uiAbsPartIdx );

  m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUInterDirSCModel.get( 0, 0, uiCtx ) );

  if ( uiSymbol )
  {
    uiSymbol = 2;
  }
#if DCM_COMB_LIST
  else if(pcCU->getSlice()->getNumRefIdx(REF_PIC_LIST_C) > 0)
  {
    uiSymbol = 0;
  }
#endif
  else if ( pcCU->getSlice()->getNoBackPredFlag() )
  {
    uiSymbol = 0;
  }
  else
  {
    m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUInterDirSCModel.get( 0, 0, 3 ) );
  }
  uiSymbol++;
  ruiInterDir = uiSymbol;
  return;
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseRefFrmIdx( TComDataCU* pcCU, Int& riRefFrmIdx, UInt uiAbsPartIdx, UInt uiDepth, RefPicList eRefList )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cCURefPicSCModel.get();
#else
  ContextModel *pCtx = m_cCURefPicSCModel.get();
#endif

#if DCM_COMB_LIST
  if( pcCU->getSlice()->getNumRefIdx( REF_PIC_LIST_C ) > 0 && eRefList == REF_PIC_LIST_C )
  {
#if HHI_NO_GET_CTX_CALL
    UInt uiSymbol = xDecodeBin( *pCtx );
#else
    UInt uiSymbol = xDecodeBin( *( pCtx + pcCU->getCtxRefIdx( uiAbsPartIdx, RefPicList( pcCU->getSlice()->getListIdFromIdxOfLC( 0 ) ) ) ) );
#endif

    if( uiSymbol )
    {
      uiSymbol = xReadUnaryMaxSymbol( pCtx + 4, 1, pcCU->getSlice()->getNumRefIdx( REF_PIC_LIST_C )-2 );
      uiSymbol++;
    }

    riRefFrmIdx = uiSymbol;
  }
  else
  {
#endif
#if HHI_NO_GET_CTX_CALL
    UInt uiSymbol = xDecodeBin( *pCtx );
#else
    UInt uiSymbol = xDecodeBin( *( pCtx + pcCU->getCtxRefIdx( uiAbsPartIdx, eRefList ) ) );
#endif

    if( uiSymbol )
    {
      uiSymbol = xReadUnaryMaxSymbol( pCtx + 4, 1, pcCU->getSlice()->getNumRefIdx( eRefList )-2 );
      uiSymbol++;
    }

    riRefFrmIdx = uiSymbol;
#if DCM_COMB_LIST
  }
#endif
  return;
}
#else
Void TDecSbac::parseRefFrmIdx( TComDataCU* pcCU, Int& riRefFrmIdx, UInt uiAbsPartIdx, UInt uiDepth, RefPicList eRefList )
{
  UInt uiSymbol;

#if DCM_COMB_LIST
  if(pcCU->getSlice()->getNumRefIdx(REF_PIC_LIST_C ) > 0 && eRefList==REF_PIC_LIST_C)
  {
    UInt uiCtx;

    uiCtx = pcCU->getCtxRefIdx( uiAbsPartIdx, RefPicList(pcCU->getSlice()->getListIdFromIdxOfLC(0)) );

    m_pcTDecBinIf->decodeBin ( uiSymbol, m_cCURefPicSCModel.get( 0, 0, uiCtx ) );
    if ( uiSymbol )
    {
      xReadUnaryMaxSymbol( uiSymbol, &m_cCURefPicSCModel.get( 0, 0, 4 ), 1, pcCU->getSlice()->getNumRefIdx( REF_PIC_LIST_C )-2 );

      uiSymbol++;
    }

    riRefFrmIdx = uiSymbol;
  }
  else
  {
#endif
  UInt uiCtx = pcCU->getCtxRefIdx( uiAbsPartIdx, eRefList );

  m_pcTDecBinIf->decodeBin ( uiSymbol, m_cCURefPicSCModel.get( 0, 0, uiCtx ) );
  if ( uiSymbol )
  {
    xReadUnaryMaxSymbol( uiSymbol, &m_cCURefPicSCModel.get( 0, 0, 4 ), 1, pcCU->getSlice()->getNumRefIdx( eRefList )-2 );

    uiSymbol++;
  }
  riRefFrmIdx = uiSymbol;

#if DCM_COMB_LIST
  }
#endif

  return;
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseMvd( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiPartIdx, UInt uiDepth, RefPicList eRefList )
{
#if MVD_CTX
#if HHI_NO_GET_CTX_CALL
  TComMv cTmpMv( 0, 0 );
  pcCU->getCUMvField( eRefList )->setAllMv( cTmpMv, pcCU->getPartitionSize( uiAbsPartIdx ), uiAbsPartIdx, uiDepth );
  const Int iHor = xReadMvd( 0, 0, 0 );
  const Int iVer = xReadMvd( 0, 0, 1 );
#else
  UInt uiAbsPartIdxL;
  UInt uiAbsPartIdxA;

  TComDataCU*    pcCUL        = pcCU->getPULeft ( uiAbsPartIdxL, pcCU->getZorderIdxInCU() + uiAbsPartIdx );
  TComDataCU*    pcCUA        = pcCU->getPUAbove( uiAbsPartIdxA, pcCU->getZorderIdxInCU() + uiAbsPartIdx );
  TComCUMvField* pcCUMvFieldL = ( pcCUL == NULL || pcCUL->isIntra( uiAbsPartIdxL ) ) ? NULL : pcCUL->getCUMvField( eRefList );
  TComCUMvField* pcCUMvFieldA = ( pcCUA == NULL || pcCUA->isIntra( uiAbsPartIdxA ) ) ? NULL : pcCUA->getCUMvField( eRefList );

  const Int iHorPredL = ( (pcCUMvFieldL == NULL) ? 0 : pcCUMvFieldL->getMvd( uiAbsPartIdxL ).getAbsHor() );
  const Int iVerPredL = ( (pcCUMvFieldL == NULL) ? 0 : pcCUMvFieldL->getMvd( uiAbsPartIdxL ).getAbsVer() );
  const Int iHorPredA = ( (pcCUMvFieldA == NULL) ? 0 : pcCUMvFieldA->getMvd( uiAbsPartIdxA ).getAbsHor() );
  const Int iVerPredA = ( (pcCUMvFieldA == NULL) ? 0 : pcCUMvFieldA->getMvd( uiAbsPartIdxA ).getAbsVer() );
  TComMv cTmpMv( 0, 0 );
  pcCU->getCUMvField( eRefList )->setAllMv( cTmpMv, pcCU->getPartitionSize( uiAbsPartIdx ), uiAbsPartIdx, uiDepth );

  const Int iHor = xReadMvd( iHorPredL, iHorPredA, 0 );
  const Int iVer = xReadMvd( iVerPredL, iVerPredA, 1 );
#endif
  TComMv cMv( iHor, iVer );
  pcCU->getCUMvField( eRefList )->setAllMvd( cMv, pcCU->getPartitionSize( uiAbsPartIdx ), uiAbsPartIdx, uiDepth );
#else
  Int iHor, iVer;
  UInt uiAbsPartIdxL, uiAbsPartIdxA;
#if MVD_CTX
  Int iHorPredL, iVerPredL;
  Int iHorPredA, iVerPredA;
#else
  Int iHorPred, iVerPred;
#endif

  TComDataCU* pcCUL   = pcCU->getPULeft ( uiAbsPartIdxL, pcCU->getZorderIdxInCU() + uiAbsPartIdx );
  TComDataCU* pcCUA   = pcCU->getPUAbove( uiAbsPartIdxA, pcCU->getZorderIdxInCU() + uiAbsPartIdx );

  TComCUMvField* pcCUMvFieldL = ( pcCUL == NULL || pcCUL->isIntra( uiAbsPartIdxL ) ) ? NULL : pcCUL->getCUMvField( eRefList );
  TComCUMvField* pcCUMvFieldA = ( pcCUA == NULL || pcCUA->isIntra( uiAbsPartIdxA ) ) ? NULL : pcCUA->getCUMvField( eRefList );

#if MVD_CTX
  iHorPredL = ( (pcCUMvFieldL == NULL) ? 0 : pcCUMvFieldL->getMvd( uiAbsPartIdxL ).getAbsHor() );
  iVerPredL = ( (pcCUMvFieldL == NULL) ? 0 : pcCUMvFieldL->getMvd( uiAbsPartIdxL ).getAbsVer() );
  iHorPredA = ( (pcCUMvFieldA == NULL) ? 0 : pcCUMvFieldA->getMvd( uiAbsPartIdxA ).getAbsHor() );
  iVerPredA = ( (pcCUMvFieldA == NULL) ? 0 : pcCUMvFieldA->getMvd( uiAbsPartIdxA ).getAbsVer() );
#else
  iHorPred = ( (pcCUMvFieldL == NULL) ? 0 : pcCUMvFieldL->getMvd( uiAbsPartIdxL ).getAbsHor() ) +
    ( (pcCUMvFieldA == NULL) ? 0 : pcCUMvFieldA->getMvd( uiAbsPartIdxA ).getAbsHor() );
  iVerPred = ( (pcCUMvFieldL == NULL) ? 0 : pcCUMvFieldL->getMvd( uiAbsPartIdxL ).getAbsVer() ) +
    ( (pcCUMvFieldA == NULL) ? 0 : pcCUMvFieldA->getMvd( uiAbsPartIdxA ).getAbsVer() );
#endif

  TComMv cTmpMv( 0, 0 );
  pcCU->getCUMvField( eRefList )->setAllMv( cTmpMv, pcCU->getPartitionSize( uiAbsPartIdx ), uiAbsPartIdx, uiDepth );

#if MVD_CTX
  xReadMvd( iHor, iHorPredL, iHorPredA, 0 );
  xReadMvd( iVer, iVerPredL, iVerPredA, 1 );
#else
  xReadMvd( iHor, iHorPred, 0 );
  xReadMvd( iVer, iVerPred, 1 );
#endif

  // set mvd
  TComMv cMv( iHor, iVer );
  pcCU->getCUMvField( eRefList )->setAllMvd( cMv, pcCU->getPartitionSize( uiAbsPartIdx ), uiAbsPartIdx, uiDepth );
#endif
  return;
}
#else
Void TDecSbac::parseMvd( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiPartIdx, UInt uiDepth, RefPicList eRefList )
{
  Int iHor, iVer;
  UInt uiAbsPartIdxL, uiAbsPartIdxA;
#if MVD_CTX
  Int iHorPredL, iVerPredL;
  Int iHorPredA, iVerPredA;
#else
  Int iHorPred, iVerPred;
#endif

  TComDataCU* pcCUL   = pcCU->getPULeft ( uiAbsPartIdxL, pcCU->getZorderIdxInCU() + uiAbsPartIdx );
  TComDataCU* pcCUA   = pcCU->getPUAbove( uiAbsPartIdxA, pcCU->getZorderIdxInCU() + uiAbsPartIdx );

  TComCUMvField* pcCUMvFieldL = ( pcCUL == NULL || pcCUL->isIntra( uiAbsPartIdxL ) ) ? NULL : pcCUL->getCUMvField( eRefList );
  TComCUMvField* pcCUMvFieldA = ( pcCUA == NULL || pcCUA->isIntra( uiAbsPartIdxA ) ) ? NULL : pcCUA->getCUMvField( eRefList );

#if MVD_CTX
  iHorPredL = ( (pcCUMvFieldL == NULL) ? 0 : pcCUMvFieldL->getMvd( uiAbsPartIdxL ).getAbsHor() );
  iVerPredL = ( (pcCUMvFieldL == NULL) ? 0 : pcCUMvFieldL->getMvd( uiAbsPartIdxL ).getAbsVer() );
  iHorPredA = ( (pcCUMvFieldA == NULL) ? 0 : pcCUMvFieldA->getMvd( uiAbsPartIdxA ).getAbsHor() );
  iVerPredA = ( (pcCUMvFieldA == NULL) ? 0 : pcCUMvFieldA->getMvd( uiAbsPartIdxA ).getAbsVer() );
#else
  iHorPred = ( (pcCUMvFieldL == NULL) ? 0 : pcCUMvFieldL->getMvd( uiAbsPartIdxL ).getAbsHor() ) +
  ( (pcCUMvFieldA == NULL) ? 0 : pcCUMvFieldA->getMvd( uiAbsPartIdxA ).getAbsHor() );
  iVerPred = ( (pcCUMvFieldL == NULL) ? 0 : pcCUMvFieldL->getMvd( uiAbsPartIdxL ).getAbsVer() ) +
  ( (pcCUMvFieldA == NULL) ? 0 : pcCUMvFieldA->getMvd( uiAbsPartIdxA ).getAbsVer() );
#endif

  TComMv cTmpMv( 0, 0 );
  pcCU->getCUMvField( eRefList )->setAllMv( cTmpMv, pcCU->getPartitionSize( uiAbsPartIdx ), uiAbsPartIdx, uiDepth );

#if MVD_CTX
  xReadMvd( iHor, iHorPredL, iHorPredA, 0 );
  xReadMvd( iVer, iVerPredL, iVerPredA, 1 );
#else
  xReadMvd( iHor, iHorPred, 0 );
  xReadMvd( iVer, iVerPred, 1 );
#endif

  // set mvd
  TComMv cMv( iHor, iVer );
  pcCU->getCUMvField( eRefList )->setAllMvd( cMv, pcCU->getPartitionSize( uiAbsPartIdx ), uiAbsPartIdx, uiDepth );

  return;
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseTransformSubdivFlag( UInt& ruiSubdivFlag, UInt uiLog2TransformBlockSize )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cCUTransSubdivFlagSCModel.get() + uiLog2TransformBlockSize;
#else
  ContextModel *pCtx = m_cCUTransSubdivFlagSCModel.get() + uiLog2TransformBlockSize;
#endif
  ruiSubdivFlag = xDecodeBin( *pCtx );
  DTRACE_CABAC_V( g_nSymbolCounter++ )
  DTRACE_CABAC_T( "\tparseTransformSubdivFlag()" )
  DTRACE_CABAC_T( "\tsymbol=" )
  DTRACE_CABAC_V( ruiSubdivFlag )
  DTRACE_CABAC_T( "\tctx=" )
  DTRACE_CABAC_V( uiLog2TransformBlockSize )
  DTRACE_CABAC_T( "\n" )
  return;
}
#else
Void TDecSbac::parseTransformSubdivFlag( UInt& ruiSubdivFlag, UInt uiLog2TransformBlockSize )
{
  m_pcTDecBinIf->decodeBin( ruiSubdivFlag, m_cCUTransSubdivFlagSCModel.get( 0, 0, uiLog2TransformBlockSize ) );
  DTRACE_CABAC_V( g_nSymbolCounter++ )
  DTRACE_CABAC_T( "\tparseTransformSubdivFlag()" )
  DTRACE_CABAC_T( "\tsymbol=" )
  DTRACE_CABAC_V( ruiSubdivFlag )
  DTRACE_CABAC_T( "\tctx=" )
  DTRACE_CABAC_V( uiLog2TransformBlockSize )
  DTRACE_CABAC_T( "\n" )
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseQtRootCbf( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth, UInt& uiQtRootCbf )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cCUQtRootCbfSCModel.get();
#else
  ContextModel *pCtx = m_cCUQtRootCbfSCModel.get();
#endif
#if HHI_NO_GET_CTX_CALL
  uiQtRootCbf = xDecodeBin( *pCtx );
#else
  uiQtRootCbf = xDecodeBin( *( pCtx + pcCU->getCtxQtRootCbf( uiAbsPartIdx ) ) );
#endif
  DTRACE_CABAC_V( g_nSymbolCounter++ )
  DTRACE_CABAC_T( "\tparseQtRootCbf()" )
  DTRACE_CABAC_T( "\tsymbol=" )
  DTRACE_CABAC_V( uiQtRootCbf )
  DTRACE_CABAC_T( "\tuiAbsPartIdx=" )
  DTRACE_CABAC_V( uiAbsPartIdx )
  DTRACE_CABAC_T( "\n" )
  return;
}
#else
Void TDecSbac::parseQtRootCbf( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth, UInt& uiQtRootCbf )
{
  UInt uiSymbol;
  const UInt uiCtx = pcCU->getCtxQtRootCbf( uiAbsPartIdx );
  m_pcTDecBinIf->decodeBin( uiSymbol , m_cCUQtRootCbfSCModel.get( 0, 0, uiCtx ) );
  DTRACE_CABAC_V( g_nSymbolCounter++ )
  DTRACE_CABAC_T( "\tparseQtRootCbf()" )
  DTRACE_CABAC_T( "\tsymbol=" )
  DTRACE_CABAC_V( uiSymbol )
  DTRACE_CABAC_T( "\tctx=" )
  DTRACE_CABAC_V( uiCtx )
  DTRACE_CABAC_T( "\tuiAbsPartIdx=" )
  DTRACE_CABAC_V( uiAbsPartIdx )
  DTRACE_CABAC_T( "\n" )

  uiQtRootCbf = uiSymbol;
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseDeltaQP( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cCUDeltaQpSCModel.get();
#else
  ContextModel *pCtx = m_cCUDeltaQpSCModel.get();
#endif
  Int  iDQp  = 0;
  UInt uiDQp = xDecodeBin( *pCtx );

  if( !uiDQp )
  {
    uiDQp = pcCU->getSlice()->getSliceQp();
  }
  else
  {
    uiDQp = xReadUnarySymbol( pCtx + 2, 1 );
    iDQp  = ( uiDQp + 2 ) >> 1;

    if( uiDQp & 1 )
    {
      iDQp = -iDQp;
    }
    uiDQp = pcCU->getSlice()->getSliceQp() + iDQp;
  }

  pcCU->setQPSubParts( uiDQp, uiAbsPartIdx, uiDepth );
  return;
}
#else
Void TDecSbac::parseDeltaQP( TComDataCU* pcCU, UInt uiAbsPartIdx, UInt uiDepth )
{
  UInt uiDQp;
  Int  iDQp;

  m_pcTDecBinIf->decodeBin( uiDQp, m_cCUDeltaQpSCModel.get( 0, 0, 0 ) );

  if ( uiDQp == 0 )
  {
#if SUB_LCU_DQP
    uiDQp = pcCU->getRefQP(uiAbsPartIdx);
#else
    uiDQp = pcCU->getSlice()->getSliceQp();
#endif
  }
  else
  {
    xReadUnarySymbol( uiDQp, &m_cCUDeltaQpSCModel.get( 0, 0, 2 ), 1 );
    iDQp = ( uiDQp + 2 ) / 2;

    if ( uiDQp & 1 )
    {
      iDQp = -iDQp;
    }
#if SUB_LCU_DQP
    uiDQp = pcCU->getRefQP(uiAbsPartIdx) + iDQp;
#else
    uiDQp = pcCU->getSlice()->getSliceQp() + iDQp;
#endif
  }

#if SUB_LCU_DQP
  UInt uiAbsQpCUPartIdx = (uiAbsPartIdx>>(8-(pcCU->getSlice()->getPPS()->getMaxCuDQPDepth()<<1)))<<(8-(pcCU->getSlice()->getPPS()->getMaxCuDQPDepth()<<1)) ;
  UInt uiQpCUDepth =   min(uiDepth,pcCU->getSlice()->getPPS()->getMaxCuDQPDepth()) ;
  pcCU->setQPSubParts( uiDQp, uiAbsQpCUPartIdx, uiQpCUDepth );
#else
  pcCU->setQPSubParts( uiDQp, uiAbsPartIdx, uiDepth );
#endif
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseQtCbf( TComDataCU* pcCU, UInt uiAbsPartIdx, TextType eType, UInt uiTrDepth, UInt uiDepth )
{
#if HHI_SYS_PIPE_LC_MODE
#if HHI_NO_GET_CTX_CALL
  const ContextModel &rcCtx = m_cCUQtCbfSCModel.get( 0, eType ? eType - 1: eType, 0 );
#else
  const ContextModel &rcCtx = m_cCUQtCbfSCModel.get( 0, eType ? eType - 1: eType, pcCU->getCtxQtCbf( uiAbsPartIdx, eType, uiTrDepth ) );
#endif
#else
#if HHI_NO_GET_CTX_CALL
  ContextModel &rcCtx = m_cCUQtCbfSCModel.get( 0, eType ? eType - 1: eType, 0 );
#else
  ContextModel &rcCtx = m_cCUQtCbfSCModel.get( 0, eType ? eType - 1: eType, pcCU->getCtxQtCbf( uiAbsPartIdx, eType, uiTrDepth ) );
#endif
#endif
  const UInt uiSymbol = xDecodeBin( rcCtx );
  DTRACE_CABAC_V( g_nSymbolCounter++ )
  DTRACE_CABAC_T( "\tparseQtCbf()" )
  DTRACE_CABAC_T( "\tsymbol=" )
  DTRACE_CABAC_V( uiSymbol )
  //DTRACE_CABAC_T( "\tctx=" )
  //DTRACE_CABAC_V( uiCtx )
  DTRACE_CABAC_T( "\tetype=" )
  DTRACE_CABAC_V( eType )
  DTRACE_CABAC_T( "\tuiAbsPartIdx=" )
  DTRACE_CABAC_V( uiAbsPartIdx )
  DTRACE_CABAC_T( "\n" )
  pcCU->setCbfSubParts( uiSymbol << uiTrDepth, eType, uiAbsPartIdx, uiDepth );
  return;
}
#else
Void TDecSbac::parseQtCbf( TComDataCU* pcCU, UInt uiAbsPartIdx, TextType eType, UInt uiTrDepth, UInt uiDepth )
{
  UInt uiSymbol;
  const UInt uiCtx = pcCU->getCtxQtCbf( uiAbsPartIdx, eType, uiTrDepth );
  m_pcTDecBinIf->decodeBin( uiSymbol , m_cCUQtCbfSCModel.get( 0, eType ? eType - 1: eType, uiCtx ) );

  DTRACE_CABAC_V( g_nSymbolCounter++ )
  DTRACE_CABAC_T( "\tparseQtCbf()" )
  DTRACE_CABAC_T( "\tsymbol=" )
  DTRACE_CABAC_V( uiSymbol )
  DTRACE_CABAC_T( "\tctx=" )
  DTRACE_CABAC_V( uiCtx )
  DTRACE_CABAC_T( "\tetype=" )
  DTRACE_CABAC_V( eType )
  DTRACE_CABAC_T( "\tuiAbsPartIdx=" )
  DTRACE_CABAC_V( uiAbsPartIdx )
  DTRACE_CABAC_T( "\n" )

  pcCU->setCbfSubParts( uiSymbol << uiTrDepth, eType, uiAbsPartIdx, uiDepth );
}
#endif

#if HHI_SYS_PIPE
#if HHI_SYS_PIPE_LC_MODE
UInt
TDecSbac::xParseLastPos( const UInt uiLog2BlkSize, const TextType eTType, const UInt uiScanIdx )
{
  UInt uiLast = 1;
  UInt uiLstX = 0;
  UInt uiLstY = 0;
  const UInt uiMax = ( 1 << uiLog2BlkSize ) - 1;
  const UInt uiCtx = g_uiCtxXYOffset[ 7 - uiLog2BlkSize ];
  ContextModel *pCtxLastX = m_cCuCtxLastX.get( 0, eTType ) + uiCtx;
  ContextModel *pCtxLastY = m_cCuCtxLastY.get( 0, eTType ) + uiCtx;

  do
  {
    uiLast = xDecodeBin( pCtxLastX[ g_uiCtxXY[ uiLstX++ ] ] );
  } while( 1-uiLast && uiLstX < uiMax );

  uiLstX -= uiLast;
  uiLast = 1;

  do
  {
    uiLast = xDecodeBin( pCtxLastY[ g_uiCtxXY[ uiLstY++ ] ] );
  } while( 1-uiLast && uiLstY < uiMax );

  uiLstY -= uiLast;

#if HHI_SYS_PIPE
  if( uiScanIdx == SCAN_ZIGZAG )
  {
    const UInt uiSum = uiLstY + uiLstX;

    if( uiSum <= uiMax )
    {
      uiLast  = ( ( uiSum + 1 ) * uiSum ) >> 1;
      uiLast += uiSum & 1 ? uiLstY : uiLstX;
    }
    else
    {
      uiLast  = ( uiMax + 1 ) << uiLog2BlkSize;
      const UInt uiLastSum2 = ( ( uiMax + 1 ) << 1 ) - 2 - uiSum;
      uiLast -= ( ( uiLastSum2 + 1 ) * uiLastSum2 ) >> 1;
      uiLast -= uiLastSum2 & 1 ? ( uiMax + 1 ) - uiLstY : ( uiMax + 1 ) - uiLstX;
    }
  }
  else
  {
    uiLast = uiLstX + ( uiLstY << uiLog2BlkSize );
  }
#else
  uiLast = uiScanIdx == SCAN_VER ? uiLstY + ( uiLstX << uiLog2BlkSize ) : uiLstX + ( uiLstY << uiLog2BlkSize );
#endif
  return uiLast;
}
#else
Void TDecSbac::xParseRemainder( UInt uiScanPosLast, UInt uiScanIdx, UInt uiLog2BlockSize, TCoeff *pcCoef, TextType eTType )
{
  UInt *puiScan = g_auiSigLastScan[uiScanIdx][uiLog2BlockSize-1];
  UInt uiNumGrOneInSet = 0;
  const UInt uiNumSetsMinus1 =  uiScanPosLast >> 4;
  UInt uiCtxSet = 5 * (uiScanPosLast > 15 ? 3 : 0 );
  UInt c1 = 1;
  UInt c2 = 0;
  UInt uiGoRiceParam = 0;
  Int iScanPos = (uiNumSetsMinus1 << 4) + 15;
  for( Int iSet = uiNumSetsMinus1; iSet >= 0; )
  {
    for( UInt uiCount = 0; uiCount < 16; ++uiCount, --iScanPos )
    {
      UInt uiBlkPos  = puiScan[iScanPos];
      if( pcCoef[uiBlkPos] )
      {
        UInt uiLevel = 1;
        UInt uiSymbol;
        UInt uiCtx    = min<UInt>(c1, 4);
      uiSymbol = xDecodeBin( m_cCUOneSCModel.get( 0, eTType, uiCtxSet + uiCtx ) );
        if( uiSymbol )
        {
          c1 = 0;
          uiCtx  = min<UInt>(c2, 4);
          c2++;
        uiSymbol = xDecodeBin( m_cCUAbsSCModel.get( 0, eTType, uiCtxSet + uiCtx ) );
          if( uiSymbol )
          {
            xReadGoRiceExGolomb( uiLevel, uiGoRiceParam );
            uiLevel += 3;
          }
          else
          {
            uiLevel = 2;
          }
          uiNumGrOneInSet++;
        }
        else if( c1 )
        {
          c1++;
        }
        UInt uiSign;
        uiSign = xDecodeEPBin();
        pcCoef[uiBlkPos] = uiSign ? -Int( uiLevel ) : uiLevel;
      }
    }
    --iSet;
    uiCtxSet = (iSet ? 3 : 0 ) + min<int>( 2, uiNumGrOneInSet );
    uiCtxSet *= 5;
    uiNumGrOneInSet = 0;
    c1 = 1;
    c2 = 0;
    uiGoRiceParam = 0;
  }
}

__inline Void TDecSbac::parseLastSignificantXY( UInt& uiPosLastX, UInt& uiPosLastY, const UInt uiWidth, const TextType eTType, const UInt uiCTXIdx, const UInt uiScanIdx )
{
  UInt uiLast;
  const UInt uiCtxOffset = g_uiCtxXYOffset[uiCTXIdx];

  for(uiPosLastX=0; uiPosLastX<uiWidth-1; uiPosLastX++)
  {
    uiLast = xDecodeBin( m_cCuCtxLastX.get( 0, eTType, uiCtxOffset + g_uiCtxXY[uiPosLastX] ) );
    if(uiLast)
    {
      break;
    }
  }

  for(uiPosLastY=0; uiPosLastY<uiWidth-1; uiPosLastY++)
  {
    uiLast = xDecodeBin( m_cCuCtxLastY.get( 0, eTType, uiCtxOffset + g_uiCtxXY[uiPosLastY] ) );
    if(uiLast)
    {
      break;
    }
  }

  if( uiScanIdx == SCAN_VER )
  {
    swap( uiPosLastX, uiPosLastY );
  }
}
#endif
#else
#if PCP_SIGMAP_SIMPLE_LAST
/** Parse (X,Y) position of the last significant coefficient
 * \param uiPosLastX reference to X component of last coefficient
 * \param uiPosLastY reference to Y component of last coefficient
 * \param uiWidth block width
 * \param eTType plane type / luminance or chrominance
 * \param uiCTXIdx block size context
 * \param uiScanIdx scan type (zig-zag, hor, ver)
 * \returns Void
 * This method decodes the X and Y component within a block of the last significant coefficient.
 */
__inline Void TDecSbac::parseLastSignificantXY( UInt& uiPosLastX, UInt& uiPosLastY, const UInt uiWidth, const TextType eTType, const UInt uiCTXIdx, const UInt uiScanIdx )
{
  UInt uiLast;
  const UInt uiCtxOffset = g_uiCtxXYOffset[uiCTXIdx];

  for(uiPosLastX=0; uiPosLastX<uiWidth-1; uiPosLastX++)
  {
    m_pcTDecBinIf->decodeBin( uiLast, m_cCuCtxLastX.get( 0, eTType, uiCtxOffset + g_uiCtxXY[uiPosLastX] ) );
    if(uiLast)
    {
      break;
    }
  }

  for(uiPosLastY=0; uiPosLastY<uiWidth-1; uiPosLastY++)
  {
    m_pcTDecBinIf->decodeBin( uiLast, m_cCuCtxLastY.get( 0, eTType, uiCtxOffset + g_uiCtxXY[uiPosLastY] ) );
    if(uiLast)
    {
      break;
    }
  }

  if( uiScanIdx == SCAN_VER )
  {
    swap( uiPosLastX, uiPosLastY );
  }
}
#endif
#endif

#if HHI_SYS_PIPE
#if HHI_SYS_PIPE_LC_MODE
Void
TDecSbac::parseCoeffNxN( TComDataCU* pcCU, TCoeff* pcCoef, UInt uiAbsPartIdx, UInt uiWidth, UInt uiHeight, UInt uiDepth, TextType eTType )
{
  DTRACE_CABAC_V( g_nSymbolCounter++ )
  DTRACE_CABAC_T( "\tparseCoeffNxN()\teType=" )
  DTRACE_CABAC_V( eTType )
  DTRACE_CABAC_T( "\twidth=" )
  DTRACE_CABAC_V( uiWidth )
  DTRACE_CABAC_T( "\theight=" )
  DTRACE_CABAC_V( uiHeight )
  DTRACE_CABAC_T( "\tdepth=" )
  DTRACE_CABAC_V( uiDepth )
  DTRACE_CABAC_T( "\tabspartidx=" )
  DTRACE_CABAC_V( uiAbsPartIdx )
  DTRACE_CABAC_T( "\ttoCU-X=" )
  DTRACE_CABAC_V( pcCU->getCUPelX() )
  DTRACE_CABAC_T( "\ttoCU-Y=" )
  DTRACE_CABAC_V( pcCU->getCUPelY() )
  DTRACE_CABAC_T( "\tCU-addr=" )
  DTRACE_CABAC_V(  pcCU->getAddr() )
  DTRACE_CABAC_T( "\tinCU-X=" )
  DTRACE_CABAC_V( g_auiRasterToPelX[ g_auiZscanToRaster[uiAbsPartIdx] ] )
  DTRACE_CABAC_T( "\tinCU-Y=" )
  DTRACE_CABAC_V( g_auiRasterToPelY[ g_auiZscanToRaster[uiAbsPartIdx] ] )
  DTRACE_CABAC_T( "\tpredmode=" )
  DTRACE_CABAC_V(  pcCU->getPredictionMode( uiAbsPartIdx ) )
  DTRACE_CABAC_T( "\n" )

  //----- parse significance map -----
  const TextType eTT           = eTType == TEXT_LUMA ? TEXT_LUMA : ( eTType == TEXT_NONE ? TEXT_NONE : TEXT_CHROMA );
  const UInt     uiLog2BlkSize = g_aucConvertToBit[ uiWidth ] + 2;
  const UInt     uiOffset      = uiWidth - 4;
  const UInt     uiScanIdx     = pcCU->getCoefScanIdx( uiAbsPartIdx, uiWidth, eTT==TEXT_LUMA, pcCU->isIntra( uiAbsPartIdx ) );
  const UInt     uiLast        = xParseLastPos( uiLog2BlkSize, eTT, uiScanIdx );

  UInt uiSCntSig = 0;
  UInt uiSCntOne = 0;
  UInt uiSCntTwo = 0;
  UInt auiScanSig[ 1024 ];
  UInt auiScanOne[ 1024 ];
  UInt auiScanTwo[ 1024 ];

  //===== decode significance flags =====
  {
    const ContextModel *pCtxSig = m_cCUSigSCModel.get( 0, eTT ) + g_auiSigCtxOffset[ uiLog2BlkSize - 2 ];
    const UInt *pauiSigScan     = g_auiSigLastScan[ uiScanIdx ][ uiLog2BlkSize-1 ];

    const UInt  uiNumMultiBins  = 4;

    if( uiLast > 0 )
    {
      {
        const UInt uiSymbol = xDecodeBin( pCtxSig[ 0 ], 1 );
        SIG_BIN( 0 );
      }
      if( uiLast == 2 )
      {
        const UInt uiSymbol = xDecodeBin( pCtxSig[ 1 ], 1 );
        SIG_BIN( 0 );
      }
      else if( uiLast > 2 )
      {
        {
          const UInt uiSymbol = xDecodeBin( pCtxSig[ 1 ], 2 );
          SIG_BIN( 0 );
          SIG_BIN( 1 );
        }
        UInt       uiCtx     = 2;
        UInt       uiPos     = 3;
        UInt       uiRemain  = ( uiLast - 3 ) % uiNumMultiBins;
        const UInt uiMaxPos = uiLast - uiRemain;

        while( uiPos < uiMaxPos )
        {
          uiCtx  += ( ( uiPos + 2 ) >> uiCtx ) - 1;
          const UInt uiSymbol = xDecodeBin( pCtxSig[ uiCtx ], uiNumMultiBins );
          SIG_BIN( 0 );
          SIG_BIN( 1 );
          SIG_BIN( 2 );
          SIG_BIN( 3 );
          uiPos += uiNumMultiBins;
        }
        if( uiRemain )
        {
          uiCtx  = uiCtx + ( ( uiPos + 2 ) >> uiCtx ) - 1;
          UInt uiSymbol   = xDecodeBin( pCtxSig[ uiCtx ], uiRemain );
          UInt uiBinsLeft = uiRemain;

          while( uiBinsLeft-- )
          {
            const UInt uiBlkPos = *pauiSigScan++;
            GET_BIN( auiScanSig, uiSCntSig )
          }
        }
      }
      const UInt uiBlkPos = *pauiSigScan++;
      pcCoef[ uiBlkPos ]++;
      auiScanSig[ uiSCntSig++ ] = uiBlkPos;
    }
    else
    {
      pcCoef[ 0 ]++;
      auiScanSig[ uiSCntSig++ ] = 0;
    }
  }

  //===== code level information =====
  {
    {
      const UInt *pauiScan  = auiScanSig + uiSCntSig - 1;
      const ContextModel *pCtxOne = m_cCUOneSCModel.get( 0, eTT ) + uiOffset;
#if HHI_SYS_PIPE_ABSX_ALTCTX
      if ( uiSCntSig > 3 )
      {
        UInt        uiRemain      = ( uiSCntSig - 3 ) % 4;
        UInt        uiCtx         = 0;
        UInt        uiCtxShiftTmp = uiSCntSig >> 1;
        while( uiCtxShiftTmp )
        {
          uiCtx++;
          uiCtxShiftTmp >>=1;
        }
#else
      UInt        uiRemain  = uiSCntSig % 4;
#endif
      UInt        uiScanPos = uiSCntSig - uiRemain;


      if( uiRemain )
      {
#if !HHI_SYS_PIPE_ABSX_ALTCTX
        const UInt uiCtx    = uiScanPos >> uiLog2BlkSize;
#endif
        UInt       uiSymbol = xDecodeBin( pCtxOne[ uiCtx ], uiRemain );

        while( uiRemain-- )
        {
          const UInt uiBlkPos = *pauiScan--;
          GET_BIN( auiScanOne, uiSCntOne )
        }
      }
#if HHI_SYS_PIPE_ABSX_ALTCTX
      while( uiScanPos > 3 )
      {
        uiCtx     -= 1 - ( uiScanPos  >> uiCtx );
        uiScanPos -= 4;
#else
      while( uiScanPos )
      {
        const UInt uiCtx      = ( uiScanPos -= 4 ) >> uiLog2BlkSize;
#endif
        const UInt uiSymbol   = xDecodeBin( pCtxOne[ uiCtx ], 4 );
        ABS_ONE( 0 );
        ABS_ONE( 1 );
        ABS_ONE( 2 );
        ABS_ONE( 3 );
      }
#if HHI_SYS_PIPE_ABSX_ALTCTX
        {
          const UInt uiSymbol   = xDecodeBin( pCtxOne[ 1 ], 2 );
          ABS_ONE( 0 );
          ABS_ONE( 1 );
        }
      }
      else if( uiSCntSig == 3 )
      {
        {
          const UInt uiSymbol   = xDecodeBin( pCtxOne[ 1 ], 2 );
          ABS_ONE( 0 );
          ABS_ONE( 1 );
        }
      }
      else if( uiSCntSig == 2 )
      {
        {
          const UInt uiSymbol  = xDecodeBin( pCtxOne[ 1 ], 1 );
          ABS_ONE( 0 );
        }
      }
      {
        const UInt uiSymbol   = xDecodeBin( pCtxOne[ 0 ], 1 );
        ABS_ONE( 0 );
      }
#endif
    }

    {
      const UInt *pauiScan  = auiScanOne;
      const ContextModel *pCtxAbs = m_cCUAbsSCModel.get( 0, eTT ) + uiOffset;
#if HHI_SYS_PIPE_ABSX_ALTCTX
      if ( uiSCntOne > 3 )
      {
        UInt        uiCtx         = 0;
        UInt        uiCtxShiftTmp = uiSCntOne >> 1;
        while( uiCtxShiftTmp )
        {
          uiCtx++;
          uiCtxShiftTmp >>=1;
        }
        UInt        uiRemain      = ( uiSCntOne - 3 ) % 4;
#else
        UInt        uiRemain  = uiSCntOne % 4;
#endif
        UInt        uiScanPos = uiSCntOne - uiRemain;

        if( uiRemain )
        {
#if !HHI_SYS_PIPE_ABSX_ALTCTX
          const UInt uiCtx    = uiScanPos >> uiLog2BlkSize;
#endif
          UInt       uiSymbol = xDecodeBin( pCtxAbs[ uiCtx ], uiRemain );

          while( uiRemain-- )
          {
            const UInt uiBlkPos = *pauiScan++;
            GET_BIN( auiScanTwo, uiSCntTwo )
          }
        }
#if HHI_SYS_PIPE_ABSX_ALTCTX
        while( uiScanPos > 3 )
        {
          uiCtx     -= 1 - ( uiScanPos  >> uiCtx );
          uiScanPos -= 4;
#else
        while( uiScanPos )
        {
          const UInt uiCtx      = ( uiScanPos -= 4 ) >> uiLog2BlkSize;
#endif
          const UInt uiSymbol   = xDecodeBin( pCtxAbs[ uiCtx ], 4 );
          ABS_TWO( 0 );
          ABS_TWO( 1 );
          ABS_TWO( 2 );
          ABS_TWO( 3 );
        }
#if HHI_SYS_PIPE_ABSX_ALTCTX
        {
          const UInt uiSymbol   = xDecodeBin( pCtxAbs[ 1 ], 2 );
          ABS_TWO( 0 );
          ABS_TWO( 1 );
        }
        {
          const UInt uiSymbol   = xDecodeBin( pCtxAbs[ 0 ], 1 );
          ABS_TWO( 0 );
        }
      }
      else if( uiSCntOne == 3 )
      {
        {
          const UInt uiSymbol   = xDecodeBin( pCtxAbs[ 1 ], 2 );
          ABS_TWO( 0 );
          ABS_TWO( 1 );
        }
        {
          const UInt uiSymbol   = xDecodeBin( pCtxAbs[ 0 ], 1 );
          ABS_TWO( 0 );
        }
      }
      else if( uiSCntOne == 2 )
      {
        {
          const UInt uiSymbol  = xDecodeBin( pCtxAbs[ 1 ], 1 );
          ABS_TWO( 0 );
        }
        {
          const UInt uiSymbol   = xDecodeBin( pCtxAbs[ 0 ], 1 );
          ABS_TWO( 0 );
        }
      }
      else if( uiSCntOne == 1 )
      {
        const UInt uiSymbol   = xDecodeBin( pCtxAbs[ 0 ], 1 );
        ABS_TWO( 0 );
      }
#endif
    }

    {
      xReadGoRiceExGolomb( pcCoef, auiScanTwo, uiSCntTwo );
    }

    {
      const UInt *pauiScan  = auiScanSig + uiSCntSig - 1;

      while( uiSCntSig > 32 )
      {
        const UInt uiSign  = xDecodeEPBin( 32 );
        for( UInt uiScanPos = 32; uiScanPos--; )
        {
          const UInt uiBlkPos = *pauiScan--;
          const UInt uiCSign  = ( uiSign >> uiScanPos ) & 1;

          if( uiCSign )
          {
            pcCoef[ uiBlkPos ] = -(Int)pcCoef[ uiBlkPos ];
          }
        }

        uiSCntSig -= 32;
      }

      const UInt uiSign  = xDecodeEPBin( uiSCntSig );
      for( UInt uiScanPos = uiSCntSig; uiScanPos--; )
      {
        const UInt uiBlkPos = *pauiScan--;
        const UInt uiCSign  = ( uiSign >> uiScanPos ) & 1;

        if( uiCSign )
        {
          pcCoef[ uiBlkPos ] = -(Int)pcCoef[ uiBlkPos ];
        }
      }
    }
  }
  return;
}
#else
Void TDecSbac::parseCoeffNxN( TComDataCU* pcCU, TCoeff* pcCoef, UInt uiAbsPartIdx, UInt uiWidth, UInt uiHeight, UInt uiDepth, TextType eTType )
{
  DTRACE_CABAC_V( g_nSymbolCounter++ )
  DTRACE_CABAC_T( "\tparseCoeffNxN()\teType=" )
  DTRACE_CABAC_V( eTType )
  DTRACE_CABAC_T( "\twidth=" )
  DTRACE_CABAC_V( uiWidth )
  DTRACE_CABAC_T( "\theight=" )
  DTRACE_CABAC_V( uiHeight )
  DTRACE_CABAC_T( "\tdepth=" )
  DTRACE_CABAC_V( uiDepth )
  DTRACE_CABAC_T( "\tabspartidx=" )
  DTRACE_CABAC_V( uiAbsPartIdx )
  DTRACE_CABAC_T( "\ttoCU-X=" )
  DTRACE_CABAC_V( pcCU->getCUPelX() )
  DTRACE_CABAC_T( "\ttoCU-Y=" )
  DTRACE_CABAC_V( pcCU->getCUPelY() )
  DTRACE_CABAC_T( "\tCU-addr=" )
  DTRACE_CABAC_V(  pcCU->getAddr() )
  DTRACE_CABAC_T( "\tinCU-X=" )
  DTRACE_CABAC_V( g_auiRasterToPelX[ g_auiZscanToRaster[uiAbsPartIdx] ] )
  DTRACE_CABAC_T( "\tinCU-Y=" )
  DTRACE_CABAC_V( g_auiRasterToPelY[ g_auiZscanToRaster[uiAbsPartIdx] ] )
  DTRACE_CABAC_T( "\tpredmode=" )
  DTRACE_CABAC_V(  pcCU->getPredictionMode( uiAbsPartIdx ) )
  DTRACE_CABAC_T( "\n" )

  if( uiWidth > pcCU->getSlice()->getSPS()->getMaxTrSize() )
  {
    uiWidth  = pcCU->getSlice()->getSPS()->getMaxTrSize();
    uiHeight = pcCU->getSlice()->getSPS()->getMaxTrSize();
  }

  UInt uiCTXIdx;

  switch(uiWidth)
  {
  case  2: uiCTXIdx = 6; break;
  case  4: uiCTXIdx = 5; break;
  case  8: uiCTXIdx = 4; break;
  case 16: uiCTXIdx = 3; break;
  case 32: uiCTXIdx = 2; break;
  case 64: uiCTXIdx = 1; break;
  default: uiCTXIdx = 0; break;
  }

  eTType = eTType == TEXT_LUMA ? TEXT_LUMA : ( eTType == TEXT_NONE ? TEXT_NONE : TEXT_CHROMA );

  //----- parse significance map -----
  const UInt  uiLog2BlockSize   = g_aucConvertToBit[ uiWidth ] + 2;
  const UInt  uiMaxNumCoeff     = 1 << ( uiLog2BlockSize << 1 );
  const UInt  uiMaxNumCoeffM1   = uiMaxNumCoeff - 1;
#if !PCP_SIGMAP_SIMPLE_LAST
  bool        bLastReceived     = false;
#endif
#if QC_MDCS
  const UInt uiScanIdx = pcCU->getCoefScanIdx(uiAbsPartIdx, uiWidth, eTType==TEXT_LUMA, pcCU->isIntra(uiAbsPartIdx));
#endif //QC_MDCS
  Int sigCoeffCount = 0;

#if PCP_SIGMAP_SIMPLE_LAST
  //===== decode last significant =====
  UInt uiPosLastX, uiPosLastY;
  parseLastSignificantXY( uiPosLastX, uiPosLastY, uiWidth, eTType, uiCTXIdx, uiScanIdx );
  UInt uiBlkPosLast      = uiPosLastX + (uiPosLastY<<uiLog2BlockSize);
  pcCoef[ uiBlkPosLast ] = 1;
  sigCoeffCount++;

  //===== decode significance flags =====
  UInt uiScanPos = 0;
  for( ; uiScanPos < uiMaxNumCoeffM1; uiScanPos++ )
  {
#if QC_MDCS
    UInt uiBlkPos = g_auiSigLastScan[uiScanIdx][uiLog2BlockSize-1][uiScanPos];
#else
    UInt  uiBlkPos  = g_auiFrameScanXY[ uiLog2BlockSize-1 ][ uiScanPos ];
#endif //QC_MDCS
    if( uiBlkPosLast == uiBlkPos )
    {
      break;
    }
    UInt  uiPosY    = uiBlkPos >> uiLog2BlockSize;
    UInt  uiPosX    = uiBlkPos - ( uiPosY << uiLog2BlockSize );
    UInt  uiSig     = 0;
    UInt  uiCtxSig  = TComTrQuant::getSigCtxInc( pcCoef, uiPosX, uiPosY, uiLog2BlockSize, uiWidth );
#if SIMPLE_CONTEXT_SIG
    uiSig = xDecodeBin( m_cCUSigSCModel.get( 0, eTType, uiCtxSig ) );
#else
    uiSig = xDecodeBin( m_cCUSigSCModel.get( uiCTXIdx, eTType, uiCtxSig ) );
#endif
    pcCoef[ uiBlkPos ] = uiSig;
    sigCoeffCount += uiSig;
  }

#else
  for( UInt uiScanPos = 0; uiScanPos < uiMaxNumCoeffM1; uiScanPos++ )
  {
#if QC_MDCS
    UInt uiBlkPos = g_auiSigLastScan[uiScanIdx][uiLog2BlockSize-1][uiScanPos];
#else
    UInt  uiBlkPos  = g_auiFrameScanXY[ uiLog2BlockSize-1 ][ uiScanPos ];
#endif //QC_MDCS
    UInt  uiPosY    = uiBlkPos >> uiLog2BlockSize;
    UInt  uiPosX    = uiBlkPos - ( uiPosY << uiLog2BlockSize );

    //===== code significance flag =====
    UInt  uiSig     = 0;
    UInt  uiCtxSig  = TComTrQuant::getSigCtxInc( pcCoef, uiPosX, uiPosY, uiLog2BlockSize, uiWidth );
#if SIMPLE_CONTEXT_SIG
    m_pcTDecBinIf->decodeBin( uiSig, m_cCUSigSCModel.get( 0, eTType, uiCtxSig ) );
#else
    m_pcTDecBinIf->decodeBin( uiSig, m_cCUSigSCModel.get( uiCTXIdx, eTType, uiCtxSig ) );
#endif
    if( uiSig )
    {
      pcCoef[ uiBlkPos ] = 1;
      sigCoeffCount++;

      //===== code last flag =====
      UInt  uiLast     = 0;
      UInt  uiCtxLast  = TComTrQuant::getLastCtxInc( uiPosX, uiPosY, uiLog2BlockSize );
      m_pcTDecBinIf->decodeBin( uiLast, m_cCULastSCModel.get( uiCTXIdx, eTType, uiCtxLast ) );

      if( uiLast )
      {
        bLastReceived = true;
        break;
      }
    }
  }
  if( !bLastReceived )
  {
    pcCoef[ uiMaxNumCoeffM1 ] = 1;
    sigCoeffCount++;
  }
#endif
  /*
   * Sign and bin0 PCP (Section 3.2 and 3.3 of JCTVC-B088)
   */
  xParseRemainder( uiScanPos, uiScanIdx, uiLog2BlockSize, pcCoef, eTType );
  return;
}
#endif
#else
Void TDecSbac::parseCoeffNxN( TComDataCU* pcCU, TCoeff* pcCoef, UInt uiAbsPartIdx, UInt uiWidth, UInt uiHeight, UInt uiDepth, TextType eTType )
{
  DTRACE_CABAC_V( g_nSymbolCounter++ )
  DTRACE_CABAC_T( "\tparseCoeffNxN()\teType=" )
  DTRACE_CABAC_V( eTType )
  DTRACE_CABAC_T( "\twidth=" )
  DTRACE_CABAC_V( uiWidth )
  DTRACE_CABAC_T( "\theight=" )
  DTRACE_CABAC_V( uiHeight )
  DTRACE_CABAC_T( "\tdepth=" )
  DTRACE_CABAC_V( uiDepth )
  DTRACE_CABAC_T( "\tabspartidx=" )
  DTRACE_CABAC_V( uiAbsPartIdx )
  DTRACE_CABAC_T( "\ttoCU-X=" )
  DTRACE_CABAC_V( pcCU->getCUPelX() )
  DTRACE_CABAC_T( "\ttoCU-Y=" )
  DTRACE_CABAC_V( pcCU->getCUPelY() )
  DTRACE_CABAC_T( "\tCU-addr=" )
  DTRACE_CABAC_V(  pcCU->getAddr() )
  DTRACE_CABAC_T( "\tinCU-X=" )
  DTRACE_CABAC_V( g_auiRasterToPelX[ g_auiZscanToRaster[uiAbsPartIdx] ] )
  DTRACE_CABAC_T( "\tinCU-Y=" )
  DTRACE_CABAC_V( g_auiRasterToPelY[ g_auiZscanToRaster[uiAbsPartIdx] ] )
  DTRACE_CABAC_T( "\tpredmode=" )
  DTRACE_CABAC_V(  pcCU->getPredictionMode( uiAbsPartIdx ) )
  DTRACE_CABAC_T( "\n" )

  if( uiWidth > pcCU->getSlice()->getSPS()->getMaxTrSize() )
  {
    uiWidth  = pcCU->getSlice()->getSPS()->getMaxTrSize();
    uiHeight = pcCU->getSlice()->getSPS()->getMaxTrSize();
  }

  UInt uiCTXIdx;

  switch(uiWidth)
  {
    case  2: uiCTXIdx = 6; break;
    case  4: uiCTXIdx = 5; break;
    case  8: uiCTXIdx = 4; break;
    case 16: uiCTXIdx = 3; break;
    case 32: uiCTXIdx = 2; break;
    case 64: uiCTXIdx = 1; break;
    default: uiCTXIdx = 0; break;
  }

  eTType = eTType == TEXT_LUMA ? TEXT_LUMA : ( eTType == TEXT_NONE ? TEXT_NONE : TEXT_CHROMA );

  //----- parse significance map -----
  const UInt  uiLog2BlockSize   = g_aucConvertToBit[ uiWidth ] + 2;
  const UInt  uiMaxNumCoeff     = 1 << ( uiLog2BlockSize << 1 );
  const UInt  uiMaxNumCoeffM1   = uiMaxNumCoeff - 1;
  const UInt  uiNum4x4Blk       = max<UInt>( 1, uiMaxNumCoeff >> 4 );
#if !PCP_SIGMAP_SIMPLE_LAST
  bool        bLastReceived     = false;
#endif
#if QC_MDCS
  const UInt uiScanIdx = pcCU->getCoefScanIdx(uiAbsPartIdx, uiWidth, eTType==TEXT_LUMA, pcCU->isIntra(uiAbsPartIdx));
#endif //QC_MDCS
  Int sigCoeffCount = 0;

#if PCP_SIGMAP_SIMPLE_LAST
  //===== decode last significant =====
  UInt uiPosLastX, uiPosLastY;
  parseLastSignificantXY( uiPosLastX, uiPosLastY, uiWidth, eTType, uiCTXIdx, uiScanIdx );
  UInt uiBlkPosLast      = uiPosLastX + (uiPosLastY<<uiLog2BlockSize);
  pcCoef[ uiBlkPosLast ] = 1;
  sigCoeffCount++;

  //===== decode significance flags =====
  for( UInt uiScanPos = 0; uiScanPos < uiMaxNumCoeffM1; uiScanPos++ )
  {
#if QC_MDCS
    UInt uiBlkPos = g_auiSigLastScan[uiScanIdx][uiLog2BlockSize-1][uiScanPos];
#else
    UInt  uiBlkPos  = g_auiFrameScanXY[ uiLog2BlockSize-1 ][ uiScanPos ];
#endif //QC_MDCS
    if( uiBlkPosLast == uiBlkPos )
    {
      break;
    }
    UInt  uiPosY    = uiBlkPos >> uiLog2BlockSize;
    UInt  uiPosX    = uiBlkPos - ( uiPosY << uiLog2BlockSize );
    UInt  uiSig     = 0;
    UInt  uiCtxSig  = TComTrQuant::getSigCtxInc( pcCoef, uiPosX, uiPosY, uiLog2BlockSize, uiWidth );
#if SIMPLE_CONTEXT_SIG
    m_pcTDecBinIf->decodeBin( uiSig, m_cCUSigSCModel.get( 0, eTType, uiCtxSig ) );
#else
    m_pcTDecBinIf->decodeBin( uiSig, m_cCUSigSCModel.get( uiCTXIdx, eTType, uiCtxSig ) );
#endif
    pcCoef[ uiBlkPos ] = uiSig;
    sigCoeffCount += uiSig;
  }

#else
  for( UInt uiScanPos = 0; uiScanPos < uiMaxNumCoeffM1; uiScanPos++ )
  {
#if QC_MDCS
    UInt uiBlkPos = g_auiSigLastScan[uiScanIdx][uiLog2BlockSize-1][uiScanPos];
#else
    UInt  uiBlkPos  = g_auiFrameScanXY[ uiLog2BlockSize-1 ][ uiScanPos ];
#endif //QC_MDCS
    UInt  uiPosY    = uiBlkPos >> uiLog2BlockSize;
    UInt  uiPosX    = uiBlkPos - ( uiPosY << uiLog2BlockSize );

    //===== code significance flag =====
    UInt  uiSig     = 0;
    UInt  uiCtxSig  = TComTrQuant::getSigCtxInc( pcCoef, uiPosX, uiPosY, uiLog2BlockSize, uiWidth );
#if SIMPLE_CONTEXT_SIG
    m_pcTDecBinIf->decodeBin( uiSig, m_cCUSigSCModel.get( 0, eTType, uiCtxSig ) );
#else
    m_pcTDecBinIf->decodeBin( uiSig, m_cCUSigSCModel.get( uiCTXIdx, eTType, uiCtxSig ) );
#endif
    if( uiSig )
    {
      pcCoef[ uiBlkPos ] = 1;
      sigCoeffCount++;

      //===== code last flag =====
      UInt  uiLast     = 0;
      UInt  uiCtxLast  = TComTrQuant::getLastCtxInc( uiPosX, uiPosY, uiLog2BlockSize );
      m_pcTDecBinIf->decodeBin( uiLast, m_cCULastSCModel.get( uiCTXIdx, eTType, uiCtxLast ) );

      if( uiLast )
      {
        bLastReceived = true;
        break;
      }
    }
  }
  if( !bLastReceived )
  {
    pcCoef[ uiMaxNumCoeffM1 ] = 1;
    sigCoeffCount++;
  }
#endif

  /*
   * Sign and bin0 PCP (Section 3.2 and 3.3 of JCTVC-B088)
   */
  Int  c1, c2;
  UInt uiSign;
  UInt uiLevel;
#if E253
  UInt uiGoRiceParam = 0;
#endif

  if( uiNum4x4Blk > 1 )
  {
    Bool b1stBlk  = true;
    UInt uiNumOne = 0;

    for( UInt uiSubBlk = 0; uiSubBlk < uiNum4x4Blk && sigCoeffCount > 0; uiSubBlk++ )
    {
      UInt uiCtxSet    = 0;
      UInt uiSubNumSig = 0;
      UInt uiSubPosX   = 0;
      UInt uiSubPosY   = 0;
#if E253
      uiGoRiceParam    = 0;
#endif

      uiSubPosX = g_auiFrameScanX[ g_aucConvertToBit[ uiWidth ] - 1 ][ uiSubBlk ] << 2;
      uiSubPosY = g_auiFrameScanY[ g_aucConvertToBit[ uiWidth ] - 1 ][ uiSubBlk ] << 2;

      TCoeff* piCurr = &pcCoef[ uiSubPosX + uiSubPosY * uiWidth ];

      for( UInt uiY = 0; uiY < 4; uiY++ )
      {
        for( UInt uiX = 0; uiX < 4; uiX++ )
        {
          if( piCurr[ uiX ] )
          {
            uiSubNumSig++;
          }
        }
        piCurr += uiWidth;
      }
      sigCoeffCount -= uiSubNumSig;

      if( uiSubNumSig > 0 )
      {
        c1 = 1;
        c2 = 0;

        if( b1stBlk )
        {
          b1stBlk  = false;
          uiCtxSet = 5;
        }
        else
        {
          uiCtxSet = ( uiNumOne >> 2 ) + 1;
          uiNumOne = 0;
        }

        TCoeff sigCoeff[16];
        ContextModel *baseCtxMod;

        baseCtxMod = m_cCUOneSCModel.get( 0, eTType ) + 5 * uiCtxSet;
        for (Int idx = 0; idx < uiSubNumSig; idx++)
        {
          m_pcTDecBinIf->decodeBin( uiLevel, baseCtxMod[c1] );
          if( uiLevel == 1 )
          {
            c1 = 0;
          }
          else if( c1 & 3)
          {
            c1++;
          }
          sigCoeff[ idx ] = uiLevel + 1;
        }

        if (c1 == 0)
        {
          baseCtxMod = m_cCUAbsSCModel.get( 0, eTType ) + 5 * uiCtxSet;
          for (Int idx = 0; idx < uiSubNumSig; idx++)
          {
            if( sigCoeff[idx] == 2 )
            {
#if E253
              m_pcTDecBinIf->decodeBin( uiLevel, baseCtxMod[c2] );

              if( uiLevel )
              {
                xReadGoRiceExGolomb( uiLevel, uiGoRiceParam );
                sigCoeff[idx] = uiLevel + 3;
              }
#else
              xReadExGolombLevel( uiLevel, baseCtxMod[c2] );
              sigCoeff[idx] = uiLevel + 2;
#endif
              c2 += (c2 < 4);
              uiNumOne++;
            }
          }
        }

        Int idx = 0;
        for( UInt uiScanPos = 0; uiScanPos < 16; uiScanPos++ )
        {
          UInt  uiBlkPos  = g_auiFrameScanXY[ 1 ][ 15 - uiScanPos ];
          UInt  uiPosY    = uiBlkPos >> 2;
          UInt  uiPosX    = uiBlkPos & 3;
          UInt  uiIndex   = ((uiSubPosY + uiPosY) << uiLog2BlockSize) + uiSubPosX + uiPosX;

          uiLevel = pcCoef[ uiIndex ];

          if( uiLevel )
          {
            m_pcTDecBinIf->decodeBinEP( uiSign );
            TCoeff val = sigCoeff[idx++];
            pcCoef[ uiIndex ] = ( uiSign ? -(Int)val : (Int)val );
          }
        }
      }
    }
  }
  else
  {
    c1 = 1;
    c2 = 0;
    UInt uiSubNumSig = sigCoeffCount;
    TCoeff sigCoeff[16];
    ContextModel *baseCtxMod;

    baseCtxMod = m_cCUOneSCModel.get( 0, eTType );
    for (Int idx = 0; idx < uiSubNumSig; idx++)
    {
      m_pcTDecBinIf->decodeBin( uiLevel, baseCtxMod[c1] );
      if( uiLevel == 1 )
      {
        c1 = 0;
      }
      else if( c1 & 3)
      {
        c1++;
      }
      sigCoeff[ idx ] = uiLevel + 1;
    }

    if (c1 == 0)
    {
      baseCtxMod = m_cCUAbsSCModel.get( 0, eTType );
      for (Int idx = 0; idx < uiSubNumSig; idx++)
      {
        if( sigCoeff[idx] == 2 )
        {
#if E253
          m_pcTDecBinIf->decodeBin( uiLevel, baseCtxMod[c2] );

          if( uiLevel )
          {
            xReadGoRiceExGolomb( uiLevel, uiGoRiceParam );
            sigCoeff[idx] = uiLevel + 3;
          }
#else
          xReadExGolombLevel( uiLevel, baseCtxMod[c2] );
          sigCoeff[idx] = uiLevel + 2;
#endif
          c2 += (c2 < 4);
        }
      }
    }

    Int idx = 0;
    for( UInt uiScanPos = 0; uiScanPos < 16; uiScanPos++ )
    {
      UInt uiIndex = g_auiFrameScanXY[ 1 ][ 15 - uiScanPos ];
      uiLevel = pcCoef[ uiIndex ];

      if( uiLevel )
      {
        m_pcTDecBinIf->decodeBinEP( uiSign );
        TCoeff val = sigCoeff[idx++];
        pcCoef[ uiIndex ] = ( uiSign ? -(Int)val : (Int)val );
      }
    }
  }

  return;
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseAlfFlag( UInt& ruiVal )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cALFFlagSCModel.get();
#else
  ContextModel *pCtx = m_cALFFlagSCModel.get();
#endif
  ruiVal = xDecodeBin( *pCtx );
  return;
}
#else
Void TDecSbac::parseAlfFlag (UInt& ruiVal)
{
  UInt uiSymbol;
  m_pcTDecBinIf->decodeBin( uiSymbol, m_cALFFlagSCModel.get( 0, 0, 0 ) );

  ruiVal = uiSymbol;
}
#endif

#if TSB_ALF_HEADER
#if HHI_SYS_PIPE
Void
TDecSbac::parseAlfFlagNum( UInt& ruiVal, UInt uiMinValue, UInt uiDepth )
{
  UInt uiLength   = 0;
  UInt uiMaxValue = ( uiMinValue << ( uiDepth << 1 ) );
  UInt uiTemp     = uiMaxValue - uiMinValue;

  for( UInt ui=0; ui < 32; ui++ )
  {
    if( uiTemp & 0x1 )
    {
      uiLength = ui + 1;
    }
    uiTemp >>= 1;
  }

  ruiVal = 0;
  UInt uiBit;

  if( uiLength )
  {
    while( uiLength-- )
    {
      uiBit = xDecodeEPBin();
      ruiVal += uiBit << uiLength;
    }
  }
  else
  {
    ruiVal = 0;
  }
  ruiVal += uiMinValue;
  return;
}
#else
Void TDecSbac::parseAlfFlagNum( UInt& ruiVal, UInt minValue, UInt depth )
{
  UInt uiLength = 0;
  UInt maxValue = (minValue << (depth*2));
  UInt temp = maxValue - minValue;
  for(UInt i=0; i<32; i++)
  {
    if(temp&0x1)
    {
      uiLength = i+1;
    }
    temp = (temp >> 1);
  }
  ruiVal = 0;
  UInt uiBit;
  if(uiLength)
  {
    while( uiLength-- )
    {
      m_pcTDecBinIf->decodeBinEP( uiBit );
      ruiVal += uiBit << uiLength;
    }
  }
  else
  {
    ruiVal = 0;
  }
  ruiVal += minValue;
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseAlfCtrlFlag( UInt &ruiAlfCtrlFlag )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cCUAlfCtrlFlagSCModel.get();
#else
  ContextModel *pCtx = m_cCUAlfCtrlFlagSCModel.get();
#endif
  ruiAlfCtrlFlag = xDecodeBin( *pCtx );
  return;
}
#else
Void TDecSbac::parseAlfCtrlFlag( UInt &ruiAlfCtrlFlag )
{
  UInt uiSymbol;
  m_pcTDecBinIf->decodeBin( uiSymbol, m_cCUAlfCtrlFlagSCModel.get( 0, 0, 0 ) );
  ruiAlfCtrlFlag = uiSymbol;
}
#endif
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseAlfUvlc( UInt& ruiVal )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cALFUvlcSCModel.get();
#else
  ContextModel *pCtx = m_cALFUvlcSCModel.get();
#endif
  UInt uiVal = xDecodeBin( *pCtx );
  UInt uiCnt = 0;

  if( !uiVal )
  {
    ruiVal = 0;
    return;
  }

  pCtx++;

  do
  {
    uiVal++;
    uiCnt = xDecodeBin( *pCtx );
  } while ( uiCnt );

  ruiVal = --uiVal;
  return;
}
#else
Void TDecSbac::parseAlfUvlc (UInt& ruiVal)
{
  UInt uiCode;
  Int  i;

  m_pcTDecBinIf->decodeBin( uiCode, m_cALFUvlcSCModel.get( 0, 0, 0 ) );
  if ( uiCode == 0 )
  {
    ruiVal = 0;
    return;
  }

  i=1;
  while (1)
  {
    m_pcTDecBinIf->decodeBin( uiCode, m_cALFUvlcSCModel.get( 0, 0, 1 ) );
    if ( uiCode == 0 ) break;
    i++;
  }

  ruiVal = i;
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseAlfSvlc( Int&  riVal )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cALFSvlcSCModel.get();
#else
  ContextModel *pCtx = m_cALFSvlcSCModel.get();
#endif
  UInt uiVal = xDecodeBin( *pCtx );
  UInt uiCnt = 0;

  if( !uiVal )
  {
    riVal = 0;
    return;
  }

  pCtx++;
  const UInt uiSign = xDecodeBin( *pCtx );
  pCtx++;

  do
  {
    uiVal++;
    uiCnt = xDecodeBin( *pCtx );
  } while ( uiCnt );

  riVal = uiSign ? -(Int)--uiVal : (Int)--uiVal;
  return;
}
#else
Void TDecSbac::parseAlfSvlc (Int&  riVal)
{
  UInt uiCode;
  Int  iSign;
  Int  i;

  m_pcTDecBinIf->decodeBin( uiCode, m_cALFSvlcSCModel.get( 0, 0, 0 ) );

  if ( uiCode == 0 )
  {
    riVal = 0;
    return;
  }

  // read sign
  m_pcTDecBinIf->decodeBin( uiCode, m_cALFSvlcSCModel.get( 0, 0, 1 ) );

  if ( uiCode == 0 ) iSign =  1;
  else               iSign = -1;

  // read magnitude
  i=1;
  while (1)
  {
    m_pcTDecBinIf->decodeBin( uiCode, m_cALFSvlcSCModel.get( 0, 0, 2 ) );
    if ( uiCode == 0 ) break;
    i++;
  }

  riVal = i*iSign;
}
#endif

#if MTK_SAO
#if HHI_SYS_PIPE
Void
TDecSbac::parseAoFlag( UInt& ruiVal )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cAOFlagSCModel.get();
#else
  ContextModel *pCtx = m_cAOFlagSCModel.get();
#endif
  ruiVal = xDecodeBin( *pCtx );
  return;
}
#else
Void TDecSbac::parseAoFlag (UInt& ruiVal)
{
  UInt uiSymbol;
  m_pcTDecBinIf->decodeBin( uiSymbol, m_cAOFlagSCModel.get( 0, 0, 0 ) );

  ruiVal = uiSymbol;
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseAoUvlc( UInt& ruiVal )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cAOUvlcSCModel.get();
#else
  ContextModel *pCtx = m_cAOUvlcSCModel.get();
#endif
  UInt uiVal = xDecodeBin( *pCtx );
  UInt uiCnt = 0;

  if( !uiVal )
  {
    ruiVal = 0;
    return;
  }

  pCtx++;

  do
  {
    uiVal++;
    uiCnt = xDecodeBin( *pCtx );
  } while ( uiCnt );

  ruiVal = --uiVal;
  return;
}
#else
Void TDecSbac::parseAoUvlc (UInt& ruiVal)
{
  UInt uiCode;
  Int  i;

  m_pcTDecBinIf->decodeBin( uiCode, m_cAOUvlcSCModel.get( 0, 0, 0 ) );
  if ( uiCode == 0 )
  {
    ruiVal = 0;
    return;
  }

  i=1;
  while (1)
  {
    m_pcTDecBinIf->decodeBin( uiCode, m_cAOUvlcSCModel.get( 0, 0, 1 ) );
    if ( uiCode == 0 ) break;
    i++;
  }

  ruiVal = i;
}
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::parseAoSvlc( Int&  riVal )
{
#if HHI_SYS_PIPE_LC_MODE
  const ContextModel *pCtx = m_cAOSvlcSCModel.get();
#else
  ContextModel *pCtx = m_cAOSvlcSCModel.get();
#endif
  UInt uiVal = xDecodeBin( *pCtx );
  UInt uiCnt = 0;

  if( !uiVal )
  {
    riVal = 0;
    return;
  }

  pCtx++;
  const UInt uiSign = xDecodeBin( *pCtx );
  pCtx++;

  do
  {
    uiVal++;
    uiCnt = xDecodeBin( *pCtx );
  } while ( uiCnt );

  riVal = uiSign ? -(Int)--uiVal : (Int)--uiVal;
  return;
}
#else
Void TDecSbac::parseAoSvlc (Int&  riVal)
{
  UInt uiCode;
  Int  iSign;
  Int  i;

  m_pcTDecBinIf->decodeBin( uiCode, m_cAOSvlcSCModel.get( 0, 0, 0 ) );

  if ( uiCode == 0 )
  {
    riVal = 0;
    return;
  }

  // read sign
  m_pcTDecBinIf->decodeBin( uiCode, m_cAOSvlcSCModel.get( 0, 0, 1 ) );

  if ( uiCode == 0 ) iSign =  1;
  else               iSign = -1;

  // read magnitude
  i=1;
  while (1)
  {
    m_pcTDecBinIf->decodeBin( uiCode, m_cAOSvlcSCModel.get( 0, 0, 2 ) );
    if ( uiCode == 0 ) break;
    i++;
  }

  riVal = i*iSign;
}
#endif
#endif

#if HHI_SYS_PIPE
Void
TDecSbac::xStart()
{
  m_auiCounter[ 0 ] = PIPE_INIT_CW;
  m_auiCounter[ 1 ] = PIPE_INIT_CW;
  m_auiCounter[ 2 ] = PIPE_INIT_CW;
  m_auiCounter[ 3 ] = PIPE_INIT_CW;
  m_auiCounter[ 4 ] = PIPE_INIT_CW;
  m_auiCounter[ 5 ] = PIPE_INIT_CW;
  m_auiCounter[ 6 ] = PIPE_INIT_CW;
  m_auiCounter[ 7 ] = PIPE_INIT_CW;
}

Void
TDecSbac::xFinish()
{
}

UInt
TDecSbac::xDecodeEPBin()
{
  UInt ruiBin = 0;
  m_pcBitstream->read( 1, ruiBin );
  return ruiBin;
}

UInt
TDecSbac::xDecodeEPBin( const UInt uiBins )
{
  UInt ruiBin = 0;
  m_pcBitstream->read( uiBins, ruiBin );
  return ruiBin;
}

UInt
TDecSbac::xDecode1Bin0( const UChar ucMps )
{
  UInt uiCw, uiBin, uiBits;
  m_pcBitstream->showAndFlush( 3, uiBits, (UInt*)auiBP3BitsLenTab[1] );
  uiCw = auiBP3Tab[1][uiBits];
  uiBin = uiCw >> ( PIPE_CW_NUM_BITS-1 );

  *m_puiPipe0CwCnt = uiCw << 1;
  uiBin ^= (UInt)( ucMps );
  return uiBin;
}

UInt
TDecSbac::xDecode1Bin1( const UChar ucMps )
{
  UInt uiBit = 0;

  m_pcBitstream->read( 1, uiBit );
  if( uiBit )
  {
    *m_puiPipe1CwCnt = 1 << (PIPE_CW_NUM_BITS-2);
    return 0 ^ ucMps;
  }
  else
  {
    m_pcBitstream->read( 1, uiBit );
    *m_puiPipe1CwCnt = PIPE_2BIT_CW << ( uiBit ^ 1 );
    return ( uiBit ^ 1 ) ^ ucMps;
  }
}

UInt
TDecSbac::xDecode1Bin2( const UChar ucMps )
{
  UInt uiCw, uiBin, uiBits;
  m_pcBitstream->showAndFlush( 5, uiBits, (UInt*)auiTBCBitsLenTab );
  uiCw = auiTBCBinTab[uiBits];
  uiBin = uiCw >> ( PIPE_CW_NUM_BITS-1 );

  *m_puiPipe2CwCnt = uiCw << 1;
  uiBin ^= (UInt)( ucMps );
  return uiBin;
}

UInt
TDecSbac::xDecode1Bin3( const UChar ucMps )
{
  return xDecodeEPBin();
}

UInt
TDecSbac::xDecode1Bin4( const UChar ucMps )
{
  UInt uiBit = 0;
  m_pcBitstream->read( 1, uiBit );

  if( uiBit )
  {
    *m_puiPipe4CwCnt = (UInt)PIPE_INIT_CW >> ( ( 1 << 2 ) - 1 );
    return 0 ^ ucMps;
  }
  else
  {
    m_pcBitstream->read( 2, uiBit );
    *m_puiPipe4CwCnt = ( PIPE_2BIT_CW >> uiBit ) << 1;
    return ( !!( uiBit == 0 ) ) ^ ucMps;
  }
}

UInt
TDecSbac::xDecode1Bin5( const UChar ucMps )
{
  UInt uiBit = 0;
  m_pcBitstream->read( 1, uiBit );

  if( uiBit )
  {
    *m_puiPipe5CwCnt = PIPE_INIT_CW >> ( ( 1 << 3 ) - 1 );
    return 0 ^ ucMps;
  }
  else
  {
    m_pcBitstream->read( 3, uiBit );
    *m_puiPipe5CwCnt = ( PIPE_2BIT_CW >> uiBit ) << 1;
    return ( !!( uiBit == 0 ) ) ^ ucMps;
  }
}

UInt
TDecSbac::xDecode1Bin6( const UChar ucMps )
{
  UInt uiBit = 0;
  m_pcBitstream->read( 1, uiBit );

  if( uiBit )
  {
    *m_puiPipe6CwCnt = PIPE_INIT_CW >> ( ( 1 << 4 ) - 1 );
    return 0 ^ ucMps;
  }
  else
  {
    m_pcBitstream->read( 4, uiBit );
    *m_puiPipe6CwCnt = ( PIPE_2BIT_CW >> uiBit ) << 1;
    return ( !!( uiBit == 0 ) ) ^ ucMps;
  }
}

UInt
TDecSbac::xDecode1Bin7( const UChar ucMps )
{
  UInt uiBit = 0;
  m_pcBitstream->read( 1, uiBit );

  if( uiBit )
  {
    *m_puiPipe7CwCnt = PIPE_INIT_CW >> ( ( 1 << 5 ) - 1 );
    return 0 ^ ucMps;
  }
  else
  {
    m_pcBitstream->read( 5, uiBit );
    *m_puiPipe7CwCnt = UInt( ( ( PIPE_2BIT_CW64 >> uiBit ) << 1 ) >> 32 );
    return ( !!( uiBit == 0 ) ) ^ ucMps;
  }
}

#if HHI_SYS_PIPE_LC_MODE
UInt
TDecSbac::xDecodeBin( const ContextModel &rcCtxModel )
{
  UInt uiBin;
  const UChar ucMPS = rcCtxModel.getMps();
  const UInt  uiCw  = *rcCtxModel.m_puiCwCounter;
  if( uiCw != PIPE_INIT_CW )
  {
    *rcCtxModel.m_puiCwCounter <<= 1;
    uiBin = ( uiCw >> (PIPE_CW_NUM_BITS-1) );
    uiBin ^= ucMPS;
  }
  else
  {
    uiBin = rcCtxModel.decode1Bin( ucMPS );
  }
  return uiBin;
}

UInt
TDecSbac::xDecodeBin( const ContextModel& rcCtxModel, const UInt uiBins )
{
  UInt uiSym = rcCtxModel.decodeNBin( rcCtxModel.getMps(), uiBins );
  return uiSym;
}



#define MASK_STOP_BIT(_cw) (_cw)

#define DECODE_N_NONTAB_MIRROR( _buf, _fct ) \
{ \
  uiCw ^= (UInt)( -ucMps ); \
  uiCw = uiCw & ~( PIPE_MASK_CW >> uiBinsInBuffer ); \
  *(_buf) = PIPE_INIT_CW; \
  while( uiBinsToLoad > 0 ) \
  { \
    if( *(_buf) != PIPE_INIT_CW ) \
    { \
      UInt uiCurrBin = *(_buf) & PIPE_INIT_CW; \
      *(_buf) = *(_buf) << 1; \
      uiCurrBin ^= (UInt)ucMps << ( PIPE_CW_NUM_BITS-1 ); \
      uiCw |= uiCurrBin >> ( uiNumBins - uiBinsToLoad ); \
      uiBinsToLoad--; \
      continue; \
    } \
    UInt uiCurrBin = (_fct)( ucMps ); \
    uiCw |= uiCurrBin << ( PIPE_CW_NUM_BITS-1 - ( uiNumBins - uiBinsToLoad ) ); \
    uiBinsToLoad--; \
  } \
}
#define DECODE_N_NONTAB( _b, _f,_x ) DECODE_N_NONTAB_MIRROR( _b, _f );
#define ENOUGH_BINS(_cw) ( (_cw) << uiNumBins )
#define UPDATE_CWBUF(_buf,_cw) ( *(_buf) = (_cw) << uiNumBins )
#define NUM_BINS_IN_BUFFER( _dst, _p_src ) (_dst) = auiBinsInBufferTab[*(_p_src)>>(PIPE_CW_NUM_BITS-4)];

UInt
TDecSbac::xDecodeNBin0( const UChar ucMps ,const UInt uiNumBins )
{
  UInt uiCw = m_puiPipe0CwCnt[0];
  if( ENOUGH_BINS( uiCw ) )
  {
    UPDATE_CWBUF( m_puiPipe0CwCnt, uiCw );
    uiCw ^= (UInt)( -ucMps );
    uiCw  = MASK_STOP_BIT(uiCw);
  }
  else
  {
    UInt uiBits, uiBins;
    UInt uiBinsInBuffer;
    NUM_BINS_IN_BUFFER( uiBinsInBuffer, &uiCw );
    UInt uiBinsToLoad = uiNumBins - uiBinsInBuffer;
#if DECODE_NONTAB
    DECODE_N_NONTAB( m_puiPipe0CwCnt, xDecode1Bin0, 0 );
#else
    m_pcBitstream->showAndFlush( uiBinsToLoad > 2 ? 6 : 3, uiBits, (UInt*)auiBP3BitsLenTab[uiBinsToLoad] );
    uiBins = auiBP3Tab[uiBinsToLoad][uiBits];
    uiBins >>= uiBinsInBuffer;
    uiCw = ( uiCw ^ ( PIPE_INIT_CW >> uiBinsInBuffer ) ) | uiBins;

    *m_puiPipe0CwCnt = uiCw << uiNumBins;
    uiCw ^= (UInt)( -ucMps );
    uiCw  = MASK_STOP_BIT(uiCw);
#endif
  }
  return uiCw;
}

UInt
TDecSbac::xDecodeNBin1( const UChar ucMps, const UInt uiNumBins )
{
  UInt uiCw = *m_puiPipe1CwCnt;
  if( ENOUGH_BINS( uiCw ) )
  {
    UPDATE_CWBUF( m_puiPipe1CwCnt, uiCw );
    uiCw ^= (UInt)( -ucMps );
    uiCw  = MASK_STOP_BIT(uiCw);
  }
  else
  {
    UInt uiBits, uiBins, uiBinsInBuffer;
    NUM_BINS_IN_BUFFER( uiBinsInBuffer, &uiCw );
    UInt uiBinsToLoad = uiNumBins - uiBinsInBuffer;
#if DECODE_NONTAB
    DECODE_N_NONTAB( m_puiPipe1CwCnt, xDecode1Bin1, 0 );
#else
    m_pcBitstream->showAndFlush( uiBinsToLoad * 2, uiBits, (UInt*)auiBP2BitsLenTab[uiBinsToLoad] );
    uiBins = auiBP2Tab[uiBinsToLoad][uiBits];
    uiBins >>= uiBinsInBuffer;
    uiCw = ( uiCw ^ ( PIPE_INIT_CW >> uiBinsInBuffer ) ) | uiBins;
    *m_puiPipe1CwCnt = uiCw << uiNumBins;
    uiCw ^= (UInt)( -ucMps );
    uiCw  = MASK_STOP_BIT(uiCw);
#endif
  }
  return uiCw;
}

UInt
TDecSbac::xDecodeNBin2( const UChar ucMps, const UInt uiNumBins )
{
  UInt uiCw = *m_puiPipe2CwCnt;
  if( ENOUGH_BINS( uiCw ) )
  {
    UPDATE_CWBUF( m_puiPipe2CwCnt, uiCw );
    uiCw ^= (UInt)( -ucMps );
    uiCw  = MASK_STOP_BIT(uiCw);
  }
  else
  {
    UInt uiBits, uiBins;
    UInt uiBinsInBuffer;
    NUM_BINS_IN_BUFFER( uiBinsInBuffer, &uiCw );
#if DECODE_NONTAB
    UInt uiBinsToLoad = uiNumBins - uiBinsInBuffer;
    DECODE_N_NONTAB( m_puiPipe2CwCnt, xDecode1Bin2, 0 );
#else
    m_pcBitstream->showAndFlush( 5, uiBits, (UInt*)auiTBCBitsLenTab );
    uiBins = auiTBCBinTab[uiBits];
    uiBins >>= uiBinsInBuffer;
    uiCw = ( uiCw ^ ( PIPE_INIT_CW >> uiBinsInBuffer ) ) | uiBins;
    if( !ENOUGH_BINS( uiCw ) )
    {
      m_pcBitstream->showAndFlush( 5, uiBits, (UInt*)auiTBCBitsLenTab );
      uiBinsInBuffer = 3; // NOTE: short-cut, it works only for uiNumBins <= 4
      uiBins = auiTBCBinTab[uiBits];
      uiBins >>= uiBinsInBuffer;
      uiCw = ( uiCw ^ ( PIPE_INIT_CW >> uiBinsInBuffer ) ) | uiBins;
    }
    *m_puiPipe2CwCnt = uiCw << uiNumBins;
    uiCw ^= (UInt)( -ucMps );
    uiCw  = MASK_STOP_BIT(uiCw);
#endif
  }
  return uiCw;
}

UInt
TDecSbac::xDecodeNBin3( const UChar ucMps, const UInt uiNumBins )
{
  UInt uiBins;
  m_pcBitstream->read( uiNumBins, uiBins );
  uiBins <<= ( PIPE_CW_NUM_BITS - uiNumBins );
  return uiBins;
}

UInt
TDecSbac::xDecodeNBin4( const UChar ucMps, const UInt uiNumBins )
{
  UInt uiBin;
  UInt uiCw = *m_puiPipe4CwCnt;
  if( ENOUGH_BINS( uiCw ) )
  {
    UPDATE_CWBUF( m_puiPipe4CwCnt, uiCw );
    uiCw ^= (UInt)( -ucMps );
    uiBin = MASK_STOP_BIT(uiCw);
  }
  else
  {
    UInt uiBit;
    UInt uiBinsInBuffer;
    const UInt uiIdLength = 2;

    NUM_BINS_IN_BUFFER( uiBinsInBuffer, &uiCw );
#if DECODE_NONTAB
    UInt uiBinsToLoad = uiNumBins - uiBinsInBuffer;
    DECODE_N_NONTAB( m_puiPipe4CwCnt, xDecode1Bin4, 0 );
    uiBin = uiCw;
#else
    while( uiBinsInBuffer < uiNumBins )
    {
      m_pcBitstream->read( 1, uiBit );
      if( uiBit )
      {
        uiCw ^= (UInt)PIPE_INIT_CW >> uiBinsInBuffer;
        uiCw |= (UInt)PIPE_INIT_CW >> ( ( 1 << uiIdLength ) + uiBinsInBuffer );
        break;
      }
      else
      {
        m_pcBitstream->read( uiIdLength, uiBit );
        uiCw ^= (UInt)PIPE_INIT_CW >> uiBinsInBuffer;
        uiCw |= ( PIPE_2BIT_CW >> ( uiBit + uiBinsInBuffer ) );
        uiBinsInBuffer += ( uiBit + 1 );
      }
    }
    *m_puiPipe4CwCnt = ( uiCw << uiNumBins );
    uiBin = (uiCw) ^ (UInt)( -ucMps );
    uiBin = MASK_STOP_BIT(uiBin);
#endif
  }
  return uiBin;
}

UInt
TDecSbac::xDecodeNBin5( const UChar ucMps, const UInt uiNumBins )
{
  UInt uiBin;
  UInt uiCw = *m_puiPipe5CwCnt;
  if( ENOUGH_BINS( uiCw ) )
  {
    UPDATE_CWBUF( m_puiPipe5CwCnt, uiCw );
    uiCw ^= (UInt)( -ucMps );
    uiBin = MASK_STOP_BIT(uiCw);
  }
  else
  {
    UInt uiBit;
    UInt uiBinsInBuffer;
    const UInt uiIdLength = 3;

    NUM_BINS_IN_BUFFER( uiBinsInBuffer, &uiCw );

#if DECODE_NONTAB
    UInt uiBinsToLoad = uiNumBins - uiBinsInBuffer;
    DECODE_N_NONTAB( m_puiPipe5CwCnt, xDecode1Bin5, 0 );
    uiBin = uiCw;
#else
    while( uiBinsInBuffer < uiNumBins )
    {
      m_pcBitstream->read( 1, uiBit );
      if( uiBit )
      {
        uiCw ^= (UInt)PIPE_INIT_CW >> uiBinsInBuffer;
        uiCw |= (UInt)PIPE_INIT_CW >> ( ( 1 << uiIdLength ) + uiBinsInBuffer );
        break;
      }
      else
      {
        m_pcBitstream->read( uiIdLength, uiBit );
        uiCw ^= (UInt)PIPE_INIT_CW >> uiBinsInBuffer;
        uiCw |= ( PIPE_2BIT_CW >> ( uiBit + uiBinsInBuffer ) );
        uiBinsInBuffer += ( uiBit + 1 );
      }
    }
    *m_puiPipe5CwCnt = ( uiCw << uiNumBins );
    uiBin = (uiCw) ^ (UInt)( -ucMps );
    uiBin = MASK_STOP_BIT(uiBin);
#endif
  }
  return uiBin;
}

UInt
TDecSbac::xDecodeNBin6( const UChar ucMps, const UInt uiNumBins )
{
  UInt uiBin;
  UInt uiCw = *m_puiPipe6CwCnt;
  if( ENOUGH_BINS( uiCw ) )
  {
    UPDATE_CWBUF( m_puiPipe6CwCnt, uiCw );
    uiCw ^= (UInt)( -ucMps );
    uiBin = MASK_STOP_BIT(uiCw);
  }
  else
  {
    UInt uiBit;
    UInt uiBinsInBuffer;
    const UInt uiIdLength = 4;

    NUM_BINS_IN_BUFFER( uiBinsInBuffer, &uiCw );

#if DECODE_NONTAB
    UInt uiBinsToLoad = uiNumBins - uiBinsInBuffer;
    DECODE_N_NONTAB( m_puiPipe6CwCnt, xDecode1Bin6, 0 );
    uiBin = uiCw;
#else
    while( uiBinsInBuffer < uiNumBins )
    {
      m_pcBitstream->read( 1, uiBit );
      if( uiBit )
      {
        uiCw ^= (UInt)PIPE_INIT_CW >> uiBinsInBuffer;
        uiCw |= (UInt)PIPE_INIT_CW >> ( ( 1 << uiIdLength ) + uiBinsInBuffer );
        break;
      }
      else
      {
        m_pcBitstream->read( uiIdLength, uiBit );
        uiCw ^= (UInt)PIPE_INIT_CW >> uiBinsInBuffer;
        uiCw |= ( PIPE_2BIT_CW >> ( uiBit + uiBinsInBuffer ) );
        uiBinsInBuffer += ( uiBit + 1 );
      }
    }
    *m_puiPipe6CwCnt = ( uiCw << uiNumBins );
    uiBin = (uiCw) ^ (UInt)( -ucMps );
    uiBin = MASK_STOP_BIT(uiBin);
#endif
  }
  return uiBin;
}

UInt
TDecSbac::xDecodeNBin7( const UChar ucMps, const UInt uiNumBins )
{
  UInt uiBin;
  UInt ui32Cw = *m_puiPipe7CwCnt;
  if( ENOUGH_BINS( ui32Cw ) )
  {
    UPDATE_CWBUF( m_puiPipe7CwCnt, ui32Cw );
    ui32Cw ^= (UInt)( -ucMps );
    uiBin   = MASK_STOP_BIT(ui32Cw);
  }
  else
  {
    UInt uiBit;
    UInt uiBinsInBuffer;
    const UInt uiIdLength = 5;

    NUM_BINS_IN_BUFFER( uiBinsInBuffer, &ui32Cw );

#if DECODE_NONTAB
    UInt64 uiCw = (UInt64)ui32Cw;
    UInt uiBinsToLoad = uiNumBins - uiBinsInBuffer;
    DECODE_N_NONTAB( m_puiPipe7CwCnt, xDecode1Bin7, 0 );
    uiBin = (UInt)uiCw;
#else
    UInt64 uiCw = (UInt64)ui32Cw << 32;
    while( uiBinsInBuffer < uiNumBins )
    {
      m_pcBitstream->read( 1, uiBit );
      if( uiBit )
      {
        uiCw ^= PIPE_INIT_CW64 >> uiBinsInBuffer;
        uiCw |= PIPE_INIT_CW64 >> ( ( 1 << uiIdLength ) + uiBinsInBuffer );
        break;
      }
      else
      {
        m_pcBitstream->read( uiIdLength, uiBit );
        uiCw ^= PIPE_INIT_CW64 >> uiBinsInBuffer;
        uiCw |= ( PIPE_2BIT_CW64 >> ( uiBit + uiBinsInBuffer ) );
        uiBinsInBuffer += ( uiBit + 1 );
      }
    }
    *m_puiPipe7CwCnt = (UInt)(( uiCw << uiNumBins ) >> 32 );
    uiBin = (UInt)(uiCw >> 32) ^ (UInt)( -ucMps );
    uiBin = MASK_STOP_BIT(uiBin);
#endif
  }
  return uiBin;
}

UInt
TDecSbac::xDecodeBinTrm()
{
  const UInt uiCw = m_auiCounter[ PIPE_ID_UR5 ];
  UInt uiBin;
  if( uiCw != PIPE_INIT_CW )
  {
    m_auiCounter[PIPE_ID_UR5] <<= 1;
    uiBin = uiCw >> ( PIPE_CW_NUM_BITS-1 );
  }
  else
  {
    uiBin = xDecode1Bin7( 0 );
  }
  return uiBin;
}

#else

UInt
TDecSbac::xDecodeBin( ContextModel &rcCtxModel )
{
  UInt uiBin;
  const UInt  uiPipeId = rcCtxModel.getPipeId();
  const UChar ucMPS = rcCtxModel.getMps();
  const UInt  uiCw  = m_auiCounter[ uiPipeId ];

  if( uiCw != PIPE_INIT_CW )
  {
    m_auiCounter[ uiPipeId ] = uiCw << 1;
    uiBin = ( uiCw >> (PIPE_CW_NUM_BITS-1) );
    rcCtxModel.updateState( uiBin );
    uiBin ^= ucMPS;
  }
  else
  {
    uiBin = decodePipe1Bin[uiPipeId]( ucMPS );
    rcCtxModel.updateState( uiBin ^ ucMPS );
  }

  return uiBin;
}

UInt
TDecSbac::xDecodeBinTrm()
{
  const UInt uiCw = m_auiCounter[ PIPE_ID_UR5 ];
  UInt uiBin;
  if( uiCw != PIPE_INIT_CW )
  {
    m_auiCounter[PIPE_ID_UR5] <<= 1;
    uiBin = uiCw >> ( PIPE_CW_NUM_BITS-1 );
  }
  else
  {
    uiBin = xDecode1Bin7( 0 );
  }
  return uiBin;
}

#endif //#if HHI_SYS_PIPE_LC_MODE
#endif //#if HHI_SYS_PIPE
